@@ -30,6 +30,7 @@ import org.apache.atlas.model.instance.AtlasEntityHeader;
 import org.apache.atlas.model.instance.EntityMutationResponse;
 import org.apache.atlas.model.instance.EntityMutations;
 import org.apache.atlas.utils.AuthenticationUtil;
+import org.apache.atlas.utils.HdfsNameServiceResolver;
 import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.model.instance.AtlasEntity.AtlasEntityWithExtInfo;
 import org.apache.atlas.model.instance.AtlasEntity.AtlasEntitiesWithExtInfo;
@@ -66,6 +67,7 @@ import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileReader;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
@@ -82,22 +84,21 @@ public class HiveMetaStoreBridge {
 
     public static final String CONF_PREFIX                     = "atlas.hook.hive.";
     public static final String HIVE_CLUSTER_NAME               = "atlas.cluster.name";
-    public static final String DEFAULT_CLUSTER_NAME            = "primary";
     public static final String HDFS_PATH_CONVERT_TO_LOWER_CASE = CONF_PREFIX + "hdfs_path.convert_to_lowercase";
+    public static final String DEFAULT_CLUSTER_NAME            = "primary";
     public static final String TEMP_TABLE_PREFIX               = "_temp-";
     public static final String ATLAS_ENDPOINT                  = "atlas.rest.address";
     public static final String SEP                             = ":".intern();
     public static final String HDFS_PATH                       = "hdfs_path";
-    public static final String DB                              = "db";
 
     private static final int    EXIT_CODE_SUCCESS = 0;
     private static final int    EXIT_CODE_FAILED  = 1;
     private static final String DEFAULT_ATLAS_URL = "http://localhost:21000/";
 
-    private final String        clusterName;
-    private final Hive          hiveClient;
-    private final AtlasClientV2 atlasClientV2;
-    private final boolean       convertHdfsPathToLowerCase;
+    private final String                  clusterName;
+    private final Hive                    hiveClient;
+    private final AtlasClientV2           atlasClientV2;
+    private final boolean                 convertHdfsPathToLowerCase;
 
 
     public static void main(String[] args) {
@@ -238,10 +239,6 @@ public class HiveMetaStoreBridge {
         return hiveClient;
     }
 
-    public AtlasClientV2 getAtlasClient() {
-        return atlasClientV2;
-    }
-
     public boolean isConvertHdfsPathToLowerCase() {
         return convertHdfsPathToLowerCase;
     }
@@ -255,11 +252,25 @@ public class HiveMetaStoreBridge {
     }
 
     private void importDatabases(boolean failOnError, String databaseToImport, String tableToImport) throws Exception {
-        final List<String> databaseNames;
+        List<String> databaseNames = null;
 
-        if (StringUtils.isEmpty(databaseToImport)) {
+        if (StringUtils.isEmpty(databaseToImport) && StringUtils.isEmpty(tableToImport)) {
+            //when both database and table to import are empty, import all
             databaseNames = hiveClient.getAllDatabases();
+        } else if (StringUtils.isEmpty(databaseToImport) && StringUtils.isNotEmpty(tableToImport)) {
+            //when database is empty and table is not, then check table has database name in it and import that db and table
+            if (isTableWithDatabaseName(tableToImport)) {
+                String val[] = tableToImport.split("\\.");
+                if (val.length > 1) {
+                    databaseToImport = val[0];
+                    tableToImport = val[1];
+                }
+                databaseNames = hiveClient.getDatabasesByPattern(databaseToImport);
+            } else {
+                databaseNames = hiveClient.getAllDatabases();
+            }
         } else {
+            //when database to import has some value then, import that db and all table under it.
             databaseNames = hiveClient.getDatabasesByPattern(databaseToImport);
         }
 
@@ -409,16 +420,12 @@ public class HiveMetaStoreBridge {
             if (tableEntity == null) {
                 tableEntity = toTableEntity(dbEntity, table);
 
-                tableEntity.addReferredEntity(dbEntity);
-
                 ret = registerInstance(tableEntity);
             } else {
                 LOG.info("Table {}.{} is already registered with id {}. Updating entity.", table.getDbName(), table.getTableName(), tableEntity.getEntity().getGuid());
 
                 ret = toTableEntity(dbEntity, table, tableEntity);
 
-                ret.addReferredEntity(dbEntity);
-
                 updateInstance(ret);
             }
 
@@ -465,6 +472,8 @@ public class HiveMetaStoreBridge {
             }
         }
 
+        clearRelationshipAttributes(ret);
+
         return ret;
     }
 
@@ -501,6 +510,8 @@ public class HiveMetaStoreBridge {
             }
         }
 
+        clearRelationshipAttributes(ret);
+
         return ret;
     }
 
@@ -537,11 +548,11 @@ public class HiveMetaStoreBridge {
         dbEntity.setAttribute(ATTRIBUTE_OWNER, hiveDB.getOwnerName());
 
         dbEntity.setAttribute(ATTRIBUTE_CLUSTER_NAME, clusterName);
-        dbEntity.setAttribute(ATTRIBUTE_LOCATION, hiveDB.getLocationUri());
+        dbEntity.setAttribute(ATTRIBUTE_LOCATION, HdfsNameServiceResolver.getPathWithNameServiceID(hiveDB.getLocationUri()));
         dbEntity.setAttribute(ATTRIBUTE_PARAMETERS, hiveDB.getParameters());
 
         if (hiveDB.getOwnerType() != null) {
-            dbEntity.setAttribute(ATTRIBUTE_OWNER_TYPE,OWNER_TYPE_TO_ENUM_VALUE.get(hiveDB.getOwnerType().getValue()));
+            dbEntity.setAttribute(ATTRIBUTE_OWNER_TYPE, OWNER_TYPE_TO_ENUM_VALUE.get(hiveDB.getOwnerType().getValue()));
         }
 
         return dbEntity;
@@ -596,10 +607,6 @@ public class HiveMetaStoreBridge {
         tableEntity.setAttribute(ATTRIBUTE_PARTITION_KEYS, BaseHiveEvent.getObjectIds(partKeys));
         tableEntity.setAttribute(ATTRIBUTE_COLUMNS, BaseHiveEvent.getObjectIds(columns));
 
-        if (MapUtils.isNotEmpty(table.getReferredEntities())) {
-            table.getReferredEntities().clear();
-        }
-
         table.addReferredEntity(database);
         table.addReferredEntity(sdEntity);
 
@@ -615,6 +622,8 @@ public class HiveMetaStoreBridge {
             }
         }
 
+        table.setEntity(tableEntity);
+
         return table;
     }
 
@@ -624,7 +633,7 @@ public class HiveMetaStoreBridge {
         ret.setAttribute(ATTRIBUTE_TABLE, tableId);
         ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, sdQualifiedName);
         ret.setAttribute(ATTRIBUTE_PARAMETERS, storageDesc.getParameters());
-        ret.setAttribute(ATTRIBUTE_LOCATION, storageDesc.getLocation());
+        ret.setAttribute(ATTRIBUTE_LOCATION, HdfsNameServiceResolver.getPathWithNameServiceID(storageDesc.getLocation()));
         ret.setAttribute(ATTRIBUTE_INPUT_FORMAT, storageDesc.getInputFormat());
         ret.setAttribute(ATTRIBUTE_OUTPUT_FORMAT, storageDesc.getOutputFormat());
         ret.setAttribute(ATTRIBUTE_COMPRESSED, storageDesc.isCompressed());
@@ -692,17 +701,28 @@ public class HiveMetaStoreBridge {
 
     private AtlasEntity toHdfsPathEntity(String pathUri) {
         AtlasEntity ret           = new AtlasEntity(HDFS_PATH);
+        String      nameServiceID = HdfsNameServiceResolver.getNameServiceIDForPath(pathUri);
         Path        path          = new Path(pathUri);
 
         ret.setAttribute(ATTRIBUTE_NAME, Path.getPathWithoutSchemeAndAuthority(path).toString());
         ret.setAttribute(ATTRIBUTE_CLUSTER_NAME, clusterName);
-        ret.setAttribute(ATTRIBUTE_PATH, pathUri);
 
-        // Only append clusterName for the HDFS path
-        if (pathUri.startsWith("hdfs://")) {
-            ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getHdfsPathQualifiedName(pathUri));
+        if (StringUtils.isNotEmpty(nameServiceID)) {
+            // Name service resolution is successful, now get updated HDFS path where the host port info is replaced by resolved name service
+            String updatedHdfsPath = HdfsNameServiceResolver.getPathWithNameServiceID(pathUri);
+
+            ret.setAttribute(ATTRIBUTE_PATH, updatedHdfsPath);
+            ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getHdfsPathQualifiedName(updatedHdfsPath));
+            ret.setAttribute(ATTRIBUTE_NAMESERVICE_ID, nameServiceID);
         } else {
-            ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, pathUri);
+            ret.setAttribute(ATTRIBUTE_PATH, pathUri);
+
+            // Only append clusterName for the HDFS path
+            if (pathUri.startsWith(HdfsNameServiceResolver.HDFS_SCHEME)) {
+                ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getHdfsPathQualifiedName(pathUri));
+            } else {
+                ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, pathUri);
+            }
         }
 
         return ret;
@@ -754,10 +774,10 @@ public class HiveMetaStoreBridge {
     }
 
     private AtlasEntityWithExtInfo findEntity(final String typeName, final String qualifiedName) throws AtlasServiceException {
-        AtlasClientV2 atlasClientV2 = getAtlasClient();
+        AtlasEntityWithExtInfo ret = null;
 
         try {
-            return atlasClientV2.getEntityByAttribute(typeName, Collections.singletonMap(ATTRIBUTE_QUALIFIED_NAME, qualifiedName));
+            ret = atlasClientV2.getEntityByAttribute(typeName, Collections.singletonMap(ATTRIBUTE_QUALIFIED_NAME, qualifiedName));
         } catch (AtlasServiceException e) {
             if(e.getStatus() == ClientResponse.Status.NOT_FOUND) {
                 return null;
@@ -765,6 +785,10 @@ public class HiveMetaStoreBridge {
 
             throw e;
         }
+
+        clearRelationshipAttributes(ret);
+
+        return ret;
     }
 
     private String getCreateTableString(Table table, String location){
@@ -789,7 +813,7 @@ public class HiveMetaStoreBridge {
 
     private String lower(String str) {
         if (StringUtils.isEmpty(str)) {
-            return null;
+            return "";
         }
 
         return str.toLowerCase().trim();
@@ -860,12 +884,10 @@ public class HiveMetaStoreBridge {
     public static String getTableQualifiedName(String clusterName, String dbName, String tableName) {
         return getTableQualifiedName(clusterName, dbName, tableName, false);
     }
-
     public static String getStorageDescQFName(String tableQualifiedName) {
         return tableQualifiedName + "_storage";
     }
 
-
     public static String getColumnQualifiedName(final String tableQualifiedName, final String colName) {
         final String[] parts       = tableQualifiedName.split("@");
         final String   tableName   = parts[0];
@@ -877,4 +899,50 @@ public class HiveMetaStoreBridge {
     public static long getTableCreatedTime(Table table) {
         return table.getTTable().getCreateTime() * MILLIS_CONVERT_FACTOR;
     }
+
+    private void clearRelationshipAttributes(AtlasEntitiesWithExtInfo entities) {
+        if (entities != null) {
+            if (entities.getEntities() != null) {
+                for (AtlasEntity entity : entities.getEntities()) {
+                    clearRelationshipAttributes(entity);;
+                }
+            }
+
+            if (entities.getReferredEntities() != null) {
+                clearRelationshipAttributes(entities.getReferredEntities().values());
+            }
+        }
+    }
+
+    private void clearRelationshipAttributes(AtlasEntityWithExtInfo entity) {
+        if (entity != null) {
+            clearRelationshipAttributes(entity.getEntity());
+
+            if (entity.getReferredEntities() != null) {
+                clearRelationshipAttributes(entity.getReferredEntities().values());
+            }
+        }
+    }
+
+    private void clearRelationshipAttributes(Collection<AtlasEntity> entities) {
+        if (entities != null) {
+            for (AtlasEntity entity : entities) {
+                clearRelationshipAttributes(entity);
+            }
+        }
+    }
+
+    private void clearRelationshipAttributes(AtlasEntity entity) {
+        if (entity != null && entity.getRelationshipAttributes() != null) {
+            entity.getRelationshipAttributes().clear();
+        }
+    }
+
+    private boolean isTableWithDatabaseName(String tableName) {
+        boolean ret = false;
+        if (tableName.contains(".")) {
+            ret = true;
+        }
+        return ret;
+    }
 }
