@@ -26,8 +26,10 @@ import org.apache.atlas.model.instance.AtlasEntity.AtlasEntityWithExtInfo;
 import org.apache.atlas.model.instance.AtlasEntity.AtlasEntityExtInfo;
 import org.apache.atlas.model.instance.AtlasObjectId;
 import org.apache.atlas.model.instance.AtlasStruct;
-import org.apache.atlas.notification.hook.HookNotification.HookNotificationMessage;
+import org.apache.atlas.model.notification.HookNotification;
+import org.apache.atlas.utils.HdfsNameServiceResolver;
 import org.apache.commons.collections.CollectionUtils;
+import org.apache.commons.collections.MapUtils;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hive.metastore.api.Database;
@@ -42,7 +44,6 @@ import org.apache.hadoop.hive.ql.hooks.LineageInfo.DependencyKey;
 import org.apache.hadoop.hive.ql.hooks.WriteEntity;
 import org.apache.hadoop.hive.ql.metadata.Hive;
 import org.apache.hadoop.hive.ql.metadata.Table;
-import org.apache.hadoop.hive.ql.parse.SemanticAnalyzer;
 import org.apache.hadoop.hive.ql.plan.HiveOperation;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.slf4j.Logger;
@@ -76,6 +77,15 @@ public abstract class BaseHiveEvent {
     public static final String HIVE_TYPE_SERDE          = "hive_serde";
     public static final String HIVE_TYPE_ORDER          = "hive_order";
     public static final String HDFS_TYPE_PATH           = "hdfs_path";
+    public static final String HBASE_TYPE_TABLE         = "hbase_table";
+    public static final String HBASE_TYPE_NAMESPACE     = "hbase_namespace";
+    public static final String AWS_S3_BUCKET            = "aws_s3_bucket";
+    public static final String AWS_S3_PSEUDO_DIR        = "aws_s3_pseudo_dir";
+    public static final String AWS_S3_OBJECT            = "aws_s3_object";
+
+    public static final String SCHEME_SEPARATOR         = "://";
+    public static final String S3_SCHEME                = "s3" + SCHEME_SEPARATOR;
+    public static final String S3A_SCHEME               = "s3a" + SCHEME_SEPARATOR;
 
     public static final String ATTRIBUTE_QUALIFIED_NAME            = "qualifiedName";
     public static final String ATTRIBUTE_NAME                      = "name";
@@ -125,10 +135,18 @@ public abstract class BaseHiveEvent {
     public static final String ATTRIBUTE_DEPENDENCY_TYPE           = "depenendencyType";
     public static final String ATTRIBUTE_EXPRESSION                = "expression";
     public static final String ATTRIBUTE_ALIASES                   = "aliases";
+    public static final String ATTRIBUTE_URI                       = "uri";
+    public static final String ATTRIBUTE_STORAGE_HANDLER           = "storage_handler";
+    public static final String ATTRIBUTE_NAMESPACE                 = "namespace";
+    public static final String ATTRIBUTE_OBJECT_PREFIX             = "objectPrefix";
+    public static final String ATTRIBUTE_BUCKET                    = "bucket";
 
-    public static final String HDFS_PATH_PREFIX = "hdfs://";
-
-    public static final long   MILLIS_CONVERT_FACTOR  = 1000;
+    public static final String HBASE_STORAGE_HANDLER_CLASS         = "org.apache.hadoop.hive.hbase.HBaseStorageHandler";
+    public static final String HBASE_DEFAULT_NAMESPACE             = "default";
+    public static final String HBASE_NAMESPACE_TABLE_DELIMITER     = ":";
+    public static final String HBASE_PARAM_TABLE_NAME              = "hbase.table.name";
+    public static final long   MILLIS_CONVERT_FACTOR               = 1000;
+    public static final String HDFS_PATH_PREFIX                    = "hdfs://";
 
     public static final Map<Integer, String> OWNER_TYPE_TO_ENUM_VALUE = new HashMap<>();
 
@@ -150,7 +168,7 @@ public abstract class BaseHiveEvent {
         return context;
     }
 
-    public List<HookNotificationMessage> getNotificationMessages() throws Exception {
+    public List<HookNotification> getNotificationMessages() throws Exception {
         return null;
     }
 
@@ -158,6 +176,10 @@ public abstract class BaseHiveEvent {
         return table.getTTable() != null ? (table.getTTable().getCreateTime() * MILLIS_CONVERT_FACTOR) : System.currentTimeMillis();
     }
 
+    public static String getTableOwner(Table table) {
+        return table.getTTable() != null ? (table.getOwner()): "";
+    }
+
     public static AtlasObjectId getObjectId(AtlasEntity entity) {
         String        qualifiedName = (String) entity.getAttribute(ATTRIBUTE_QUALIFIED_NAME);
         AtlasObjectId ret           = new AtlasObjectId(entity.getGuid(), entity.getTypeName(), Collections.singletonMap(ATTRIBUTE_QUALIFIED_NAME, qualifiedName));
@@ -218,6 +240,7 @@ public abstract class BaseHiveEvent {
             case DATABASE: {
                 if (!context.getIgnoreDummyDatabaseName().contains(entity.getDatabase().getName())) {
                     Database db = getHive().getDatabase(entity.getDatabase().getName());
+
                     ret = toDbEntity(db);
                 }
             }
@@ -225,34 +248,33 @@ public abstract class BaseHiveEvent {
 
             case TABLE:
             case PARTITION: {
+                String  dbName    = entity.getTable().getDbName();
+                String  tableName = entity.getTable().getTableName();
+                boolean skipTable = StringUtils.isNotEmpty(context.getIgnoreValuesTmpTableNamePrefix()) && tableName.toLowerCase().startsWith(context.getIgnoreValuesTmpTableNamePrefix());
 
-                    String  dbName    = entity.getTable().getDbName();
-                    String  tableName = entity.getTable().getTableName();
-                    boolean skipTable = StringUtils.isNotEmpty(context.getIgnoreValuesTmpTableNamePrefix()) && tableName.toLowerCase().startsWith(context.getIgnoreValuesTmpTableNamePrefix());
-
-                    if (!skipTable) {
-                        skipTable = context.getIgnoreDummyTableName().contains(tableName) && context.getIgnoreDummyDatabaseName().contains(dbName);
-                    }
-
-                    if (!skipTable) {
-                        Table table = getHive().getTable(dbName, tableName);
-
-                        ret = toTableEntity(table, entityExtInfo);
-                    }
+                if (!skipTable) {
+                    skipTable = context.getIgnoreDummyTableName().contains(tableName) && context.getIgnoreDummyDatabaseName().contains(dbName);
                 }
-                break;
+
+                if (!skipTable) {
+                    Table table = getHive().getTable(dbName, tableName);
+
+                    ret = toTableEntity(table, entityExtInfo);
+                }
+            }
+            break;
 
             case DFS_DIR: {
                 URI location = entity.getLocation();
 
                 if (location != null) {
-                    ret = getHDFSPathEntity(new Path(entity.getLocation()));
+                    ret = getPathEntity(new Path(entity.getLocation()), entityExtInfo);
                 }
             }
             break;
 
             default:
-                break;
+            break;
         }
 
         return ret;
@@ -280,7 +302,7 @@ public abstract class BaseHiveEvent {
             ret.setAttribute(ATTRIBUTE_OWNER, db.getOwnerName());
 
             ret.setAttribute(ATTRIBUTE_CLUSTER_NAME, getClusterName());
-            ret.setAttribute(ATTRIBUTE_LOCATION, db.getLocationUri());
+            ret.setAttribute(ATTRIBUTE_LOCATION, HdfsNameServiceResolver.getPathWithNameServiceID(db.getLocationUri()));
             ret.setAttribute(ATTRIBUTE_PARAMETERS, db.getParameters());
 
             if (db.getOwnerType() != null) {
@@ -434,14 +456,14 @@ public abstract class BaseHiveEvent {
             ret.setAttribute(ATTRIBUTE_TABLE, tableId);
             ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, sdQualifiedName);
             ret.setAttribute(ATTRIBUTE_PARAMETERS, sd.getParameters());
-            ret.setAttribute(ATTRIBUTE_LOCATION, sd.getLocation());
+            ret.setAttribute(ATTRIBUTE_LOCATION, HdfsNameServiceResolver.getPathWithNameServiceID(sd.getLocation()));
             ret.setAttribute(ATTRIBUTE_INPUT_FORMAT, sd.getInputFormat());
             ret.setAttribute(ATTRIBUTE_OUTPUT_FORMAT, sd.getOutputFormat());
             ret.setAttribute(ATTRIBUTE_COMPRESSED, sd.isCompressed());
             ret.setAttribute(ATTRIBUTE_NUM_BUCKETS, sd.getNumBuckets());
             ret.setAttribute(ATTRIBUTE_STORED_AS_SUB_DIRECTORIES, sd.isStoredAsSubDirectories());
 
-            if (sd.getBucketCols().size() > 0) {
+            if (sd.getBucketCols() != null && sd.getBucketCols().size() > 0) {
                 ret.setAttribute(ATTRIBUTE_BUCKET_COLS, sd.getBucketCols());
             }
 
@@ -478,62 +500,107 @@ public abstract class BaseHiveEvent {
     }
 
     protected List<AtlasEntity> getColumnEntities(AtlasObjectId tableId, Table table, List<FieldSchema> fieldSchemas) {
-        List<AtlasEntity> ret          = new ArrayList<>();
-        boolean           isKnownTable = tableId.getGuid() == null;
+        List<AtlasEntity> ret            = new ArrayList<>();
+        boolean           isKnownTable   = tableId.getGuid() == null;
+        int               columnPosition = 0;
 
-        int columnPosition = 0;
-        for (FieldSchema fieldSchema : fieldSchemas) {
-            String      colQualifiedName = getQualifiedName(table, fieldSchema);
-            AtlasEntity column           = context.getEntity(colQualifiedName);
+        if (CollectionUtils.isNotEmpty(fieldSchemas)) {
+            for (FieldSchema fieldSchema : fieldSchemas) {
+                String      colQualifiedName = getQualifiedName(table, fieldSchema);
+                AtlasEntity column           = context.getEntity(colQualifiedName);
 
-            if (column == null) {
-                column = new AtlasEntity(HIVE_TYPE_COLUMN);
+                if (column == null) {
+                    column = new AtlasEntity(HIVE_TYPE_COLUMN);
 
-                // if column's table was sent in an earlier notification, set 'guid' to null - which will:
-                //  - result in this entity to be not included in 'referredEntities'
-                //  - cause Atlas server to resolve the entity by its qualifiedName
-                if (isKnownTable) {
-                    column.setGuid(null);
+                    // if column's table was sent in an earlier notification, set 'guid' to null - which will:
+                    //  - result in this entity to be not included in 'referredEntities'
+                    //  - cause Atlas server to resolve the entity by its qualifiedName
+                    if (isKnownTable) {
+                        column.setGuid(null);
+                    }
+
+                    column.setAttribute(ATTRIBUTE_TABLE, tableId);
+                    column.setAttribute(ATTRIBUTE_QUALIFIED_NAME, colQualifiedName);
+                    column.setAttribute(ATTRIBUTE_NAME, fieldSchema.getName());
+                    column.setAttribute(ATTRIBUTE_OWNER, table.getOwner());
+                    column.setAttribute(ATTRIBUTE_COL_TYPE, fieldSchema.getType());
+                    column.setAttribute(ATTRIBUTE_COL_POSITION, columnPosition++);
+                    column.setAttribute(ATTRIBUTE_COMMENT, fieldSchema.getComment());
+
+                    context.putEntity(colQualifiedName, column);
                 }
 
-                column.setAttribute(ATTRIBUTE_TABLE, tableId);
-                column.setAttribute(ATTRIBUTE_QUALIFIED_NAME, colQualifiedName);
-                column.setAttribute(ATTRIBUTE_NAME, fieldSchema.getName());
-                column.setAttribute(ATTRIBUTE_OWNER, table.getOwner());
-                column.setAttribute(ATTRIBUTE_COL_TYPE, fieldSchema.getType());
-                column.setAttribute(ATTRIBUTE_COL_POSITION, columnPosition++);
-                column.setAttribute(ATTRIBUTE_COMMENT, fieldSchema.getComment());
-
-                context.putEntity(colQualifiedName, column);
+                ret.add(column);
             }
-
-            ret.add(column);
         }
 
         return ret;
     }
 
-    protected AtlasEntity getHDFSPathEntity(Path path) {
+    protected AtlasEntity getPathEntity(Path path, AtlasEntityExtInfo extInfo) {
+        AtlasEntity ret;
         String strPath = path.toString();
-        String name    = Path.getPathWithoutSchemeAndAuthority(path).toString();
 
         if (strPath.startsWith(HDFS_PATH_PREFIX) && context.isConvertHdfsPathToLowerCase()) {
             strPath = strPath.toLowerCase();
-            name    = name.toLowerCase();
         }
 
-        String      pathQualifiedName = getQualifiedName(strPath);
-        AtlasEntity ret               = context.getEntity(pathQualifiedName);
+        if (isS3Path(strPath)) {
+            String      bucketName          = path.toUri().getAuthority();
+            String      bucketQualifiedName = (path.toUri().getScheme() + SCHEME_SEPARATOR + path.toUri().getAuthority() + QNAME_SEP_CLUSTER_NAME).toLowerCase() + getClusterName();
+            String      pathQualifiedName   = (strPath + QNAME_SEP_CLUSTER_NAME).toLowerCase() + getClusterName();
+            AtlasEntity bucketEntity        = context.getEntity(bucketQualifiedName);
 
-        if (ret == null) {
-            ret = new AtlasEntity(HDFS_TYPE_PATH);
+            ret = context.getEntity(pathQualifiedName);
 
-            ret.setAttribute(ATTRIBUTE_PATH, strPath);
-            ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, pathQualifiedName);
-            ret.setAttribute(ATTRIBUTE_NAME, name);
-            ret.setAttribute(ATTRIBUTE_CLUSTER_NAME, getClusterName());
+            if (ret == null) {
+                if (bucketEntity == null) {
+                    bucketEntity = new AtlasEntity(AWS_S3_BUCKET);
 
-            context.putEntity(pathQualifiedName, ret);
+                    bucketEntity.setAttribute(ATTRIBUTE_QUALIFIED_NAME, bucketQualifiedName);
+                    bucketEntity.setAttribute(ATTRIBUTE_NAME, bucketName);
+
+                    context.putEntity(bucketQualifiedName, bucketEntity);
+                }
+
+                extInfo.addReferredEntity(bucketEntity);
+
+                ret = new AtlasEntity(AWS_S3_PSEUDO_DIR);
+
+                ret.setAttribute(ATTRIBUTE_BUCKET, getObjectId(bucketEntity));
+                ret.setAttribute(ATTRIBUTE_OBJECT_PREFIX, Path.getPathWithoutSchemeAndAuthority(path).toString().toLowerCase());
+                ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, pathQualifiedName);
+                ret.setAttribute(ATTRIBUTE_NAME, Path.getPathWithoutSchemeAndAuthority(path).toString().toLowerCase());
+
+                context.putEntity(pathQualifiedName, ret);
+            }
+        } else {
+            String nameServiceID     = HdfsNameServiceResolver.getNameServiceIDForPath(strPath);
+            String attrPath          = StringUtils.isEmpty(nameServiceID) ? strPath : HdfsNameServiceResolver.getPathWithNameServiceID(strPath);
+            String pathQualifiedName = getQualifiedName(attrPath);
+
+            ret = context.getEntity(pathQualifiedName);
+
+            if (ret == null) {
+                ret = new AtlasEntity(HDFS_TYPE_PATH);
+
+                if (StringUtils.isNotEmpty(nameServiceID)) {
+                    ret.setAttribute(ATTRIBUTE_NAMESERVICE_ID, nameServiceID);
+                }
+
+                String name = Path.getPathWithoutSchemeAndAuthority(path).toString();
+
+                if (strPath.startsWith(HDFS_PATH_PREFIX) && context.isConvertHdfsPathToLowerCase()) {
+                    name = name.toLowerCase();
+                }
+
+                ret.setAttribute(ATTRIBUTE_PATH, attrPath);
+                ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, pathQualifiedName);
+                ret.setAttribute(ATTRIBUTE_NAME, name);
+                ret.setAttribute(ATTRIBUTE_CLUSTER_NAME, getClusterName());
+
+                context.putEntity(pathQualifiedName, ret);
+            }
         }
 
         return ret;
@@ -671,11 +738,14 @@ public abstract class BaseHiveEvent {
             strPath = strPath.toLowerCase();
         }
 
-        return getQualifiedName(strPath);
+        String nameServiceID = HdfsNameServiceResolver.getNameServiceIDForPath(strPath);
+        String attrPath      = StringUtils.isEmpty(nameServiceID) ? strPath : HdfsNameServiceResolver.getPathWithNameServiceID(strPath);
+
+        return getQualifiedName(attrPath);
     }
 
     protected String getQualifiedName(String path) {
-        if (path.startsWith(HDFS_PATH_PREFIX)) {
+        if (path.startsWith(HdfsNameServiceResolver.HDFS_SCHEME)) {
             return path + QNAME_SEP_CLUSTER_NAME + getClusterName();
         }
 
@@ -729,6 +799,53 @@ public abstract class BaseHiveEvent {
         return sb.toString();
     }
 
+    protected AtlasEntity toReferencedHBaseTable(Table table, AtlasEntitiesWithExtInfo entities) {
+        AtlasEntity    ret            = null;
+        HBaseTableInfo hBaseTableInfo = new HBaseTableInfo(table);
+        String         hbaseNameSpace = hBaseTableInfo.getHbaseNameSpace();
+        String         hbaseTableName = hBaseTableInfo.getHbaseTableName();
+
+        if (hbaseTableName != null) {
+            AtlasEntity nsEntity = new AtlasEntity(HBASE_TYPE_NAMESPACE);
+            nsEntity.setAttribute(ATTRIBUTE_NAME, hbaseNameSpace);
+            nsEntity.setAttribute(ATTRIBUTE_CLUSTER_NAME, getClusterName());
+            nsEntity.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getHBaseNameSpaceQualifiedName(getClusterName(), hbaseNameSpace));
+
+            ret = new AtlasEntity(HBASE_TYPE_TABLE);
+
+            ret.setAttribute(ATTRIBUTE_NAME, hbaseTableName);
+            ret.setAttribute(ATTRIBUTE_URI, hbaseTableName);
+            ret.setAttribute(ATTRIBUTE_NAMESPACE, getObjectId(nsEntity));
+            ret.setAttribute(ATTRIBUTE_QUALIFIED_NAME, getHBaseTableQualifiedName(getClusterName(), hbaseNameSpace, hbaseTableName));
+
+            entities.addReferredEntity(nsEntity);
+            entities.addEntity(ret);
+        }
+
+        return ret;
+    }
+
+    protected boolean isHBaseStore(Table table) {
+        boolean             ret        = false;
+        Map<String, String> parameters = table.getParameters();
+
+        if (MapUtils.isNotEmpty(parameters)) {
+            String storageHandler = parameters.get(ATTRIBUTE_STORAGE_HANDLER);
+
+            ret = (storageHandler != null && storageHandler.equals(HBASE_STORAGE_HANDLER_CLASS));
+        }
+
+        return ret;
+    }
+
+    private static String getHBaseTableQualifiedName(String clusterName, String nameSpace, String tableName) {
+        return String.format("%s:%s@%s", nameSpace.toLowerCase(), tableName.toLowerCase(), clusterName);
+    }
+
+    private static String getHBaseNameSpaceQualifiedName(String clusterName, String nameSpace) {
+        return String.format("%s@%s", nameSpace.toLowerCase(), clusterName);
+    }
+
     private boolean ignoreHDFSPathsinProcessQualifiedName() {
         switch (context.getHiveOperation()) {
             case LOAD:
@@ -799,17 +916,17 @@ public abstract class BaseHiveEvent {
                 if (writeEntity.getWriteType() != null && HiveOperation.QUERY.equals(context.getHiveOperation())) {
                     boolean addWriteType = false;
 
-                    switch (writeEntity.getWriteType()) {
+                    switch (((WriteEntity) entity).getWriteType()) {
                         case INSERT:
                         case INSERT_OVERWRITE:
                         case UPDATE:
                         case DELETE:
                             addWriteType = true;
-                            break;
+                        break;
 
                         case PATH_WRITE:
                             addWriteType = !Entity.Type.LOCAL_DIR.equals(entity.getType());
-                            break;
+                        break;
                     }
 
                     if (addWriteType) {
@@ -826,7 +943,7 @@ public abstract class BaseHiveEvent {
         }
     }
 
-    public boolean isAlterTableOperation() {
+    private boolean isAlterTableOperation() {
         switch (context.getHiveOperation()) {
             case ALTERTABLE_FILEFORMAT:
             case ALTERTABLE_CLUSTER_SORT:
@@ -849,6 +966,10 @@ public abstract class BaseHiveEvent {
         return false;
     }
 
+    private boolean isS3Path(String strPath) {
+        return strPath != null && (strPath.startsWith(S3_SCHEME) || strPath.startsWith(S3A_SCHEME));
+    }
+
 
     static final class EntityComparator implements Comparator<Entity> {
         @Override
@@ -866,4 +987,37 @@ public abstract class BaseHiveEvent {
     }
 
     static final Comparator<Entity> entityComparator = new EntityComparator();
+
+    static final class HBaseTableInfo {
+        String hbaseNameSpace = null;
+        String hbaseTableName = null;
+
+         HBaseTableInfo(Table table) {
+            Map<String, String> parameters = table.getParameters();
+
+            if (MapUtils.isNotEmpty(parameters)) {
+                hbaseNameSpace = HBASE_DEFAULT_NAMESPACE;
+                hbaseTableName = parameters.get(HBASE_PARAM_TABLE_NAME);
+
+                if (hbaseTableName != null) {
+                    if (hbaseTableName.contains(HBASE_NAMESPACE_TABLE_DELIMITER)) {
+                        String[] hbaseTableInfo = hbaseTableName.split(HBASE_NAMESPACE_TABLE_DELIMITER);
+
+                        if (hbaseTableInfo.length > 1) {
+                            hbaseNameSpace = hbaseTableInfo[0];
+                            hbaseTableName = hbaseTableInfo[1];
+                        }
+                    }
+                }
+            }
+        }
+
+        public String getHbaseNameSpace() {
+            return hbaseNameSpace;
+        }
+
+        public String getHbaseTableName() {
+            return hbaseTableName;
+        }
+    }
 }
