@@ -29,7 +29,7 @@ import org.apache.atlas.hive.model.HiveDataTypes;
 import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.model.instance.AtlasObjectId;
 import org.apache.atlas.model.instance.AtlasStruct;
-import org.apache.atlas.notification.hook.HookNotification.HookNotificationMessage;
+import org.apache.atlas.model.notification.HookNotification;
 import org.apache.atlas.utils.AuthenticationUtil;
 import org.apache.atlas.utils.ParamChecker;
 import org.apache.commons.configuration.Configuration;
@@ -52,7 +52,6 @@ import org.apache.hadoop.security.UserGroupInformation;
 import org.json.JSONObject;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-import org.testng.Assert;
 import org.testng.annotations.BeforeClass;
 
 import java.io.File;
@@ -60,7 +59,7 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
-import java.util.HashMap;
+import java.util.HashSet;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
@@ -68,8 +67,10 @@ import java.util.Set;
 import java.util.SortedMap;
 import java.util.SortedSet;
 
+import static org.apache.atlas.hive.bridge.HiveMetaStoreBridge.HDFS_PATH;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertNotNull;
+import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.fail;
 
 public class HiveITBase {
@@ -148,6 +149,7 @@ public class HiveITBase {
 
     protected void runCommandWithDelay(Driver driver, String cmd, int sleepMs) throws Exception {
         LOG.debug("Running command '{}'", cmd);
+        ss.setCommandType(null);
         CommandProcessorResponse response = driver.run(cmd);
         assertEquals(response.getResponseCode(), 0);
         if (sleepMs != 0) {
@@ -218,7 +220,7 @@ public class HiveITBase {
     }
 
     protected AtlasEntity assertEntityIsRegistedViaEntity(final String typeName, final String property, final String value,
-                                                          final HiveHookIT.AssertPredicate assertPredicate) throws Exception {
+                                              final HiveHookIT.AssertPredicate assertPredicate) throws Exception {
         waitFor(80000, new HiveHookIT.Predicate() {
             @Override
             public void evaluate() throws Exception {
@@ -286,18 +288,28 @@ public class HiveITBase {
             String testPathNormed = lower(path.toString());
             String hdfsPathId     = assertHDFSPathIsRegistered(testPathNormed);
 
-            Assert.assertEquals(hdfsPathIds.get(0).getGuid(), hdfsPathId);
+            assertHDFSPathIdsContain(hdfsPathIds, hdfsPathId);
         }
     }
 
+    private void assertHDFSPathIdsContain(List<AtlasObjectId> hdfsPathObjectIds, String hdfsPathId) {
+        Set<String> hdfsPathGuids = new HashSet<>();
+
+        for (AtlasObjectId hdfsPathObjectId : hdfsPathObjectIds) {
+            hdfsPathGuids.add(hdfsPathObjectId.getGuid());
+        }
+
+        assertTrue(hdfsPathGuids.contains(hdfsPathId));
+    }
+
     protected String assertHDFSPathIsRegistered(String path) throws Exception {
         LOG.debug("Searching for hdfs path {}", path);
         // ATLAS-2444 HDFS name node federation adds the cluster name to the qualifiedName
         if (path.startsWith("hdfs://")) {
             String pathWithCluster = path + "@" + CLUSTER_NAME;
-            return assertEntityIsRegistered(HiveMetaStoreBridge.HDFS_PATH, REFERENCEABLE_ATTRIBUTE_NAME, pathWithCluster, null);
+            return assertEntityIsRegistered(HDFS_PATH, REFERENCEABLE_ATTRIBUTE_NAME, pathWithCluster, null);
         } else {
-            return assertEntityIsRegistered(HiveMetaStoreBridge.HDFS_PATH, REFERENCEABLE_ATTRIBUTE_NAME, path, null);
+            return assertEntityIsRegistered(HDFS_PATH, REFERENCEABLE_ATTRIBUTE_NAME, path, null);
         }
     }
 
@@ -337,7 +349,7 @@ public class HiveITBase {
 
         public Map<String, List<ColumnLineageUtils.HiveColumnLineageInfo>> lineageInfo;
 
-        private List<HookNotificationMessage> messages = new ArrayList<>();
+        private List<HookNotification> messages = new ArrayList<>();
 
         public void setInputs(Set<ReadEntity> inputs) {
             this.inputs = inputs;
@@ -420,11 +432,11 @@ public class HiveITBase {
             return queryStartTime;
         }
 
-        public void addMessage(HookNotificationMessage message) {
+        public void addMessage(HookNotification message) {
             messages.add(message);
         }
 
-        public List<HookNotificationMessage> getMessages() {
+        public List<HookNotification> getMessages() {
             return messages;
         }
     }
@@ -432,10 +444,10 @@ public class HiveITBase {
 
     @VisibleForTesting
     protected static String getProcessQualifiedName(HiveMetaStoreBridge dgiBridge, HiveEventContext eventContext,
-                                                    final SortedSet<ReadEntity> sortedHiveInputs,
-                                                    final SortedSet<WriteEntity> sortedHiveOutputs,
-                                                    SortedMap<ReadEntity, AtlasEntity> hiveInputsMap,
-                                                    SortedMap<WriteEntity, AtlasEntity> hiveOutputsMap) throws HiveException {
+                                          final SortedSet<ReadEntity> sortedHiveInputs,
+                                          final SortedSet<WriteEntity> sortedHiveOutputs,
+                                          SortedMap<ReadEntity, AtlasEntity> hiveInputsMap,
+                                          SortedMap<WriteEntity, AtlasEntity> hiveOutputsMap) throws HiveException {
         HiveOperation op = eventContext.getOperation();
         if (isCreateOp(eventContext)) {
             Entity entity = getEntityByType(sortedHiveOutputs, Entity.Type.TABLE);
@@ -564,7 +576,7 @@ public class HiveITBase {
                                 buffer.append(((WriteEntity) entity).getWriteType().name());
                             }
 
-                            if (output.getType() == Entity.Type.PARTITION || output.getType() == Entity.Type.TABLE) {
+                            if ( output.getType() == Entity.Type.PARTITION || output.getType() == Entity.Type.TABLE) {
                                 Table outputTable = refreshTable(hiveBridge, output.getTable().getDbName(), output.getTable().getTableName());
 
                                 if (outputTable != null) {
@@ -676,18 +688,13 @@ public class HiveITBase {
         return ret;
     }
 
-
     protected AtlasStruct toAtlasStruct(Object obj) {
         final AtlasStruct ret;
 
         if (obj instanceof AtlasStruct) {
             ret = (AtlasStruct) obj;
         } else if (obj instanceof Map) {
-            Map    map        = (Map) obj;
-            Object typeName   = map.get("typeName");
-            Map    attributes = (map.get("attributes") instanceof Map) ? (Map) map.get("attributes") : map;
-
-            ret = new AtlasStruct(typeName == null ? "" : typeName.toString(), attributes);
+            ret = new AtlasStruct((Map) obj);
         } else {
             ret = null;
         }
@@ -723,4 +730,5 @@ public class HiveITBase {
         }
 
         return ret;
-    }}
+    }
+}
