@@ -18,8 +18,9 @@
 
 package org.apache.atlas.hive.hook;
 
+import com.fasterxml.jackson.databind.JsonNode;
+import com.fasterxml.jackson.databind.node.ObjectNode;
 import com.google.common.base.Joiner;
-import com.google.common.collect.ImmutableSet;
 import com.sun.jersey.api.client.ClientResponse;
 import org.apache.atlas.AtlasClient;
 import org.apache.atlas.AtlasServiceException;
@@ -48,8 +49,8 @@ import org.apache.hadoop.hive.ql.hooks.WriteEntity;
 import org.apache.hadoop.hive.ql.metadata.HiveException;
 import org.apache.hadoop.hive.ql.metadata.Table;
 import org.apache.hadoop.hive.ql.plan.HiveOperation;
+import org.apache.hadoop.hive.ql.session.SessionState;
 import org.apache.hadoop.security.UserGroupInformation;
-import org.codehaus.jettison.json.JSONObject;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.testng.Assert;
@@ -515,26 +516,11 @@ public class HiveHookIT extends HiveITBase {
     }
 
     private AtlasEntity validateProcess(HiveEventContext event, Set<ReadEntity> inputTables, Set<WriteEntity> outputTables) throws Exception {
-        String              processId     = assertProcessIsRegistered(event, inputTables, outputTables);
-        AtlasEntity         processEntity = atlasClientV2.getEntityByGuid(processId).getEntity();
-        List<AtlasObjectId> inputs        = toAtlasObjectIdList(processEntity.getAttribute(INPUTS));
-        List<AtlasObjectId> outputs       = toAtlasObjectIdList(processEntity.getAttribute(OUTPUTS));
+        String      processId     = assertProcessIsRegistered(event, inputTables, outputTables);
+        AtlasEntity processEntity = atlasClientV2.getEntityByGuid(processId).getEntity();
 
-        if (inputTables == null) {
-            Assert.assertTrue(CollectionUtils.isEmpty(inputs));
-        } else {
-            Assert.assertEquals(inputs.size(), inputTables.size());
-
-            validateInputTables(processEntity, inputTables);
-        }
-
-        if (outputTables == null) {
-            Assert.assertTrue(CollectionUtils.isEmpty(outputs));
-        } else {
-            Assert.assertEquals(outputs.size(), outputTables.size());
-
-            validateOutputTables(processEntity, outputTables);
-        }
+        validateInputTables(processEntity, inputTables);
+        validateOutputTables(processEntity, outputTables);
 
         return processEntity;
     }
@@ -729,6 +715,7 @@ public class HiveHookIT extends HiveITBase {
         Set<ReadEntity> inputs = getInputs(tableName, Entity.Type.TABLE);
         Set<WriteEntity> outputs = getOutputs(insertTableName, Entity.Type.TABLE);
 
+        outputs.iterator().next().setName(getQualifiedTblName(insertTableName + HiveMetaStoreBridge.TEMP_TABLE_PREFIX + SessionState.get().getSessionId()));
         outputs.iterator().next().setWriteType(WriteEntity.WriteType.INSERT);
 
         validateProcess(constructEvent(query,  HiveOperation.QUERY, inputs, outputs));
@@ -1017,7 +1004,7 @@ public class HiveHookIT extends HiveITBase {
         //add trait
         //valid type names in v2 must consist of a letter followed by a sequence of letter, number, or _ characters
         String                 traitName = "PII_Trait" + random();
-        AtlasClassificationDef piiTrait  =  AtlasTypeUtil.createTraitTypeDef(traitName, ImmutableSet.<String>of());
+        AtlasClassificationDef piiTrait  =  AtlasTypeUtil.createTraitTypeDef(traitName, Collections.<String>emptySet());
 
         atlasClientV2.createAtlasTypeDefs(new AtlasTypesDef(Collections.emptyList(), Collections.emptyList(), Collections.singletonList(piiTrait), Collections.emptyList()));
         atlasClientV2.addClassifications(guid, Collections.singletonList(new AtlasClassification(piiTrait.getName())));
@@ -1270,20 +1257,20 @@ public class HiveHookIT extends HiveITBase {
         AtlasLineageInfo               atlasLineageInfoInput = atlasClientV2.getLineageInfo(dest_a_guid, AtlasLineageInfo.LineageDirection.INPUT,0);
         Map<String, AtlasEntityHeader> entityMap             = atlasLineageInfoInput.getGuidEntityMap();
 
-        JSONObject response   = atlasClient.getInputGraphForEntity(dest_a_guid);
-        JSONObject vertices   = response.getJSONObject("values").getJSONObject("vertices");
-        JSONObject dest_a_val = vertices.getJSONObject(dest_a_guid);
-        JSONObject src_a_val  = vertices.getJSONObject(a_guid);
-        JSONObject src_b_val  = vertices.getJSONObject(b_guid);
+        ObjectNode response   = atlasClient.getInputGraphForEntity(dest_a_guid);
+        JsonNode   vertices   = response.get("values").get("vertices");
+        JsonNode   dest_a_val = vertices.get(dest_a_guid);
+        JsonNode   src_a_val  = vertices.get(a_guid);
+        JsonNode   src_b_val  = vertices.get(b_guid);
 
         Assert.assertNotNull(dest_a_val);
         Assert.assertNotNull(src_a_val);
         Assert.assertNotNull(src_b_val);
 
-        JSONObject b_response  = atlasClient.getInputGraphForEntity(dest_b_guid);
-        JSONObject b_vertices  = b_response.getJSONObject("values").getJSONObject("vertices");
-        JSONObject b_val       = b_vertices.getJSONObject(dest_b_guid);
-        JSONObject src_tbl_val = b_vertices.getJSONObject(sourceTableGUID);
+        ObjectNode b_response  = atlasClient.getInputGraphForEntity(dest_b_guid);
+        JsonNode   b_vertices  = b_response.get("values").get("vertices");
+        JsonNode   b_val       = b_vertices.get(dest_b_guid);
+        JsonNode   src_tbl_val = b_vertices.get(sourceTableGUID);
 
         Assert.assertNotNull(b_val);
         Assert.assertNotNull(src_tbl_val);
@@ -1307,7 +1294,7 @@ public class HiveHookIT extends HiveITBase {
         Map<String, AtlasEntityHeader> entityMap             = atlasLineageInfoInput.getGuidEntityMap();
 
         //Below should be assertTrue - Fix https://issues.apache.org/jira/browse/ATLAS-653
-        Assert.assertTrue(entityMap.containsKey(tableId));
+        Assert.assertFalse(entityMap.containsKey(tableId));
     }
 
     @Test
@@ -1378,7 +1365,7 @@ public class HiveHookIT extends HiveITBase {
 
         assertEquals(guid2, guid);
 
-        assertTrue(atlasClient.getEntity(guid2).getTraits().contains(trait));
+        assertTrue(atlasClient.getEntity(guid2).getTraitNames().contains(trait));
     }
 
     @Test
@@ -1619,7 +1606,19 @@ public class HiveHookIT extends HiveITBase {
 
         runCommand(query);
 
-        assertDBIsNotRegistered(dbName);
+        String dbQualifiedName = HiveMetaStoreBridge.getDBQualifiedName(CLUSTER_NAME, dbName);
+
+        Thread.sleep(5000);
+
+        try {
+            atlasClientV2.getEntityByAttribute(HiveDataTypes.HIVE_DB.getName(), Collections.singletonMap(ATTRIBUTE_QUALIFIED_NAME, dbQualifiedName));
+        } catch (AtlasServiceException e) {
+            if (e.getStatus() == ClientResponse.Status.NOT_FOUND) {
+                return;
+            }
+        }
+
+        fail(String.format("Entity was not supposed to exist for typeName = %s, attributeName = %s, attributeValue = %s", HiveDataTypes.HIVE_DB.getName(), ATTRIBUTE_QUALIFIED_NAME, dbQualifiedName));
     }
 
     @Test
