@@ -17,7 +17,9 @@
  */
 package org.apache.atlas;
 
-import org.codehaus.jackson.jaxrs.JacksonJaxbJsonProvider;
+import com.fasterxml.jackson.databind.JsonNode;
+import com.fasterxml.jackson.databind.node.ObjectNode;
+import com.fasterxml.jackson.jaxrs.json.JacksonJaxbJsonProvider;
 import com.google.common.annotations.VisibleForTesting;
 import com.sun.jersey.api.client.Client;
 import com.sun.jersey.api.client.ClientHandlerException;
@@ -42,13 +44,12 @@ import org.apache.atlas.model.impexp.AtlasImportResult;
 import org.apache.atlas.model.metrics.AtlasMetrics;
 import org.apache.atlas.security.SecureClientUtils;
 import org.apache.atlas.type.AtlasType;
+import org.apache.atlas.utils.AtlasJson;
 import org.apache.atlas.utils.AuthenticationUtil;
 import org.apache.commons.configuration.Configuration;
 import org.apache.commons.io.IOUtils;
 import org.apache.commons.lang.StringUtils;
 import org.apache.hadoop.security.UserGroupInformation;
-import org.codehaus.jettison.json.JSONException;
-import org.codehaus.jettison.json.JSONObject;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -64,6 +65,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.net.ConnectException;
+import java.net.URI;
 import java.nio.file.Paths;
 import java.util.List;
 import java.util.Map;
@@ -83,17 +85,18 @@ public abstract class AtlasBaseClient {
     public static final String QUERY = "query";
     public static final String LIMIT = "limit";
     public static final String OFFSET = "offset";
+    public static final String STATUS = "Status";
 
-    public static final API API_STATUS = new API(BASE_URI + ADMIN_STATUS, HttpMethod.GET, Response.Status.OK);
-    public static final API API_VERSION = new API(BASE_URI + ADMIN_VERSION, HttpMethod.GET, Response.Status.OK);
-    public static final API API_METRICS = new API(BASE_URI + ADMIN_METRICS, HttpMethod.GET, Response.Status.OK);
+    public static final API API_STATUS  = new API(BASE_URI + ADMIN_STATUS, HttpMethod.GET, Response.Status.OK);;
+    public static final API API_VERSION = new API(BASE_URI + ADMIN_VERSION, HttpMethod.GET, Response.Status.OK);;
+    public static final API API_METRICS = new API(BASE_URI + ADMIN_METRICS, HttpMethod.GET, Response.Status.OK);;
 
-    static final String JSON_MEDIA_TYPE = MediaType.APPLICATION_JSON + "; charset=UTF-8";
-    static final String UNKNOWN_STATUS = "Unknown status";
-    static final String ATLAS_CLIENT_HA_RETRIES_KEY = "atlas.client.ha.retries";
+    static final        String JSON_MEDIA_TYPE                       = MediaType.APPLICATION_JSON + "; charset=UTF-8";
+    static final        String UNKNOWN_STATUS                        = "Unknown status";
+    static final        String ATLAS_CLIENT_HA_RETRIES_KEY           = "atlas.client.ha.retries";
     // Setting the default value based on testing failovers while client code like quickstart is running.
-    static final int DEFAULT_NUM_RETRIES = 4;
-    static final String ATLAS_CLIENT_HA_SLEEP_INTERVAL_MS_KEY = "atlas.client.ha.sleep.interval.ms";
+    static final        int    DEFAULT_NUM_RETRIES                   = 4;
+    static final        String ATLAS_CLIENT_HA_SLEEP_INTERVAL_MS_KEY = "atlas.client.ha.sleep.interval.ms";
     // Setting the default value based on testing failovers while client code like quickstart is running.
     // With number of retries, this gives a total time of about 20s for the server to start.
     static final int DEFAULT_SLEEP_BETWEEN_RETRIES_MS = 5000;
@@ -181,7 +184,7 @@ public abstract class AtlasBaseClient {
     public boolean isServerReady() throws AtlasServiceException {
         WebResource resource   = getResource(API_VERSION.getNormalizedPath());
         try {
-            callAPIWithResource(API_VERSION, resource, null, JSONObject.class);
+            callAPIWithResource(API_VERSION, resource, null, ObjectNode.class);
             return true;
         } catch (ClientHandlerException che) {
             return false;
@@ -204,12 +207,12 @@ public abstract class AtlasBaseClient {
     public String getAdminStatus() throws AtlasServiceException {
         String      result    = AtlasBaseClient.UNKNOWN_STATUS;
         WebResource resource  = getResource(service, API_STATUS.getNormalizedPath());
-        JSONObject  response  = callAPIWithResource(API_STATUS, resource, null, JSONObject.class);
-        try {
-            result = response.getString("Status");
-        } catch (JSONException e) {
-            LOG.error("Exception while parsing admin status response. Returned response {}", response.toString(), e);
+        ObjectNode  response  = callAPIWithResource(API_STATUS, resource, null, ObjectNode.class);
+
+        if (response.has(STATUS)) {
+            result = response.get(STATUS).asText();
         }
+
         return result;
     }
 
@@ -264,8 +267,8 @@ public abstract class AtlasBaseClient {
         DefaultClientConfig config = new DefaultClientConfig();
         // Enable POJO mapping feature
         config.getFeatures().put(JSONConfiguration.FEATURE_POJO_MAPPING, Boolean.TRUE);
-        config.getClasses().add(MultiPartWriter.class);
         config.getClasses().add(JacksonJaxbJsonProvider.class);
+        config.getClasses().add(MultiPartWriter.class);
 
         int readTimeout = configuration.getInt("atlas.client.readTimeoutMSecs", 60000);
         int connectTimeout = configuration.getInt("atlas.client.connectTimeoutMSecs", 60000);
@@ -354,12 +357,14 @@ public abstract class AtlasBaseClient {
         ClientResponse clientResponse = null;
         int i = 0;
         do {
-            LOG.info("------------------------------------------------------");
-            LOG.info("Call         : {} {}", api.getMethod(), api.getNormalizedPath());
-            LOG.info("Content-type : {} ", api.getConsumes());
-            LOG.info("Accept       : {} ", api.getProduces());
-            if (requestObject != null) {
-                LOG.info("Request      : {}", requestObject);
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("------------------------------------------------------");
+                LOG.debug("Call         : {} {}", api.getMethod(), api.getNormalizedPath());
+                LOG.debug("Content-type : {} ", api.getConsumes());
+                LOG.debug("Accept       : {} ", api.getProduces());
+                if (requestObject != null) {
+                    LOG.debug("Request      : {}", requestObject);
+                }
             }
 
             WebResource.Builder requestBuilder = resource.getRequestBuilder();
@@ -376,8 +381,11 @@ public abstract class AtlasBaseClient {
 
             clientResponse = requestBuilder.method(api.getMethod(), ClientResponse.class, requestObject);
 
-            if (LOG.isDebugEnabled()) {
-                LOG.debug("API {} returned status {}", resource.getURI(), clientResponse.getStatus());
+            LOG.debug("HTTP Status  : {}", clientResponse.getStatus());
+
+            if (!LOG.isDebugEnabled()) {
+                LOG.info("method={} path={} contentType={} accept={} status={}", api.getMethod(),
+                    api.getNormalizedPath(), api.getConsumes(), api.getProduces(), clientResponse.getStatus());
             }
 
             if (clientResponse.getStatus() == api.getExpectedStatus().getStatusCode()) {
@@ -387,20 +395,20 @@ public abstract class AtlasBaseClient {
                 try {
                     if(api.getProduces().equals(MediaType.APPLICATION_OCTET_STREAM)) {
                         return (T) clientResponse.getEntityInputStream();
-                    } else if (responseType.getRawClass().equals(JSONObject.class)) {
+                    } else if (responseType.getRawClass().equals(ObjectNode.class)) {
                         String stringEntity = clientResponse.getEntity(String.class);
                         try {
-                            JSONObject jsonObject = new JSONObject(stringEntity);
-                            LOG.debug("Response = {}", jsonObject);
-                            LOG.info("------------------------------------------------------");
+                            JsonNode jsonObject = AtlasJson.parseToV1JsonNode(stringEntity);
+                            LOG.debug("Response     : {}", jsonObject);
+                            LOG.debug("------------------------------------------------------");
                             return (T) jsonObject;
-                        } catch (JSONException e) {
+                        } catch (IOException e) {
                             throw new AtlasServiceException(api, e);
                         }
                     } else {
                         T entity = clientResponse.getEntity(responseType);
-                        LOG.debug("Response = {}", entity);
-                        LOG.info("------------------------------------------------------");
+                        LOG.debug("Response     : {}", entity);
+                        LOG.debug("------------------------------------------------------");
                         return entity;
                     }
                 } catch (ClientHandlerException e) {
@@ -551,13 +559,13 @@ public abstract class AtlasBaseClient {
     }
 
     @VisibleForTesting
-    JSONObject callAPIWithRetries(API api, Object requestObject, ResourceCreator resourceCreator)
+    ObjectNode callAPIWithRetries(API api, Object requestObject, ResourceCreator resourceCreator)
             throws AtlasServiceException {
         for (int i = 0; i < getNumberOfRetries(); i++) {
             WebResource resource = resourceCreator.createResource();
             try {
                 LOG.debug("Using resource {} for {} times", resource.getURI(), i + 1);
-                return callAPIWithResource(api, resource, requestObject, JSONObject.class);
+                return callAPIWithResource(api, resource, requestObject, ObjectNode.class);
             } catch (ClientHandlerException che) {
                 if (i == (getNumberOfRetries() - 1)) {
                     throw che;
@@ -681,6 +689,8 @@ public abstract class AtlasBaseClient {
         private final String produces;
         private final Response.Status status;
 
+        private static final Logger LOG = LoggerFactory.getLogger(API.class);
+
         public API(String path, String method, Response.Status status) {
             this(path, method, status, JSON_MEDIA_TYPE, MediaType.APPLICATION_JSON);
         }
@@ -702,7 +712,25 @@ public abstract class AtlasBaseClient {
         }
 
         public String getNormalizedPath() {
-            return Paths.get(path).normalize().toString();
+            // This method used to return Paths.get(path).normalize().toString(), but
+            // the use of Paths.get(path) on Windows produces a path with Windows
+            // path separators (i.e. back-slashes) which is not valid for a URI
+            // and will result in an HTTP 404 status code.
+            URI uri = null;
+            String resultUri = null;
+
+            try {
+                uri = new URI(path);
+                if (uri != null) {
+                    URI normalizedUri = uri.normalize();
+                    resultUri = normalizedUri.toString();
+                }
+            } catch (Exception e) {
+                LOG.error("getNormalizedPath() caught exception for path={}", path, e);
+                resultUri = null;
+            }
+
+            return resultUri;
         }
 
         public Response.Status getExpectedStatus() {
@@ -720,7 +748,7 @@ public abstract class AtlasBaseClient {
 
     /**
      * A class to capture input state while creating the client.
-     * <p>
+     *
      * The information here will be reused when the client is re-initialized on switch-over
      * in case of High Availability.
      */
