@@ -20,69 +20,80 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
     'use strict';
 
     var CommonViewFunction = {};
-    CommonViewFunction.deleteTagModel = function(options) {
-        var modal = new Modal({
-            title: options.titleMessage,
-            okText: options.buttonText,
-            htmlContent: options.msg,
-            cancelText: "Cancel",
-            allowCancel: true,
-            okCloses: true,
-            showFooter: true,
-        }).open();
-        return modal;
-    };
     CommonViewFunction.deleteTag = function(options) {
         require(['models/VTag'], function(VTag) {
-            var tagModel = new VTag();
             if (options && options.guid && options.tagName) {
-                if (options.showLoader) {
-                    options.showLoader();
-                }
-                tagModel.deleteAssociation(options.guid, options.tagName, {
-                    skipDefaultError: true,
-                    success: function(data) {
-                        var msg = "Tag " + name.name + Messages.removeSuccessMessage;
-                        if (options.tagOrTerm === "term") {
-                            msg = "Term " + options.tagName + Messages.removeSuccessMessage;
-                        } else if (options.tagOrTerm === "tag") {
-                            msg = "Tag " + options.tagName + Messages.removeSuccessMessage;
-                        }
-                        Utils.notifySuccess({
-                            content: msg
-                        });
-                        if (options.callback) {
-                            options.callback();
-                        }
-                        if (options.collection) {
-                            options.collection.fetch({ reset: true });
-                        }
+                var tagModel = new VTag(),
+                    notifyObj = {
+                        modal: true,
+                        text: options.msg,
+                        title: options.titleMessage,
+                        okText: options.okText,
+                        ok: function(argument) {
+                            if (options.showLoader) {
+                                options.showLoader();
+                            }
+                            tagModel.deleteAssociation(options.guid, options.tagName, options.associatedGuid, {
+                                skipDefaultError: true,
+                                success: function(data) {
+                                    Utils.notifySuccess({
+                                        content: "Classification " + options.tagName + Messages.removeSuccessMessage
+                                    });
+                                    if (options.callback) {
+                                        options.callback();
+                                    }
+                                    if (options.collection) {
+                                        options.collection.fetch({ reset: true });
+                                    }
 
-                    },
-                    cust_error: function(model, response) {
-                        var message = options.tagName + Messages.deleteErrorMessage;
-                        if (response && response.responseJSON) {
-                            message = response.responseJSON.errorMessage;
+                                },
+                                cust_error: function(model, response) {
+                                    var message = options.tagName + Messages.deleteErrorMessage;
+                                    if (response && response.responseJSON) {
+                                        message = response.responseJSON.errorMessage;
+                                    }
+                                    if (options.hideLoader) {
+                                        options.hideLoader();
+                                    }
+                                    Utils.notifyError({
+                                        content: message
+                                    });
+                                }
+                            });
+                        },
+                        cancel: function(argument) {
+                            if (options.hideLoader) {
+                                options.hideLoader();
+                            }
                         }
-                        if (options.hideLoader) {
-                            options.hideLoader();
-                        }
-                        Utils.notifyError({
-                            content: message
-                        });
-                    }
-                });
+                    };
+                Utils.notifyConfirm(notifyObj);
             }
         });
     };
     CommonViewFunction.propertyTable = function(options) {
         var scope = options.scope,
+            sortBy = options.sortBy,
             valueObject = options.valueObject,
             extractJSON = options.extractJSON,
             isTable = _.isUndefined(options.isTable) ? true : options.isTable,
-            attributeDefs = options.attributeDefs;
+            attributeDefs = options.attributeDefs,
+            formatIntVal = options.formatIntVal,
+            showListCount = options.showListCount || true,
+            numberFormat = options.numberFormat || _.numberFormatWithComa;
 
         var table = "",
+            getValue = function(val) {
+                if (val) {
+                    if ((_.isNumber(val) || !_.isNaN(parseInt(val))) && formatIntVal) {
+                        return numberFormat(val);
+                    } else {
+                        return val;
+                    }
+                } else {
+                    return "N/A";
+                }
+            },
             fetchInputOutputValue = function(id, defEntity) {
                 var that = this;
                 scope.entityModel.getEntityHeader(id, {
@@ -93,13 +104,13 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                         value = Utils.getName(data);
                         var id = "";
                         if (data.guid) {
-                            if (Enums.entityStateReadOnly[data.status]) {
+                            if (Enums.entityStateReadOnly[data.status || data.entityStatus]) {
                                 deleteButton += '<button title="Deleted" class="btn btn-action btn-md deleteBtn"><i class="fa fa-trash"></i></button>';
                             }
                             id = data.guid;
                         }
                         if (value.length > 0) {
-                            scope.$('td div[data-id="' + id + '"]').html('<a href="#!/detailPage/' + id + '">' + value + '</a>');
+                            scope.$('td div[data-id="' + id + '"]').html('<a href="#!/detailPage/' + id + '">' + getValue(value) + '</a>');
                         } else {
                             scope.$('td div[data-id="' + id + '"]').html('<a href="#!/detailPage/' + id + '">' + _.escape(id) + '</a>');
                         }
@@ -120,8 +131,11 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                     complete: function() {}
                 });
             },
-            extractObject = function(keyValue, defEntity) {
-                var valueOfArray = [];
+            extractObject = function(opt) {
+                var valueOfArray = [],
+                    keyValue = opt.keyValue,
+                    key = opt.key,
+                    defEntity = opt.defEntity;
                 if (!_.isArray(keyValue) && _.isObject(keyValue)) {
                     keyValue = [keyValue];
                 }
@@ -130,7 +144,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                     var inputOutputField = keyValue[i],
                         id = inputOutputField.guid || (_.isObject(inputOutputField.id) ? inputOutputField.id.id : inputOutputField.id),
                         tempLink = "",
-                        status = inputOutputField.status || (_.isObject(inputOutputField.id) ? inputOutputField.id.state : inputOutputField.state),
+                        status = (inputOutputField.status || inputOutputField.entityStatus) || (_.isObject(inputOutputField.id) ? inputOutputField.id.state : inputOutputField.state),
                         readOnly = Enums.entityStateReadOnly[status];
                     if (!inputOutputField.attributes && inputOutputField.values) {
                         inputOutputField['attributes'] = inputOutputField.values;
@@ -138,7 +152,7 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                     if (_.isString(inputOutputField) || _.isBoolean(inputOutputField) || _.isNumber(inputOutputField)) {
                         var tempVarfor$check = inputOutputField.toString();
                         if (tempVarfor$check.indexOf("$") == -1) {
-                            valueOfArray.push('<span class="json-string">' + _.escape(inputOutputField) + '</span>');
+                            valueOfArray.push('<span class="json-string">' + getValue(_.escape(inputOutputField)) + '</span>');
                         }
                     } else if (_.isObject(inputOutputField) && !id) {
                         var attributesList = inputOutputField;
@@ -170,7 +184,6 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                         } else {
                             valueOfArray.push(Utils.JSONPrettyPrint(attributesList));
                         }
-
                     }
                     if (id && inputOutputField) {
                         var name = Utils.getName(inputOutputField);
@@ -205,23 +218,29 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 }
                 return subLink;
             }
-        _.sortBy(_.keys(valueObject)).map(function(key) {
+        var valueObjectKeysList = _.keys(valueObject);
+        if (_.isUndefined(sortBy) || sortBy == true) {
+            valueObjectKeysList = _.sortBy(valueObjectKeysList);
+        }
+        valueObjectKeysList.map(function(key) {
+
             key = _.escape(key);
             if (key == "profileData") {
                 return;
             }
-            var keyValue = valueObject[key];
+            var keyValue = valueObject[key],
+                listCount = showListCount && _.isArray(keyValue) && keyValue.length > 0 ? ' (' + numberFormat(keyValue.length) + ')' : "";
             var defEntity = _.find(attributeDefs, { name: key });
             if (defEntity && defEntity.typeName) {
                 var defEntityType = defEntity.typeName.toLocaleLowerCase();
-                if (defEntityType === 'date' || defEntityType === 'time') {
+                if (defEntityType === 'date') {
                     keyValue = new Date(keyValue);
                 } else if (_.isObject(keyValue)) {
-                    keyValue = extractObject(keyValue, defEntity);
+                    keyValue = extractObject({ "keyValue": keyValue, "key": key, 'defEntity': defEntity });
                 }
             } else {
                 if (_.isObject(keyValue)) {
-                    keyValue = extractObject(keyValue)
+                    keyValue = extractObject({ "keyValue": keyValue, "key": key });
                 }
             }
             var val = "";
@@ -231,171 +250,58 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
                 val = '<a target="_blank" class="blue-link" href="' + keyValue + '">' + keyValue + '</a>';
             } else if (key === 'guid' || key === "__guid") {
                 val = '<a title="' + key + '" href="#!/detailPage/' + keyValue + '">' + keyValue + '</a>';
-            } else if (key.toLocaleLowerCase().indexOf("time") !== -1 || key.toLocaleLowerCase().indexOf("date") !== -1) {
-                val = new Date(keyValue);
             } else {
                 val = _.escape(keyValue);
             }
             if (isTable) {
-                var htmlTag = '<div class="scroll-y">' + val + '</div>';
-                if (_.isObject(valueObject[key])) {
+                var htmlTag = '<div class="scroll-y">' + getValue(val) + '</div>';
+                if (_.isObject(valueObject[key]) && !_.isEmpty(valueObject[key])) {
                     var matchedLinkString = val.match(/href|value-loader\w*/g),
                         matchedJson = val.match(/json-value|json-string\w*/g),
-                        isMatchLinkStringIsSingle = matchedLinkString && matchedLinkString.length == 1,
+                        isMatchLinkStringIsSingle = matchedLinkString && matchedLinkString.length <= 5,
                         isMatchJSONStringIsSingle = matchedJson && matchedJson.length == 1,
                         expandCollapseButton = "";
                     if ((matchedJson && !isMatchJSONStringIsSingle) || (matchedLinkString && !isMatchLinkStringIsSingle)) {
-                        var expandCollapseButton = '<button class="expand-collapse-button"><i class="fa"></i></button>'
+                        expandCollapseButton = '<button class="expand-collapse-button"><i class="fa"></i></button>';
+                        htmlTag = '<pre class="shrink code-block ' + (isMatchJSONStringIsSingle ? 'fixed-height' : '') + '">' + expandCollapseButton + '<code>' + val + '</code></pre>';
                     }
-                    var htmlTag = '<pre class="shrink code-block ' + (isMatchJSONStringIsSingle ? 'fixed-height' : '') + '">' + expandCollapseButton + '<code>' + val + '</code></pre>';
                 }
-                table += '<tr><td>' + _.escape(key) + '</td><td>' + htmlTag + '</td></tr>';
+                table += '<tr><td>' + (_.escape(key) + listCount) + '</td><td>' + htmlTag + '</td></tr>';
             } else {
                 table += '<div>' + val + '</div>';
             }
 
         });
-        return table;
-    }
-    CommonViewFunction.breadcrumbUrlMaker = function(url) {
-        if (url) {
-            var urlList = [];
-            var splitURL = url.split("api/atlas/v1/taxonomies/");
-            if (splitURL.length > 1) {
-                var splitUrlWithoutTerm = splitURL[1].split("/terms/");
-                if (splitUrlWithoutTerm.length == 1) {
-                    splitUrlWithoutTerm = splitUrlWithoutTerm[0].split("/");
-                }
-            } else {
-                var splitUrlWithoutTerm = splitURL[0].split("/terms/");
-                if (splitUrlWithoutTerm.length == 1) {
-                    splitUrlWithoutTerm = splitUrlWithoutTerm[0].split("/");
-                }
-            }
-
-            var href = "";
-            for (var i in splitUrlWithoutTerm) {
-                if (i == 0) {
-                    href = splitUrlWithoutTerm[i];
-                    urlList.push({
-                        value: _.escape(splitUrlWithoutTerm[i]),
-                        href: href
-                    });
-                } else {
-                    href += "/terms/" + splitUrlWithoutTerm[i];
-                    urlList.push({
-                        value: _.escape(splitUrlWithoutTerm[i]),
-                        href: href
-                    });
-                };
-            }
-            return urlList;
-        }
-    }
-    CommonViewFunction.breadcrumbMaker = function(options) {
-        var li = "";
-        if (options.urlList) {
-            _.each(options.urlList, function(object) {
-                li += '<li><a class="link" href="#!/taxonomy/detailCatalog/api/atlas/v1/taxonomies/' + object.href + '?load=true">' + _.escape(object.value) + '</a></li>';
-            });
-        }
-        if (options.scope) {
-            options.scope.html(li);
-            options.scope.asBreadcrumbs("destroy");
-            options.scope.asBreadcrumbs({
-                namespace: 'breadcrumb',
-                overflow: "left",
-                responsive: false,
-                toggleIconClass: 'fa fa-ellipsis-h',
-                dropdown: function(classes) {
-                    var dropdownClass = 'dropdown';
-                    var dropdownMenuClass = 'dropdown-menu popover popoverTerm bottom arrowPosition';
-                    if (this.options.overflow === 'right') {
-                        dropdownMenuClass += ' dropdown-menu-right';
-                    }
-
-                    return '<li class="' + dropdownClass + ' ' + classes.dropdownClass + '">' +
-                        '<a href="javascript:void(0);" class="' + classes.toggleClass + '" data-toggle="dropdown">' +
-                        '<i class="' + classes.toggleIconClass + '"></i>' +
-                        '</a>' +
-                        '<ul class="' + dropdownMenuClass + ' ' + classes.dropdownMenuClass + '">' +
-                        '<div class="arrow"></div>' +
-                        '</ul>' +
-                        '</li>';
-                }
-            });
-        }
-    }
-    CommonViewFunction.termTableBreadcrumbMaker = function(obj) {
-        if (!obj) {
-            return "";
-        }
-        var traits = obj.classificationNames || _.pluck(obj.classifications, 'typeName'),
-            url = "",
-            deleteHtml = "",
-            html = "",
-            id = obj.guid,
-            terms = [],
-            entityName = Utils.getName(obj);
-        if (traits) {
-            traits.map(function(term) {
-                var checkTagOrTerm = Utils.checkTagOrTerm(term);
-                if (checkTagOrTerm.term) {
-                    terms.push({
-                        deleteHtml: '<a href="javascript:void(0)" class="pull-left" title="Remove Term"><i class="fa fa-trash" data-id="tagClick" data-type="term" data-assetname="' + entityName + '" data-name="' + term + '" data-guid="' + obj.guid + '" ></i></a>',
-                        url: _.unescape(term).split(".").join("/"),
-                        name: term
-                    });
-                }
-            });
-        }
-        _.each(terms, function(obj, i) {
-            var className = "";
-            if (i >= 1) {
-                className += "showHideDiv hide";
-            }
-            obj['valueUrl'] = CommonViewFunction.breadcrumbUrlMaker(obj.url);
-            html += '<div class="' + className + '" dataterm-name="' + obj.name + '"><div class="liContent"></div>' + obj.deleteHtml + '</div>';
-        })
-        if (terms.length > 1) {
-            html += '<div><a  href="javascript:void(0)" data-id="showMoreLessTerm" class="btn btn-action btn-sm btn-icon-pd"><span>Show More </span><i class="fa fa-angle-right"></i></a></div>'
-        }
-        if (!Enums.entityStateReadOnly[obj.status]) {
-            if (obj.guid) {
-                html += '<div><a href="javascript:void(0)" class="btn btn-action btn-sm" data-id="addTerm" data-guid="' + (obj.guid) + '"><i class="fa fa-plus"></i></a></div>'
-            } else {
-                html += '<div><a href="javascript:void(0)" class="btn btn-action btn-sm" data-id="addTerm"><i class="fa fa-plus"></i></a></div>'
-            }
-        }
-        return {
-            html: '<div class="termTableBreadcrumb btn-inline" dataterm-id="' + id + '">' + html + '</div>',
-            object: { scopeId: id, value: terms }
-        }
-
+        return table && table.length > 0 ? table : '<tr class="empty"><td colspan="22"><span>No Record found!</span></td></tr>';
     }
     CommonViewFunction.tagForTable = function(obj) {
-        var traits = obj.classificationNames || _.pluck(obj.classifications, 'typeName'),
-            atags = "",
+        var traits = obj.classifications,
+            tagHtml = "",
             addTag = "",
             popTag = "",
             count = 0,
             entityName = Utils.getName(obj);
         if (traits) {
             traits.map(function(tag) {
-                var checkTagOrTerm = Utils.checkTagOrTerm(tag);
-                if (checkTagOrTerm.tag) {
-                    var className = "btn btn-action btn-sm btn-blue btn-icon",
-                        tagString = '<a class="' + className + '" data-id="tagClick"><span title="' + tag + '">' + tag + '</span><i class="fa fa-times" data-id="delete"  data-assetname="' + entityName + '"data-name="' + tag + '" data-type="tag" data-guid="' + obj.guid + '" ></i></a>';
-                    if (count >= 1) {
-                        popTag += tagString;
-                    } else {
-                        atags += tagString;
-                    }
-                    ++count;
+                var className = "btn btn-action btn-sm btn-blue btn-icon",
+                    deleteIcon = "";
+                if (obj.guid === tag.entityGuid) {
+                    deleteIcon = '<i class="fa fa-times" data-id="delete"  data-assetname="' + entityName + '"data-name="' + tag.typeName + '" data-type="tag" data-guid="' + obj.guid + '" ></i>';
+                } else if (obj.guid !== tag.entityGuid && tag.entityStatus === "DELETED") {
+                    deleteIcon = '<i class="fa fa-times" data-id="delete"  data-assetname="' + entityName + '"data-name="' + tag.typeName + '" data-type="tag" data-entityguid="' + tag.entityGuid + '" data-guid="' + obj.guid + '" ></i>';
+                } else {
+                    className += " propagte-classification";
                 }
+                var tagString = '<a class="' + className + '" data-id="tagClick"><span title="' + tag.typeName + '">' + tag.typeName + '</span>' + deleteIcon + '</a>';
+                if (count >= 1) {
+                    popTag += tagString;
+                } else {
+                    tagHtml += tagString;
+                }
+                ++count;
             });
         }
-        if (!Enums.entityStateReadOnly[obj.status]) {
+        if (!Enums.entityStateReadOnly[obj.status || obj.entityStatus]) {
             if (obj.guid) {
                 addTag += '<a href="javascript:void(0)" data-id="addTag" class="btn btn-action btn-sm assignTag" data-guid="' + obj.guid + '" ><i class="fa fa-plus"></i></a>';
             } else {
@@ -403,9 +309,41 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
             }
         }
         if (count > 1) {
-            addTag += '<div data-id="showMoreLess" class="btn btn-action btn-sm assignTag"><i class="fa fa-ellipsis-h" aria-hidden="true"></i><div class="popup-tag">' + popTag + '</div></div>'
+            addTag += '<div data-id="showMoreLess" class="btn btn-action btn-sm assignTag"><i class="fa fa-ellipsis-h" aria-hidden="true"></i><div class="popup-tag-term">' + popTag + '</div></div>'
         }
-        return '<div class="tagList btn-inline btn-fixed-width">' + atags + addTag + '</div>';
+        return '<div class="tagList btn-inline btn-fixed-width">' + tagHtml + addTag + '</div>';
+    }
+    CommonViewFunction.termForTable = function(obj) {
+        var terms = obj.meanings,
+            termHtml = "",
+            addTerm = "",
+            popTerm = "",
+            count = 0,
+            entityName = Utils.getName(obj);
+        if (terms) {
+            terms.map(function(term) {
+                var className = "btn btn-action btn-sm btn-blue btn-icon",
+                    deleteIcon = '<i class="fa fa-times" data-id="delete"  data-assetname="' + entityName + '"data-name="' + term.displayText + '" data-type="term" data-guid="' + obj.guid + '" data-termGuid="' + term.termGuid + '" ></i>',
+                    termString = '<a class="' + className + '" data-id="termClick"><span title="' + _.escape(term.displayText) + '">' + _.escape(term.displayText) + '</span>' + deleteIcon + '</a>';
+                if (count >= 1) {
+                    popTerm += termString;
+                } else {
+                    termHtml += termString;
+                }
+                ++count;
+            });
+        }
+        if (!Enums.entityStateReadOnly[obj.status || obj.entityStatus]) {
+            if (obj.guid) {
+                addTerm += '<a href="javascript:void(0)" data-id="addTerm" class="btn btn-action btn-sm assignTag" data-guid="' + obj.guid + '" ><i class="fa fa-plus"></i></a>';
+            } else {
+                addTerm += '<a href="javascript:void(0)" data-id="addTerm" class="btn btn-action btn-sm assignTag"><i style="right:0" class="fa fa-plus"></i></a>';
+            }
+        }
+        if (count > 1) {
+            addTerm += '<div data-id="showMoreLess" class="btn btn-action btn-sm assignTerm"><i class="fa fa-ellipsis-h" aria-hidden="true"></i><div class="popup-tag-term">' + popTerm + '</div></div>'
+        }
+        return '<div class="tagList btn-inline btn-fixed-width">' + termHtml + addTerm + '</div>';
     }
     CommonViewFunction.generateQueryOfFilter = function(value) {
         var entityFilters = CommonViewFunction.attributeFilter.extractUrl({ "value": value.entityFilters, "formatDate": true }),
@@ -431,13 +369,17 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
             queryArray.push(typeKeyValue)
         }
         if (value.tag) {
-            var tagKeyValue = '<span class="key">Tag:</span>&nbsp<span class="value">' + _.escape(value.tag) + '</span>';
+            var tagKeyValue = '<span class="key">Classification:</span>&nbsp<span class="value">' + _.escape(value.tag) + '</span>';
             if (tagFilters) {
                 var conditionFortag = tagFilters.rules.length == 1 ? '' : 'AND';
                 tagKeyValue += '&nbsp<span class="operator">' + conditionFortag + '</span>&nbsp(<span class="operator">' + tagFilters.condition + '</span>&nbsp(' + objToString(tagFilters) + '))';
             }
             queryArray.push(tagKeyValue);
         }
+        if (value.term) {
+            var tagKeyValue = '<span class="key">Term:</span>&nbsp<span class="value">' + _.escape(value.term) + '</span>';
+            queryArray.push(tagKeyValue);
+        }
         if (value.query) {
             queryArray.push('<span class="key">Query:</span>&nbsp<span class="value">' + _.trim(_.escape(value.query)) + '</span>&nbsp');
         }
@@ -448,32 +390,6 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
 
         }
     }
-    CommonViewFunction.saveTermToAsset = function(options, that) {
-        require(['models/VCatalog'], function(Vcatalog) {
-            var VCatalog = new Vcatalog();
-            var name = options.termName;
-            ++that.asyncFetchCounter;
-            VCatalog.url = function() {
-                return "api/atlas/v1/entities/" + options.guid + "/tags/" + name;
-            };
-            VCatalog.save(null, {
-                success: function(data) {
-                    Utils.notifySuccess({
-                        content: "Term " + name + Messages.addTermToEntitySuccessMessage
-                    });
-                    if (options.collection) {
-                        options.collection.fetch({ reset: true });
-                    }
-                },
-                complete: function() {
-                    --that.asyncFetchCounter
-                    if (that.callback && that.asyncFetchCounter === 0) {
-                        that.callback(); // It will call to parent of parent Callback i.e callback of searchLayoutView
-                    }
-                }
-            });
-        })
-    }
     CommonViewFunction.generateObjectForSaveSearchApi = function(options) {
         var obj = {
             name: options.name,
@@ -702,8 +618,193 @@ define(['require', 'utils/Utils', 'modules/Modal', 'utils/Messages', 'utils/Enum
             }
         }
     }
+    CommonViewFunction.createEditGlossaryCategoryTerm = function(options) {
+        if (options) {
+            var model = options.model,
+                isTermView = options.isTermView,
+                isGlossaryView = options.isGlossaryView,
+                collection = options.collection
+        }
+        require([
+            'views/glossary/CreateEditCategoryTermLayoutView',
+            'views/glossary/CreateEditGlossaryLayoutView',
+            'modules/Modal'
+        ], function(CreateEditCategoryTermLayoutView, CreateEditGlossaryLayoutView, Modal) {
+            var view = null,
+                title = null;
+            if (isGlossaryView) {
+                view = new CreateEditGlossaryLayoutView({ "glossaryCollection": collection, "model": model });
+                title = "Glossary";
+            } else {
+                view = new CreateEditCategoryTermLayoutView({ "glossaryCollection": collection, "modelJSON": model });
+                title = (isTermView ? 'Term' : 'Category');
+            }
+
+            var modal = new Modal({
+                "title": ((model ? "Update " : "Create ") + title),
+                "content": view,
+                "cancelText": "Cancel",
+                "okCloses": false,
+                "okText": model ? "Update" : "Create",
+                "allowCancel": true
+            }).open();
+            modal.$el.find('button.ok').attr("disabled", "true");
+            if (model) {
+                view.$('input,textarea').on('keyup', function(e) {
+                    modal.$el.find('button.ok').attr("disabled", false);
+                });
+            } else {
+                view.ui.name.on('keyup', function(e) {
+                    modal.$el.find('button.ok').attr("disabled", false);
+                });
+            }
+            view.ui.name.on('keyup', function(e) {
+                if ((e.keyCode == 8 || e.keyCode == 32 || e.keyCode == 46) && e.currentTarget.value.trim() == "") {
+                    modal.$el.find('button.ok').attr("disabled", true);
+                }
+            });
+            modal.on('ok', function() {
+                modal.$el.find('button.ok').attr("disabled", true);
+                CommonViewFunction.createEditGlossaryCategoryTermSubmit(_.extend({ "ref": view, "modal": modal }, options));
+            });
+            modal.on('closeModal', function() {
+                modal.trigger('cancel');
+                if (options.onModalClose) {
+                    options.onModalClose()
+                }
+            });
+        });
+    }
+    CommonViewFunction.createEditGlossaryCategoryTermSubmit = function(options) {
+        if (options) {
+            var ref = options.ref,
+                modal = options.modal,
+                model = options.model,
+                node = options.node,
+                isTermView = options.isTermView,
+                isCategoryView = options.isCategoryView,
+                collection = options.collection,
+                isGlossaryView = options.isGlossaryView,
+                data = ref.ui[(isGlossaryView ? "glossaryForm" : "categoryTermForm")].serializeArray().reduce(function(obj, item) {
+                    obj[item.name] = item.value;
+                    return obj;
+                }, {}),
+                newModel = new options.collection.model(),
+                messageType = "Glossary ";
+        }
+        if (isTermView) {
+            messageType = "Term ";
+        } else if (isCategoryView) {
+            messageType = "Category ";
+        }
+        var ajaxOptions = {
+            silent: true,
+            success: function(rModel, response) {
+                Utils.notifySuccess({
+                    content: messageType + ref.ui.name.val() + Messages[model ? "editSuccessMessage" : "addSuccessMessage"]
+                });
+                if (options.callback) {
+                    options.callback(rModel);
+                }
+                modal.trigger('closeModal');
+            },
+            cust_error: function() {
+                modal.$el.find('button.ok').attr("disabled", false);
+            }
+        }
+        if (model) {
+            if (isGlossaryView) {
+                model.clone().set(data, { silent: true }).save(null, ajaxOptions)
+            } else {
+                newModel[isTermView ? "createEditTerm" : "createEditCategory"](_.extend(ajaxOptions, {
+                    guid: model.guid,
+                    data: JSON.stringify(_.extend({}, model, data)),
+                }));
+            }
+
+        } else {
+            if (isGlossaryView) {
+                new collection.model().set(data).save(null, ajaxOptions);
+            } else {
+                if (node) {
+                    var key = "anchor",
+                        guidKey = "glossaryGuid";
+                    data["anchor"] = {
+                        "glossaryGuid": node.glossaryId || node.guid,
+                        "displayText": node.glossaryName || node.text
+                    }
+                    if (node.type == "GlossaryCategory") {
+                        data["parentCategory"] = {
+                            "categoryGuid": node.guid
+                        }
+                    }
+                }
+                newModel[isTermView ? "createEditTerm" : "createEditCategory"](_.extend(ajaxOptions, {
+                    data: JSON.stringify(data),
+                }));
+            }
+        }
+
+    }
+    CommonViewFunction.removeCategoryTermAssociation = function(options) {
+        if (options) {
+            var selectedGuid = options.selectedGuid,
+                termGuid = options.termGuid,
+                isCategoryView = options.isCategoryView,
+                isTermView = options.isTermView,
+                isEntityView = options.isEntityView,
+                collection = options.collection,
+                model = options.model,
+                newModel = new options.collection.model(),
+                ajaxOptions = {
+                    success: function(rModel, response) {
+                        Utils.notifySuccess({
+                            content: ((isCategoryView || isEntityView ? "Term" : "Category") + " association is removed successfully")
+                        });
+                        if (options.callback) {
+                            options.callback();
+                        }
+                    },
+                    cust_error: function() {
+                        if (options.hideLoader) {
+                            options.hideLoader();
+                        }
+                    }
+                },
+                notifyObj = {
+                    modal: true,
+                    text: options.msg,
+                    title: options.titleMessage,
+                    okText: options.buttonText,
+                    ok: function(argument) {
+                        if (options.showLoader) {
+                            options.showLoader();
+                        }
+                        if (isEntityView && model) {
+                            var data = [model];
+                            newModel.removeTermFromEntity(termGuid, _.extend(ajaxOptions, {
+                                data: JSON.stringify(data)
+                            }))
+                        } else {
+                            var data = _.extend({}, model);
+                            if (isTermView) {
+                                data.categories = _.reject(data.categories, function(term) { return term.categoryGuid == selectedGuid });
+                            } else {
+                                data.terms = _.reject(data.terms, function(term) { return term.termGuid == selectedGuid });
+                            }
+
+                            newModel[isTermView ? "createEditTerm" : "createEditCategory"](_.extend(ajaxOptions, {
+                                guid: model.guid,
+                                data: JSON.stringify(_.extend({}, model, data)),
+                            }));
+                        }
+                    },
+                    cancel: function(argument) {}
+                };
+            Utils.notifyConfirm(notifyObj);
+        }
+    }
     CommonViewFunction.addRestCsrfCustomHeader = function(xhr, settings) {
-        //    if (settings.url == null || !settings.url.startsWith('/webhdfs/')) {
         if (settings.url == null) {
             return;
         }
