@@ -40,6 +40,9 @@ define(['require', 'utils/Enums', 'utils/Utils', 'underscore'], function(require
         enumDefApiUrl: function(name) {
             return this.getDefApiUrl('enum', name);
         },
+        entityCountApi: function(){
+            return this.baseUrl + '/admin/metrics'
+        },
         getDefApiUrl: function(type, name) {
             var defApiUrl = this.typedefsUrl(), defUrl;
             if (name) {
@@ -53,19 +56,16 @@ define(['require', 'utils/Enums', 'utils/Utils', 'underscore'], function(require
                 return defUrl;
             }
         },
-        taxonomiesApiUrl: function() {
-            return this.baseUrl + '/v1/taxonomies';
-        },
-        taxonomiesTermsApiUrl: function(name) {
-            return this.baseUrl + '/v1/taxonomies' + '/' + name + '/terms';
-        },
         entitiesApiUrl: function(options) {
             var entitiesUrl = this.baseUrlV2 + '/entity';
             if (options) {
                 var guid = options.guid,
+                    associatedGuid = options.associatedGuid,
                     name = options.name,
                     minExtInfo = options.minExtInfo;
-                if (guid && name) {
+                if (guid && name && associatedGuid) {
+                    return entitiesUrl + '/guid/' + guid + '/classification/' + name + '?associatedEntityGuid=' + associatedGuid;
+                } else if (guid && name) {
                     entitiesUrl += '/guid/' + guid + '/classification/' + name;
                 } else if (guid && !name) {
                     entitiesUrl += '/guid/' + guid;
@@ -90,7 +90,7 @@ define(['require', 'utils/Enums', 'utils/Utils', 'underscore'], function(require
             }
         },
         entityCollectionaudit: function(guid) {
-            return this.baseUrl + '/entities/' + guid + '/audit';
+            return this.baseUrlV2 + '/entity/' + guid + '/audit';
         },
         expimpAudit: function(options) {
             var url = this.baseUrl + '/admin/expimp/audit',
@@ -124,7 +124,7 @@ define(['require', 'utils/Enums', 'utils/Utils', 'underscore'], function(require
             }
         },
         typesApiUrl: function() {
-            return this.typedefsUrl().defs + '/headers'
+            return this.typedefsUrl().defs + '/headers?excludeInternalTypesAndReferences=true'
         },
         lineageApiUrl: function(guid) {
             var lineageUrl = this.baseUrlV2 + '/lineage';
@@ -134,6 +134,14 @@ define(['require', 'utils/Enums', 'utils/Utils', 'underscore'], function(require
                 return lineageUrl
             }
         },
+        relationshipApiUrl: function(guid) {
+            var relationshipUrl = this.baseUrlV2 + '/relationship';
+            if (guid) {
+                return relationshipUrl + '/guid/' + guid + '?extendedInfo=true';
+            } else {
+                return relationshipUrl
+            }
+        },
         schemaApiUrl: function(guid) {
             var lineageUrl = this.baseUrl + '/lineage';
             if (guid) {
@@ -158,6 +166,51 @@ define(['require', 'utils/Enums', 'utils/Utils', 'underscore'], function(require
                 return saveSearchUrl;
             }
         },
+        glossaryApiUrl: function(options) {
+            var guid = options && options.guid,
+                glossaryUrl = this.baseUrlV2 + '/glossary';
+            if (guid) {
+                return glossaryUrl + '/' + guid;
+            } else {
+                return glossaryUrl;
+            }
+        },
+        categoryApiUrl: function(options) {
+            var guid = options && options.guid,
+                list = options && options.list,
+                related = options && options.related,
+                categoryUrl = this.glossaryApiUrl() + '/' + (list ? 'categories' : 'category');
+            if (guid) {
+                if (related) {
+                    return categoryUrl + '/' + guid + "/related";
+                } else {
+                    return categoryUrl + '/' + guid;
+                }
+            } else {
+                return categoryUrl;
+            }
+        },
+        termApiUrl: function(options) {
+            var guid = options && options.guid,
+                list = options && options.list,
+                related = options && options.related,
+                termUrl = this.glossaryApiUrl() + '/' + (list ? 'terms' : 'term');
+            if (guid) {
+                if (related) {
+                    return termUrl + '/' + guid + "/related";
+                } else {
+                    return termUrl + '/' + guid;
+                }
+            } else {
+                return termUrl;
+            }
+        },
+        termToEntityApiUrl: function(guid) {
+            var termUrl = this.termApiUrl({ list: true });
+            if (guid) {
+                return termUrl + '/' + guid + '/assignedEntities';
+            }
+        },
         versionApiUrl: function() {
             return this.baseUrl + '/admin/version';
         },
