@@ -76,7 +76,6 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
     Utils.getBaseUrl = function(url) {
         return url.replace(/\/[\w-]+.(jsp|html)|\/+$/ig, '');
     };
-
     Utils.getEntityIconPath = function(options) {
         var entityData = options && options.entityData,
             serviceType,
@@ -175,7 +174,7 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
         var modal = {};
         if (options && options.modal) {
             var myStack = { "dir1": "down", "dir2": "right", "push": "top", 'modal': true };
-            modal['addclass'] = 'stack-modal';
+            modal['addclass'] = 'stack-modal ' + (options.modalClass ? modalClass : 'width-500');
             modal['stack'] = myStack;
         }
         notify(_.extend({
@@ -184,7 +183,7 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
             confirm: {
                 confirm: true,
                 buttons: [{
-                        text: 'cancel',
+                        text: options.cancelText || 'Cancel',
                         addClass: 'btn-action btn-md',
                         click: function(notice) {
                             options.cancel(notice);
@@ -192,7 +191,7 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
                         }
                     },
                     {
-                        text: 'Ok',
+                        text: options.okText || 'Ok',
                         addClass: 'btn-atlas btn-md',
                         click: function(notice) {
                             options.ok(notice);
@@ -332,10 +331,10 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
                 };
                 if (Utils.getUrlState.isTagTab(options.url)) {
                     urlUpdate['tagUrl'] = options.url;
-                } else if (Utils.getUrlState.isTaxonomyTab(options.url)) {
-                    urlUpdate['taxonomyUrl'] = options.url;
                 } else if (Utils.getUrlState.isSearchTab(options.url)) {
                     urlUpdate['searchUrl'] = options.url;
+                } else if (Utils.getUrlState.isGlossaryTab(options.url)) {
+                    urlUpdate['glossaryUrl'] = options.url;
                 }
                 $.extend(Globals.saveApplicationState.tabState, urlUpdate);
             }
@@ -356,21 +355,38 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
                 lastValue: hashValue.split('/')[hashValue.split('/').length - 1]
             }
         },
+        checkTabUrl: function(options) {
+            var url = options && options.url,
+                matchString = options && options.matchString,
+                quey = this.getQueryUrl(url);
+            return quey.firstValue == matchString || quey.queyParams[0] == "#!/" + matchString;
+        },
         isInitial: function() {
             return this.getQueryUrl().firstValue == undefined;
         },
         isTagTab: function(url) {
-            var quey = this.getQueryUrl(url);
-            return quey.firstValue == "tag" || quey.queyParams[0] == "#!/tag";
-        },
-        isTaxonomyTab: function(url) {
-            return this.getQueryUrl(url).firstValue == "taxonomy" ? true : false;
+            return this.checkTabUrl({
+                url: url,
+                matchString: "tag"
+            });
         },
         isSearchTab: function(url) {
-            return this.getQueryUrl(url).firstValue == "search";
+            return this.checkTabUrl({
+                url: url,
+                matchString: "search"
+            });
+        },
+        isGlossaryTab: function(url) {
+            return this.checkTabUrl({
+                url: url,
+                matchString: "glossary"
+            });
         },
         isDetailPage: function(url) {
-            return this.getQueryUrl(url).firstValue == "detailPage";
+            return this.checkTabUrl({
+                url: url,
+                matchString: "detailPage"
+            });
         },
         getLastValue: function() {
             return this.getQueryUrl().lastValue;
@@ -406,63 +422,6 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
             }
         }
     }
-    Utils.checkTagOrTerm = function(value, isTermView) {
-        if (value && _.isString(value) && isTermView) {
-            // For string break
-            if (value == "TaxonomyTerm") {
-                return {}
-            }
-            var name = _.escape(value).split('.');
-            return {
-                term: true,
-                tag: false,
-                name: name[name.length - 1],
-                fullName: value
-            }
-        }
-        if (value && _.isString(value)) {
-            value = {
-                typeName: value
-            }
-        }
-        if (_.isObject(value)) {
-            var name = "";
-            if (value && value.$typeName$) {
-                name = value.$typeName$;
-            } else if (value && value.typeName) {
-                name = value.typeName;
-            }
-            if (name === "TaxonomyTerm") {
-                return {}
-            }
-            name = _.escape(name).split('.');
-
-            var trem = false;
-            if (value['taxonomy.namespace']) {
-                trem = true;
-            } else if (value.values && value.values['taxonomy.namespace']) {
-                trem = true;
-            } else if (Globals.taxonomy && name.length > 1) {
-                trem = true; // Temp fix
-            }
-
-            if (trem) {
-                return {
-                    term: true,
-                    tag: false,
-                    name: name[name.length - 1],
-                    fullName: name.join('.')
-                }
-            } else {
-                return {
-                    term: false,
-                    tag: true,
-                    name: name[name.length - 1],
-                    fullName: name.join('.')
-                }
-            }
-        }
-    }
     Utils.getName = function() {
         return Utils.extractKeyValueFromEntity.apply(this, arguments).name;
     }
@@ -471,12 +430,13 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
     }
     Utils.extractKeyValueFromEntity = function() {
         var collectionJSON = arguments[0],
-            priorityAttribute = arguments[1];
-        var returnObj = {
-            name: '-',
-            found: true,
-            key: null
-        }
+            priorityAttribute = arguments[1],
+            skipAttribute = arguments[2],
+            returnObj = {
+                name: '-',
+                found: true,
+                key: null
+            }
         if (collectionJSON) {
             if (collectionJSON.attributes && collectionJSON.attributes[priorityAttribute]) {
                 returnObj.name = _.escape(collectionJSON.attributes[priorityAttribute]);
@@ -494,11 +454,26 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
                     returnObj.key = 'name';
                     return returnObj;
                 }
+                if (collectionJSON.attributes.displayName) {
+                    returnObj.name = _.escape(collectionJSON.attributes.displayName);
+                    returnObj.key = 'displayName';
+                    return returnObj;
+                }
                 if (collectionJSON.attributes.qualifiedName) {
                     returnObj.name = _.escape(collectionJSON.attributes.qualifiedName);
                     returnObj.key = 'qualifiedName';
                     return returnObj;
                 }
+                if (collectionJSON.attributes.displayText) {
+                    returnObj.name = _.escape(collectionJSON.attributes.displayText);
+                    returnObj.key = 'displayText';
+                    return returnObj;
+                }
+                if (collectionJSON.attributes.guid) {
+                    returnObj.name = _.escape(collectionJSON.attributes.guid);
+                    returnObj.key = 'guid';
+                    return returnObj;
+                }
                 if (collectionJSON.attributes.id) {
                     if (_.isObject(collectionJSON.attributes.id)) {
                         if (collectionJSON.id.id) {
@@ -516,6 +491,11 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
                 returnObj.key = 'name';
                 return returnObj;
             }
+            if (collectionJSON.displayName) {
+                returnObj.name = _.escape(collectionJSON.displayName);
+                returnObj.key = 'displayName';
+                return returnObj;
+            }
             if (collectionJSON.qualifiedName) {
                 returnObj.name = _.escape(collectionJSON.qualifiedName);
                 returnObj.key = 'qualifiedName';
@@ -544,25 +524,36 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
             }
         }
         returnObj.found = false;
-        return returnObj;
+        if (skipAttribute && returnObj.key == skipAttribute) {
+            return {
+                name: '-',
+                found: true,
+                key: null
+            }
+        } else {
+            return returnObj;
+        }
+
     }
     Utils.showTitleLoader = function(loaderEl, titleBoxEl) {
-        loaderEl.css({
+        loaderEl.css ? loaderEl.css({
             'display': 'block',
             'position': 'relative',
             'height': '85px',
             'marginTop': '85px',
             'marginLeft': '50%',
             'left': '0%'
-        });
-        titleBoxEl.hide();
+        }) : null;
+        titleBoxEl.hide ? titleBoxEl.hide() : null;
     }
     Utils.hideTitleLoader = function(loaderEl, titleBoxEl) {
-        loaderEl.hide();
-        titleBoxEl.fadeIn();
+        loaderEl.hide ? loaderEl.hide() : null;
+        titleBoxEl.fadeIn ? titleBoxEl.fadeIn() : null;
     }
-    Utils.findAndMergeRefEntity = function(attributeObject, referredEntities) {
-        var megeObject = function(obj) {
+    Utils.findAndMergeRefEntity = function(options) {
+        var attributeObject = options.attributeObject,
+            referredEntities = options.referredEntities
+        var mergeObject = function(obj) {
             if (obj) {
                 if (obj.attributes) {
                     Utils.findAndMergeRefEntity(obj.attributes, referredEntities);
@@ -576,15 +567,43 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
                 if (_.isObject(obj)) {
                     if (_.isArray(obj)) {
                         _.each(obj, function(value) {
-                            megeObject(value);
+                            mergeObject(value);
                         });
                     } else {
-                        megeObject(obj);
+                        mergeObject(obj);
                     }
                 }
             });
         }
     }
+
+    Utils.findAndMergeRelationShipEntity = function(options) {
+        var attributeObject = options.attributeObject,
+            relationshipAttributes = options.relationshipAttributes;
+        _.each(attributeObject, function(val, key) {
+            var attributVal = val;
+            if (relationshipAttributes && relationshipAttributes[key]) {
+                var relationShipVal = relationshipAttributes[key];
+                if (_.isObject(val)) {
+                    if (_.isArray(val)) {
+                        _.each(val, function(attr) {
+                            if (attr && attr.attributes === undefined) {
+                                var entityFound = _.find(relationShipVal, { guid: attr.guid });
+                                if (entityFound) {
+                                    attr.attributes = _.omit(entityFound, 'typeName', 'guid', 'entityStatus');
+                                    attr.status = entityFound.entityStatus;
+                                }
+                            }
+                        });
+                    } else if (relationShipVal && val.attributes === undefined) {
+                        val.attributes = _.omit(relationShipVal, 'typeName', 'guid', 'entityStatus');
+                        val.status = relationShipVal.entityStatus;
+                    }
+                }
+            }
+        })
+    }
+
     Utils.getNestedSuperTypes = function(options) {
         var data = options.data,
             collection = options.collection,
@@ -608,42 +627,97 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
         return _.uniq(superTypes);
     }
     Utils.getNestedSuperTypeObj = function(options) {
-        var flag = 0,
-            data = options.data,
-            collection = options.collection;
-        if (options.attrMerge) {
-            var attributeDefs = [];
-        } else {
-            var attributeDefs = {};
+        var mainData = options.data,
+            collection = options.collection,
+            attrMerge = options.attrMerge,
+            seperateRelatioshipAttr = options.seperateRelatioshipAttr || false,
+            mergeRelationAttributes = options.mergeRelationAttributes || (seperateRelatioshipAttr ? false : true);
+
+        if (mergeRelationAttributes && seperateRelatioshipAttr) {
+            throw "Both mergeRelationAttributes & seperateRelatioshipAttr cannot be true!"
+        }
+        var attributeDefs = {};
+        if (attrMerge && !seperateRelatioshipAttr) {
+            attributeDefs = [];
+        } else if (options.attrMerge && seperateRelatioshipAttr) {
+            attributeDefs = {
+                attributeDefs: [],
+                relationshipAttributeDefs: []
+            }
+        }
+        var getRelationshipAttributeDef = function(data) {
+            return _.filter(
+                data.relationshipAttributeDefs,
+                function(obj, key) {
+                    return obj;
+                })
         }
         var getData = function(data, collection) {
             if (options.attrMerge) {
-                attributeDefs = attributeDefs.concat(data.attributeDefs);
+                if (seperateRelatioshipAttr) {
+                    attributeDefs.attributeDefs = attributeDefs.attributeDefs.concat(data.attributeDefs);
+                    attributeDefs.relationshipAttributeDefs = attributeDefs.relationshipAttributeDefs.concat(getRelationshipAttributeDef(data));
+                } else {
+                    attributeDefs = attributeDefs.concat(data.attributeDefs);
+                    if (mergeRelationAttributes) {
+                        attributeDefs = attributeDefs.concat(getRelationshipAttributeDef(data));
+                    }
+                }
             } else {
                 if (attributeDefs[data.name]) {
-                    if (_.isArray(attributeDefs[data.name])) {
-                        attributeDefs[data.name] = attributeDefs[data.name].concat(data.attributeDefs);
-                    } else {
-                        _.extend(attributeDefs[data.name], data.attributeDefs);
-                    }
-
+                    attributeDefs[data.name] = _.toArrayifObject(attributeDefs[data.name]).concat(data.attributeDefs);
                 } else {
-                    attributeDefs[data.name] = data.attributeDefs;
+                    if (seperateRelatioshipAttr) {
+                        attributeDefs[data.name] = {
+                            attributeDefs: data.attributeDefs,
+                            relationshipAttributeDefs: data.relationshipAttributeDefs
+                        };
+                    } else {
+                        attributeDefs[data.name] = data.attributeDefs;
+                        if (mergeRelationAttributes) {
+                            attributeDefs[data.name] = _.toArrayifObject(attributeDefs[data.name]).concat(getRelationshipAttributeDef(data));
+                        }
+                    }
                 }
             }
             if (data.superTypes && data.superTypes.length) {
                 _.each(data.superTypes, function(superTypeName) {
                     if (collection.fullCollection) {
-                        var collectionData = collection.fullCollection.findWhere({ name: superTypeName }).toJSON();
+                        var collectionData = collection.fullCollection.findWhere({ name: superTypeName });
                     } else {
-                        var collectionData = collection.findWhere({ name: superTypeName }).toJSON();
+                        var collectionData = collection.findWhere({ name: superTypeName });
+                    }
+                    collectionData = collectionData && collectionData.toJSON ? collectionData.toJSON() : collectionData;
+                    if (collectionData) {
+                        return getData(collectionData, collection);
+                    } else {
+                        return;
                     }
-                    return getData(collectionData, collection);
                 });
             }
         }
-        getData(data, collection);
-        return attributeDefs
+        getData(mainData, collection);
+        if (attrMerge) {
+            if (seperateRelatioshipAttr) {
+                attributeDefs = {
+                    attributeDefs: _.uniq(_.sortBy(attributeDefs.attributeDefs, 'name'), true, function(obj) {
+                        return obj.name
+                    }),
+                    relationshipAttributeDefs: _.uniq(_.sortBy(attributeDefs.relationshipAttributeDefs, 'name'), true, function(obj) {
+                        return (obj.name + obj.relationshipTypeName)
+                    })
+                }
+            } else {
+                attributeDefs = _.uniq(_.sortBy(attributeDefs, 'name'), true, function(obj) {
+                    if (obj.relationshipTypeName) {
+                        return (obj.name + obj.relationshipTypeName)
+                    } else {
+                        return (obj.name)
+                    }
+                });
+            }
+        }
+        return attributeDefs;
     }
 
     Utils.getProfileTabType = function(profileData, skipData) {
@@ -733,6 +807,7 @@ define(['require', 'utils/Globals', 'pnotify', 'utils/Messages', 'utils/Enums',
             }
         }
     }
+
     Utils.isUrl = function(url) {
         var regexp = /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/
         return regexp.test(url);
