@@ -774,18 +774,11 @@ define(['require',
                             clearTimeout(waitForDoubleClick)
                             waitForDoubleClick = null;
                             tooltip.hide(d);
-                            if (that.guid == d) {
-                                Utils.notifyInfo({
-                                    html: true,
-                                    content: "You are already on " + "<b>" + that.entityName + "</b> detail page."
-                                });
-                            } else {
-                                Utils.setUrl({
-                                    url: '#!/detailPage/' + d + '?tabActive=lineage',
-                                    mergeBrowserUrl: false,
-                                    trigger: true
-                                });
-                            }
+                            Utils.setUrl({
+                                url: '#!/detailPage/' + d + '?tabActive=lineage',
+                                mergeBrowserUrl: false,
+                                trigger: true
+                            });
                         } else {
                             var currentEvent = d3.event
                             waitForDoubleClick = setTimeout(function() {
@@ -794,10 +787,25 @@ define(['require',
                                 $(el).find('circle').addClass('node-detail-highlight');
                                 that.updateRelationshipDetails({ guid: d });
                                 waitForDoubleClick = null;
-                            }, 170)
+                            }, 150)
                         }
                     });
 
+                svgGroup.selectAll("g.edgePath path.path").on('click', function(d) {
+                    var data = { obj: _.find(that.lineageData.relations, { "fromEntityId": d.v, "toEntityId": d.w }) };
+                    if (data.obj) {
+                        var relationshipId = data.obj.relationshipId;
+                        require(['views/graph/PropagationPropertyModal'], function(PropagationPropertyModal) {
+                            var view = new PropagationPropertyModal({
+                                edgeInfo: data,
+                                relationshipId: relationshipId,
+                                lineageData: that.lineageData,
+                                apiGuid: that.apiGuid,
+                                detailPageFetchCollection: that.fetchCollection
+                            });
+                        });
+                    }
+                })
                 $('body').on('mouseover', '.d3-tip', function(el) {
                     that.activeTip = true;
                 });
@@ -1003,7 +1011,7 @@ define(['require',
                         try {
                             var a = document.createElement("a"),
                                 entityAttributes = that.entity && that.entity.attributes;
-                            a.download = ((entityAttributes && (entityAttributes.qualifiedName || entityAttributes.name) || "lineage_export") + ".png");
+                            a.download = ((entityAttributes && entityAttributes.qualifiedName) || "lineage_export") + ".png";
                             document.body.appendChild(a);
                             ctx.drawImage(img, 50, 50, canvas.width, canvas.height);
                             canvas.toBlob(function(blob) {
