@@ -51,10 +51,9 @@ define(['require',
             /** ui selector cache */
             ui: {
                 tagClick: '[data-id="tagClick"]',
+                termClick: '[data-id="termClick"]',
                 addTag: '[data-id="addTag"]',
                 addTerm: '[data-id="addTerm"]',
-                showMoreLess: '[data-id="showMoreLess"]',
-                showMoreLessTerm: '[data-id="showMoreLessTerm"]',
                 paginationDiv: '[data-id="paginationDiv"]',
                 previousData: "[data-id='previousData']",
                 nextData: "[data-id='nextData']",
@@ -76,8 +75,8 @@ define(['require',
                 return {
                     entityCreate: Globals.entityCreate,
                     searchType: this.searchType,
-                    taxonomy: Globals.taxonomy,
                     fromView: this.fromView,
+                    isGlossaryView: this.fromView == "glossary",
                     isSearchTab: Utils.getUrlState.isSearchTab()
                 };
             },
@@ -90,24 +89,27 @@ define(['require',
                     if (e.target.nodeName.toLocaleLowerCase() == "i") {
                         this.onClickTagCross(e);
                     } else {
-                        if (scope.hasClass('term')) {
-                            var url = scope.data('href').split(".").join("/terms/");
-                            this.triggerUrl({
-                                url: '#!/taxonomy/detailCatalog' + UrlLinks.taxonomiesApiUrl() + '/' + url,
-                                urlParams: null,
-                                mergeBrowserUrl: false,
-                                trigger: true,
-                                updateTabState: null
-                            });
-                        } else {
-                            this.triggerUrl({
-                                url: '#!/tag/tagAttribute/' + scope.text(),
-                                urlParams: null,
-                                mergeBrowserUrl: false,
-                                trigger: true,
-                                updateTabState: null
-                            });
-                        }
+                        this.triggerUrl({
+                            url: '#!/tag/tagAttribute/' + scope.text(),
+                            urlParams: null,
+                            mergeBrowserUrl: false,
+                            trigger: true,
+                            updateTabState: null
+                        });
+                    }
+                };
+                events["click " + this.ui.termClick] = function(e) {
+                    var scope = $(e.currentTarget);
+                    if (e.target.nodeName.toLocaleLowerCase() == "i") {
+                        this.onClickTermCross(e);
+                    } else {
+                        this.triggerUrl({
+                            url: '#!/glossary/' + scope.find('i').data('termguid'),
+                            urlParams: { gType: "term", viewType: "term", fromView: "entity" },
+                            mergeBrowserUrl: false,
+                            trigger: true,
+                            updateTabState: null
+                        });
                     }
                 };
                 events["keyup " + this.ui.gotoPage] = function(e) {
@@ -127,18 +129,8 @@ define(['require',
                 events["change " + this.ui.showPage] = 'changePageLimit';
                 events["click " + this.ui.gotoPagebtn] = 'gotoPagebtn';
                 events["click " + this.ui.addTag] = 'checkedValue';
-                events["click " + this.ui.addTerm] = 'checkedValue';
+                events["click " + this.ui.addTerm] = 'onClickAddTermBtn';
                 events["click " + this.ui.addAssignTag] = 'checkedValue';
-                events["click " + this.ui.showMoreLessTerm] = function(e) {
-                    e.stopPropagation();
-                    $(e.currentTarget).find('i').toggleClass('fa fa-angle-right fa fa-angle-up');
-                    $(e.currentTarget).parents('.searchTerm').find('div.termTableBreadcrumb>div.showHideDiv').toggleClass('hide');
-                    if ($(e.currentTarget).find('i').hasClass('fa-angle-right')) {
-                        $(e.currentTarget).find('span').text('Show More');
-                    } else {
-                        $(e.currentTarget).find('span').text('Show less');
-                    }
-                };
                 events["click " + this.ui.nextData] = "onClicknextData";
                 events["click " + this.ui.previousData] = "onClickpreviousData";
                 events["click " + this.ui.createEntity] = 'onClickCreateEntity';
@@ -152,14 +144,13 @@ define(['require',
              * @constructs
              */
             initialize: function(options) {
-                _.extend(this, _.pick(options, 'value', 'initialView', 'isTypeTagNotExists', 'classificationDefCollection', 'entityDefCollection', 'typeHeaders', 'searchVent', 'enumDefCollection', 'tagCollection', 'searchTableColumns', 'isDisable', 'fromView'));
+                _.extend(this, _.pick(options, 'value', 'guid', 'initialView', 'isTypeTagNotExists', 'classificationDefCollection', 'entityDefCollection', 'typeHeaders', 'searchVent', 'enumDefCollection', 'tagCollection', 'searchTableColumns', 'isDisable', 'fromView', 'glossaryCollection', 'termName'));
                 this.entityModel = new VEntity();
                 this.searchCollection = new VSearchList();
                 this.limit = 25;
                 this.asyncFetchCounter = 0;
                 this.offset = 0;
                 this.bindEvents();
-                this.bradCrumbList = [];
                 this.arr = [];
                 this.searchType = 'Basic Search';
                 this.columnOrder = null;
@@ -198,7 +189,6 @@ define(['require',
                     }
                     this.searchCollection.find(function(item) {
                         if (item.get('isEnable')) {
-                            var term = [];
                             var obj = item.toJSON();
                             that.arr.push({
                                 id: obj.guid,
@@ -208,14 +198,8 @@ define(['require',
                     });
 
                     if (this.arr.length > 0) {
-                        if (Globals.taxonomy) {
-                            this.$('.multiSelectTerm').show();
-                        }
                         this.$('.multiSelectTag').show();
                     } else {
-                        if (Globals.taxonomy) {
-                            this.$('.multiSelectTerm').hide();
-                        }
                         this.$('.multiSelectTag').hide();
                     }
                 });
@@ -235,7 +219,7 @@ define(['require',
                         this.updateColumnList(state);
                         var excludeDefaultColumn = [];
                         if (this.value && this.value.type) {
-                            excludeDefaultColumn = _.without(this.searchTableColumns[this.value.type], "selected", "name", "description", "typeName", "owner", "tag", "terms");
+                            excludeDefaultColumn = _.without(this.searchTableColumns[this.value.type], "selected", "name", "description", "typeName", "owner", "tag", "term");
                             if (this.searchTableColumns[this.value.type] === null) {
                                 this.ui.columnEmptyInfo.show();
                             } else {
@@ -374,7 +358,7 @@ define(['require',
                 }
 
                 if (isPostMethod && isSearchTab) {
-                    var excludeDefaultColumn = this.value.type && this.searchTableColumns ? _.without(this.searchTableColumns[this.value.type], "selected", "name", "description", "typeName", "owner", "tag", "terms") : null,
+                    var excludeDefaultColumn = this.value.type && this.searchTableColumns ? _.without(this.searchTableColumns[this.value.type], "selected", "name", "description", "typeName", "owner", "tag") : null,
                         filterObj = {
                             'entityFilters': entityFilters,
                             'tagFilters': tagFilters,
@@ -390,6 +374,9 @@ define(['require',
                     skipDefaultError: true,
                     sort: false,
                     success: function(dataOrCollection, response) {
+                        if (that.isDestroyed) {
+                            return;
+                        }
                         that.ui.gotoPage.val('');
                         that.ui.gotoPage.parent().removeClass('has-error');
                         that.ui.gotoPagebtn.prop("disabled", true);
@@ -431,7 +418,10 @@ define(['require',
                         }
                         if (isPostMethod) {
                             that.searchCollection.referredEntities = dataOrCollection.referredEntities;
-                            Utils.findAndMergeRefEntity(dataOrCollection.entities, dataOrCollection.referredEntities);
+                            Utils.findAndMergeRefEntity({
+                                attributeObject: dataOrCollection.entities,
+                                referredEntities: dataOrCollection.referredEntities
+                            });
                             that.searchCollection.reset(dataOrCollection.entities, { silent: true });
                         }
 
@@ -489,14 +479,15 @@ define(['require',
                     var checkBoxValue = {
                         'excludeDeletedEntities': (this.value.includeDE ? false : true),
                         'includeSubClassifications': (this.value.excludeSC ? false : true),
-                        'includeSubTypes': (this.value.excludeST ? false : true)
+                        'includeSubTypes': (this.value.excludeST ? false : true),
+                        'includeClassificationAttributes': true // server will return classication details with guid
                     }
                 }
                 if (value) {
                     if (value.searchType) {
                         this.searchCollection.url = UrlLinks.searchApiUrl(value.searchType);
                     }
-                    _.extend(this.searchCollection.queryParams, { 'limit': this.limit, 'offset': this.offset, 'query': _.trim(value.query), 'typeName': value.type || null, 'classification': value.tag || null });
+                    _.extend(this.searchCollection.queryParams, { 'limit': this.limit, 'offset': this.offset, 'query': _.trim(value.query), 'typeName': value.type || null, 'classification': value.tag || null, 'termName': value.term || null });
                     if (value.profileDBView && value.typeName && value.guid) {
                         var profileParam = {};
                         profileParam['guid'] = value.guid;
@@ -507,7 +498,7 @@ define(['require',
                     }
                     if (isPostMethod) {
                         this.searchCollection.filterObj = _.extend({}, filterObj);
-                        apiObj['data'] = _.extend(checkBoxValue, filterObj, _.pick(this.searchCollection.queryParams, 'query', 'excludeDeletedEntities', 'limit', 'offset', 'typeName', 'classification'))
+                        apiObj['data'] = _.extend(checkBoxValue, filterObj, _.pick(this.searchCollection.queryParams, 'query', 'excludeDeletedEntities', 'limit', 'offset', 'typeName', 'classification', 'termName'))
                         Globals.searchApiCallRef = this.searchCollection.getBasicRearchResult(apiObj);
                     } else {
                         apiObj.data = null;
@@ -519,7 +510,7 @@ define(['require',
                     }
                 } else {
                     if (isPostMethod) {
-                        apiObj['data'] = _.extend(checkBoxValue, filterObj, _.pick(this.searchCollection.queryParams, 'query', 'excludeDeletedEntities', 'limit', 'offset', 'typeName', 'classification'));
+                        apiObj['data'] = _.extend(checkBoxValue, filterObj, _.pick(this.searchCollection.queryParams, 'query', 'excludeDeletedEntities', 'limit', 'offset', 'typeName', 'classification', 'termName'));
                         Globals.searchApiCallRef = this.searchCollection.getBasicRearchResult(apiObj);
                     } else {
                         apiObj.data = null;
@@ -560,12 +551,14 @@ define(['require',
                             return this;
                         }
                     });
-                    that.bradCrumbList = [];
                     var columns = new columnCollection((that.searchCollection.dynamicTable ? that.getDaynamicColumns(that.searchCollection.toJSON()) : that.getFixedDslColumn()));
                     columns.setPositions().sort();
                     var table = new TableLayout(_.extend({}, that.commonTableOptions, {
                         columns: columns
                     }));
+                    if (!that.REntityTableLayoutView) {
+                        return;
+                    }
                     that.REntityTableLayoutView.show(table);
                     if (that.value.searchType !== "dsl") {
                         that.ui.containerCheckBox.show();
@@ -573,7 +566,6 @@ define(['require',
                         that.ui.containerCheckBox.hide();
                     }
                     that.$(".ellipsis .inputAssignTag").hide();
-                    that.renderBreadcrumb();
                     table.trigger("grid:refresh"); /*Event fire when table rendered*/
                     if (that.isDisable !== true) {
                         tableDragger(document.querySelector(".colSort")).on('drop', function(from, to, el) {
@@ -593,26 +585,17 @@ define(['require',
                 }
                 return obj;
             },
-            renderBreadcrumb: function() {
-                var that = this;
-                _.each(this.bradCrumbList, function(object) {
-                    _.each(object.value, function(subObj) {
-                        var scopeObject = that.$('[dataterm-id="' + object.scopeId + '"]').find('[dataterm-name="' + subObj.name + '"] .liContent');
-                        CommonViewFunction.breadcrumbMaker({ urlList: subObj.valueUrl, scope: scopeObject });
-                    });
-                });
-            },
             checkTableFetch: function() {
                 if (this.asyncFetchCounter <= 0) {
                     this.hideLoader();
                     Utils.generatePopover({
                         el: this.$('[data-id="showMoreLess"]'),
-                        contentClass: 'popover-tag',
+                        contentClass: 'popover-tag-term',
                         viewFixedPopover: true,
                         popoverOptions: {
                             container: null,
                             content: function() {
-                                return $(this).find('.popup-tag').children().clone();
+                                return $(this).find('.popup-tag-term').children().clone();
                             }
                         }
                     });
@@ -626,6 +609,7 @@ define(['require',
                 if (this.value && this.value.searchType === "basic" && this.searchTableColumns && (this.searchTableColumns[this.value.type] !== undefined)) {
                     columnToShow = this.searchTableColumns[this.value.type] == null ? [] : this.searchTableColumns[this.value.type];
                 }
+
                 col['Check'] = {
                     name: "selected",
                     label: "Select",
@@ -636,6 +620,7 @@ define(['require',
                     headerCell: "select-all"
                 };
 
+
                 col['name'] = {
                     label: this.value && this.value.profileDBView ? "Table Name" : "Name",
                     cell: "html",
@@ -651,7 +636,11 @@ define(['require',
                                 nameHtml = "",
                                 name = Utils.getName(obj);
                             if (obj.guid) {
-                                nameHtml = '<a title="' + name + '" href="#!/detailPage/' + obj.guid + (that.fromView ? "?from=" + that.fromView : "") + '">' + name + '</a>';
+                                if (obj.guid == "-1") {
+                                    nameHtml = '<span title="' + name + '">' + name + '</span>';
+                                } else {
+                                    nameHtml = '<a title="' + name + '" href="#!/detailPage/' + obj.guid + (that.fromView ? "?from=" + that.fromView : "") + '">' + name + '</a>';
+                                }
                             } else {
                                 nameHtml = '<span title="' + name + '">' + name + '</span>';
                             }
@@ -663,6 +652,7 @@ define(['require',
                         }
                     })
                 };
+
                 col['owner'] = {
                     label: "Owner",
                     cell: "String",
@@ -680,6 +670,8 @@ define(['require',
                         }
                     })
                 };
+
+
                 if (this.value && this.value.profileDBView) {
                     col['createTime'] = {
                         label: "Date Created",
@@ -699,6 +691,7 @@ define(['require',
                     }
                 }
                 if (this.value && !this.value.profileDBView) {
+
                     col['description'] = {
                         label: "Description",
                         cell: "String",
@@ -716,6 +709,8 @@ define(['require',
                             }
                         })
                     };
+
+
                     col['typeName'] = {
                         label: "Type",
                         cell: "Html",
@@ -733,7 +728,10 @@ define(['require',
                             }
                         })
                     };
-                    this.getTagTermCol({ 'col': col, 'columnToShow': columnToShow });
+                    this.getTagCol({ 'col': col, 'columnToShow': columnToShow });
+                    if ((!_.contains(["glossary"], this.fromView))) {
+                        this.getTermCol({ 'col': col, 'columnToShow': columnToShow });
+                    }
 
                     if (this.value && this.value.searchType === "basic") {
                         var def = this.entityDefCollection.fullCollection.find({ name: this.value.type });
@@ -798,7 +796,7 @@ define(['require',
                                         var nameHtml = "",
                                             name = modelObj[key];
                                         if (modelObj.guid) {
-                                            nameHtml = '<a title="' + name + '" href="#!/detailPage/' + modelObj.guid + '">' + name + '</a>';
+                                            nameHtml = '<a title="' + name + '" href="#!/detailPage/' + modelObj.guid + (that.fromView ? "?from=" + that.fromView : "") + '">' + name + '</a>';
                                         } else {
                                             nameHtml = '<span title="' + name + '">' + name + '</span>';
                                         }
@@ -824,13 +822,13 @@ define(['require',
                 }
                 return this.searchCollection.constructor.getTableCols(col, this.searchCollection);
             },
-            getTagTermCol: function(options) {
+            getTagCol: function(options) {
                 var that = this,
                     columnToShow = options.columnToShow,
                     col = options.col;
                 if (col) {
                     col['tag'] = {
-                        label: "Tags",
+                        label: "Classifications",
                         cell: "Html",
                         editable: false,
                         sortable: false,
@@ -841,6 +839,9 @@ define(['require',
                         formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
                             fromRaw: function(rawValue, model) {
                                 var obj = model.toJSON();
+                                if (obj.guid == "-1") {
+                                    return
+                                }
                                 if (obj.status && Enums.entityStateReadOnly[obj.status]) {
                                     return '<div class="readOnly">' + CommonViewFunction.tagForTable(obj); + '</div>';
                                 } else {
@@ -850,37 +851,43 @@ define(['require',
                             }
                         })
                     };
-                    if (Globals.taxonomy) {
-                        col['terms'] = {
-                            label: "Terms",
-                            cell: "Html",
-                            editable: false,
-                            sortable: false,
-                            resizeable: true,
-                            orderable: true,
-                            renderable: (columnToShow ? _.contains(columnToShow, 'terms') : true),
-                            className: 'searchTerm',
-                            formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
-                                fromRaw: function(rawValue, model) {
-                                    var obj = model.toJSON();
-                                    var returnObject = CommonViewFunction.termTableBreadcrumbMaker(obj);
-                                    if (returnObject.object) {
-                                        that.bradCrumbList.push(returnObject.object);
-                                    }
+                }
+            },
+            getTermCol: function(options) {
+                var that = this,
+                    columnToShow = options.columnToShow,
+                    col = options.col;
+                if (col) {
+                    col['term'] = {
+                        label: "Term",
+                        cell: "Html",
+                        editable: false,
+                        sortable: false,
+                        resizeable: true,
+                        orderable: true,
+                        renderable: (columnToShow ? _.contains(columnToShow, 'term') : true),
+                        className: 'searchTag',
+                        formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
+                            fromRaw: function(rawValue, model) {
+                                var obj = model.toJSON();
+                                if (obj.guid == "-1") {
+                                    return
+                                }
+                                if (obj.typeName && !(_.startsWith(obj.typeName, "AtlasGlossary"))) {
                                     if (obj.status && Enums.entityStateReadOnly[obj.status]) {
-                                        return '<div class="readOnly">' + returnObject.html + '</div>';
+                                        return '<div class="readOnly">' + CommonViewFunction.termForTable(obj); + '</div>';
                                     } else {
-                                        return returnObject.html;
+                                        return CommonViewFunction.termForTable(obj);
                                     }
                                 }
-                            })
-                        };
-                    }
+                            }
+                        })
+                    };
                 }
             },
             addTagModalView: function(guid, multiple) {
                 var that = this;
-                require(['views/tag/addTagModalView'], function(AddTagModalView) {
+                require(['views/tag/AddTagModalView'], function(AddTagModalView) {
                     var view = new AddTagModalView({
                         guid: guid,
                         multiple: multiple,
@@ -910,7 +917,11 @@ define(['require',
                     } else {
                         return [];
                     }
-                    return obj.classificationNames;
+                    return _.compact(_.map(obj.classifications, function(val) {
+                        if (val.entityGuid == guid) {
+                            return val.typeName
+                        }
+                    }));
                 } else {
                     return [];
                 }
@@ -927,84 +938,74 @@ define(['require',
             checkedValue: function(e) {
                 var guid = "",
                     that = this,
-                    isTagMultiSelect = $(e.currentTarget).hasClass('multiSelectTag'),
-                    isTermMultiSelect = $(e.currentTarget).hasClass('multiSelectTerm'),
-                    isTagButton = $(e.currentTarget).hasClass('assignTag');
-                if (isTagButton) {
-                    if (isTagMultiSelect && this.arr && this.arr.length) {
-                        that.addTagModalView(guid, this.arr);
-                    } else {
-                        guid = that.$(e.currentTarget).data("guid");
-                        that.addTagModalView(guid);
-                    }
+                    isTagMultiSelect = $(e.currentTarget).hasClass('multiSelectTag');
+                if (isTagMultiSelect && this.arr && this.arr.length) {
+                    that.addTagModalView(guid, this.arr);
                 } else {
-                    if (isTermMultiSelect && this.arr && this.arr.length) {
-                        that.addTermModalView(guid, this.arr);
-                    } else {
-                        guid = that.$(e.currentTarget).data("guid");
-                        that.addTermModalView(guid);
-                    }
+                    guid = that.$(e.currentTarget).data("guid");
+                    that.addTagModalView(guid);
                 }
             },
-            addTermModalView: function(guid, multiple) {
-                var that = this;
-                require([
-                    'views/business_catalog/AddTermToEntityLayoutView',
-                ], function(AddTermToEntityLayoutView) {
-                    var view = new AddTermToEntityLayoutView({
-                        guid: guid,
-                        multiple: multiple,
+            onClickAddTermBtn: function(e) {
+                var that = this,
+                    entityGuid = $(e.currentTarget).data("guid"),
+                    associatedTerms = this.searchCollection.find({ guid: entityGuid }).get('meanings');
+                require(['views/glossary/AssignTermLayoutView'], function(AssignTermLayoutView) {
+                    var view = new AssignTermLayoutView({
+                        guid: entityGuid,
+                        associatedTerms: associatedTerms,
                         callback: function() {
                             that.fetchCollection();
-                            that.arr = [];
                         },
-                        showLoader: that.showLoader.bind(that),
-                        hideLoader: that.hideLoader.bind(that)
+                        glossaryCollection: that.glossaryCollection,
                     });
                 });
             },
             onClickTagCross: function(e) {
-                var tagName = $(e.target).data("name"),
-                    guid = $(e.target).data("guid"),
-                    assetName = $(e.target).data("assetname"),
-                    tagOrTerm = $(e.target).data("type"),
-                    that = this;
-                if (tagOrTerm === "term") {
-                    var modal = CommonViewFunction.deleteTagModel({
-                        msg: "<div class='ellipsis'>Remove: " + "<b>" + _.escape(tagName) + "</b> assignment from" + " " + "<b>" + assetName + " ?</b></div>",
-                        titleMessage: Messages.removeTerm,
-                        buttonText: "Remove"
-                    });
-                } else if (tagOrTerm === "tag") {
-                    var modal = CommonViewFunction.deleteTagModel({
-                        msg: "<div class='ellipsis'>Remove: " + "<b>" + _.escape(tagName) + "</b> assignment from" + " " + "<b>" + assetName + " ?</b></div>",
-                        titleMessage: Messages.removeTag,
-                        buttonText: "Remove"
-                    });
-                }
-                if (modal) {
-                    modal.on('ok', function() {
-                        that.deleteTagData(e, tagOrTerm);
-                    });
-                    modal.on('closeModal', function() {
-                        modal.trigger('cancel');
-                    });
-                }
-            },
-            deleteTagData: function(e, tagOrTerm) {
                 var that = this,
                     tagName = $(e.target).data("name"),
-                    guid = $(e.target).data("guid");
+                    guid = $(e.target).data("guid"),
+                    entityGuid = $(e.target).data("entityguid"),
+                    assetName = $(e.target).data("assetname");
                 CommonViewFunction.deleteTag({
-                    'tagName': tagName,
-                    'guid': guid,
-                    'tagOrTerm': tagOrTerm,
+                    tagName: tagName,
+                    guid: guid,
+                    associatedGuid: guid != entityGuid ? entityGuid : null,
+                    msg: "<div class='ellipsis'>Remove: " + "<b>" + _.escape(tagName) + "</b> assignment from" + " " + "<b>" + assetName + " ?</b></div>",
+                    titleMessage: Messages.removeTag,
+                    okText: "Remove",
                     showLoader: that.showLoader.bind(that),
                     hideLoader: that.hideLoader.bind(that),
                     callback: function() {
                         that.fetchCollection();
                     }
                 });
+
+            },
+            onClickTermCross: function(e) {
+                var $el = $(e.target),
+                    termGuid = $el.data('termguid'),
+                    guid = $el.data('guid'),
+                    termName = $(e.currentTarget).text(),
+                    assetname = $el.data('assetname'),
+                    meanings = this.searchCollection.find({ "guid": guid }).get("meanings"),
+                    that = this,
+                    termObj = _.find(meanings, { termGuid: termGuid });
+                CommonViewFunction.removeCategoryTermAssociation({
+                    termGuid: termGuid,
+                    model: {
+                        guid: guid,
+                        relationshipGuid: termObj.relationGuid
+                    },
+                    collection: that.glossaryCollection,
+                    msg: "<div class='ellipsis'>Remove: " + "<b>" + _.escape(termName) + "</b> assignment from" + " " + "<b>" + assetname + "?</b></div>",
+                    titleMessage: Messages.glossary.removeTermfromEntity,
+                    isEntityView: true,
+                    buttonText: "Remove",
+                    callback: function() {
+                        that.fetchCollection();
+                    }
+                });
             },
             onClicknextData: function() {
                 this.offset = this.offset + this.limit;
