@@ -19,8 +19,10 @@ package org.apache.atlas;
 
 import org.apache.atlas.security.InMemoryJAASConfiguration;
 import org.apache.commons.configuration.Configuration;
+import org.apache.commons.configuration.ConfigurationConverter;
 import org.apache.commons.configuration.ConfigurationException;
 import org.apache.commons.configuration.PropertiesConfiguration;
+import org.apache.commons.lang.StringUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -29,17 +31,24 @@ import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.InputStream;
 import java.net.URL;
+import java.util.AbstractMap.SimpleEntry;
 import java.util.Iterator;
+import java.util.Properties;
 
 /**
  * Application properties used by Atlas.
  */
 public final class ApplicationProperties extends PropertiesConfiguration {
     public static final String ATLAS_CONFIGURATION_DIRECTORY_PROPERTY = "atlas.conf";
-
     private static final Logger LOG = LoggerFactory.getLogger(ApplicationProperties.class);
 
-    public static final String APPLICATION_PROPERTIES = "atlas-application.properties";
+    public static final String  APPLICATION_PROPERTIES     = "atlas-application.properties";
+
+    public static final SimpleEntry<String, String> DB_CACHE_CONF               = new SimpleEntry<>("atlas.graph.cache.db-cache", "true");
+    public static final SimpleEntry<String, String> DB_CACHE_CLEAN_WAIT_CONF    = new SimpleEntry<>("atlas.graph.cache.db-cache-clean-wait", "20");
+    public static final SimpleEntry<String, String> DB_CACHE_SIZE_CONF          = new SimpleEntry<>("atlas.graph.cache.db-cache-size", "0.5");
+    public static final SimpleEntry<String, String> DB_TX_CACHE_SIZE_CONF       = new SimpleEntry<>("atlas.graph.cache.tx-cache-size", "15000");
+    public static final SimpleEntry<String, String> DB_CACHE_TX_DIRTY_SIZE_CONF = new SimpleEntry<>("atlas.graph.cache.tx-dirty-size", "120");
 
     private static volatile Configuration instance = null;
 
@@ -100,7 +109,12 @@ public final class ApplicationProperties extends PropertiesConfiguration {
 
             LOG.info("Loading {} from {}", fileName, url);
 
-            Configuration configuration = new ApplicationProperties(url).interpolatedConfiguration();
+            ApplicationProperties appProperties = new ApplicationProperties(url);
+
+            appProperties.setDefaults();
+
+            Configuration configuration = appProperties.interpolatedConfiguration();
+
             logConfiguration(configuration);
             return configuration;
         } catch (Exception e) {
@@ -123,6 +137,13 @@ public final class ApplicationProperties extends PropertiesConfiguration {
         return inConf.subset(prefix);
     }
 
+    public static Properties getSubsetAsProperties(Configuration inConf, String prefix) {
+        Configuration subset = inConf.subset(prefix);
+        Properties   ret     = ConfigurationConverter.getProperties(subset);
+
+        return ret;
+    }
+
     public static Class getClass(Configuration configuration, String propertyName, String defaultValue,
                                  Class assignableClass) throws AtlasException {
         try {
@@ -141,6 +162,21 @@ public final class ApplicationProperties extends PropertiesConfiguration {
         }
     }
 
+    public static Class getClass(String fullyQualifiedClassName, Class assignableClass) throws AtlasException {
+        try {
+            Class<?> clazz = Class.forName(fullyQualifiedClassName);
+            if (assignableClass == null || assignableClass.isAssignableFrom(clazz)) {
+                return clazz;
+            } else {
+                String message = "Class " + clazz.getName() + " is not assignable to class " + assignableClass.getName();
+                LOG.error(message);
+                throw new AtlasException(message);
+            }
+        } catch (Exception e) {
+            throw new AtlasException(e);
+        }
+    }
+
     /**
      * Get the specified property as an {@link InputStream}.
      * If the property is not set, then the specified default filename
@@ -157,57 +193,83 @@ public final class ApplicationProperties extends PropertiesConfiguration {
      * @throws AtlasException if no file was found or if there was an error loading the file
      */
     public static InputStream getFileAsInputStream(Configuration configuration, String propertyName, String defaultFileName) throws AtlasException {
-        File fileToLoad = null;
-        String fileName = configuration.getString(propertyName);
+        File   fileToLoad = null;
+        String fileName   = configuration.getString(propertyName);
+
         if (fileName == null) {
             if (defaultFileName == null) {
                 throw new AtlasException(propertyName + " property not set and no default value specified");
             }
+
+            LOG.info("{} property not set; defaulting to {}", propertyName, defaultFileName);
+
             fileName = defaultFileName;
+
             String atlasConfDir = System.getProperty(ATLAS_CONFIGURATION_DIRECTORY_PROPERTY);
+
             if (atlasConfDir != null) {
                 // Look for default filename in Atlas config directory
                 fileToLoad = new File(atlasConfDir, fileName);
-            }
-            else {
+            } else {
                 // Look for default filename under the working directory
                 fileToLoad = new File(fileName);
             }
-            if (LOG.isDebugEnabled()) {
-                LOG.debug("{} property not set - defaulting to {}", propertyName, fileToLoad.getPath());
-            }
-        }
-        else {
+        } else {
             // Look for configured filename
             fileToLoad = new File(fileName);
-            if (LOG.isDebugEnabled()) {
-                LOG.debug("Using {} property setting: {}", propertyName, fileToLoad.getPath());
-            }
         }
 
         InputStream inStr = null;
+
         if (fileToLoad.exists()) {
             try {
+                LOG.info("Loading file {} from {}", fileName, fileToLoad.getPath());
+
                 inStr = new FileInputStream(fileToLoad);
             } catch (FileNotFoundException e) {
                 throw new AtlasException("Error loading file " + fileName, e);
             }
-            if (LOG.isDebugEnabled()) {
-                LOG.debug("Loaded file from : {}", fileToLoad.getPath());
-            }
-        }
-        else {
+        } else {
             // Look for file as class loader resource
             inStr = Thread.currentThread().getContextClassLoader().getResourceAsStream(fileName);
+
             if (inStr == null) {
                 String msg = fileName + " not found in file system or as class loader resource";
+
                 LOG.error(msg);
+
                 throw new AtlasException(msg);
             }
-            if (LOG.isDebugEnabled()) {
-                LOG.debug("Loaded {} as resource from : {}", fileName, Thread.currentThread().getContextClassLoader().getResource(fileName).toString());
-            }
+
+            LOG.info("Loaded {} as resource from {}", fileName, Thread.currentThread().getContextClassLoader().getResource(fileName).toString());
         }
+
         return inStr;
     }
+
+    private void setDefaults() {
+        setDbCacheConfDefaults();
+    }
+
+    void setDefault(SimpleEntry<String, String> keyValueDefault, String currentValue) {
+        if (StringUtils.isNotEmpty(currentValue)) {
+            return;
+        }
+
+        clearPropertyDirect(keyValueDefault.getKey());
+        addPropertyDirect(keyValueDefault.getKey(), keyValueDefault.getValue());
+        LOG.info("Property (set to default) {} = {}", keyValueDefault.getKey(), keyValueDefault.getValue());
+    }
+
+    private void setDbCacheConfDefaults() {
+        SimpleEntry<String, String> keyValues[] = new SimpleEntry[]{ DB_CACHE_CONF, DB_CACHE_CLEAN_WAIT_CONF,
+                                                                     DB_CACHE_SIZE_CONF, DB_TX_CACHE_SIZE_CONF,
+                                                                     DB_CACHE_TX_DIRTY_SIZE_CONF };
+
+        for(SimpleEntry<String, String> kv : keyValues) {
+            String currentValue = getString(kv.getKey());
+
+            setDefault(kv, currentValue);
+        }
+    }
 }
