@@ -23,6 +23,7 @@ import org.apache.atlas.model.typedef.AtlasBaseTypeDef;
 import org.apache.atlas.model.typedef.AtlasClassificationDef;
 import org.apache.atlas.model.typedef.AtlasEntityDef;
 import org.apache.atlas.model.typedef.AtlasEnumDef;
+import org.apache.atlas.model.typedef.AtlasRelationshipDef;
 import org.apache.atlas.model.typedef.AtlasStructDef;
 import org.apache.atlas.model.typedef.AtlasTypesDef;
 import org.apache.commons.collections.CollectionUtils;
@@ -35,14 +36,15 @@ import javax.inject.Singleton;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.locks.ReentrantLock;
 
 import static org.apache.atlas.model.typedef.AtlasBaseTypeDef.*;
-
 /**
  * registry for all types defined in Atlas.
  */
@@ -54,22 +56,28 @@ public class AtlasTypeRegistry {
 
     protected       RegistryData                   registryData;
     private   final TypeRegistryUpdateSynchronizer updateSynchronizer;
+    private   final Set<String>                    missingRelationshipDefs;
+
 
     public AtlasTypeRegistry() {
-        registryData       = new RegistryData();
-        updateSynchronizer = new TypeRegistryUpdateSynchronizer(this);
+        registryData            = new RegistryData();
+        updateSynchronizer      = new TypeRegistryUpdateSynchronizer(this);
+        missingRelationshipDefs = new HashSet<>();
     }
 
     // used only by AtlasTransientTypeRegistry
     protected AtlasTypeRegistry(AtlasTypeRegistry other) {
-        registryData       = new RegistryData();
-        updateSynchronizer = other.updateSynchronizer;
+        registryData            = new RegistryData();
+        updateSynchronizer      = other.updateSynchronizer;
+        missingRelationshipDefs = other.missingRelationshipDefs;
     }
 
     public Collection<String> getAllTypeNames() { return registryData.allTypes.getAllTypeNames(); }
 
     public Collection<AtlasType> getAllTypes() { return registryData.allTypes.getAllTypes(); }
 
+    public Set<String> getAllServiceTypes() { return registryData.allTypes.getAllServiceTypes(); }
+
     public boolean isRegisteredType(String typeName) {
         return registryData.allTypes.isKnownType(typeName);
     }
@@ -108,12 +116,14 @@ public class AtlasTypeRegistry {
         return ret;
     }
 
-    public AtlasType getTypeByGuid(String guid) {
+    public AtlasType getTypeByGuid(String guid) throws AtlasBaseException {
         if (LOG.isDebugEnabled()) {
             LOG.debug("==> AtlasTypeRegistry.getTypeByGuid({})", guid);
         }
 
         AtlasType ret = registryData.allTypes.getTypeByGuid(guid);
+        if(ret == null)
+            throw new AtlasBaseException(AtlasErrorCode.TYPE_GUID_NOT_FOUND, guid);
 
         if (LOG.isDebugEnabled()) {
             LOG.debug("<== AtlasTypeRegistry.getTypeByGuid({}): {}", guid, ret);
@@ -181,9 +191,9 @@ public class AtlasTypeRegistry {
         return registryData.classificationDefs.getTypeByName(name);
     }
 
+    public Collection<AtlasRelationshipDef> getAllRelationshipDefs() { return registryData.relationshipDefs.getAll(); }
 
     public Collection<AtlasEntityDef> getAllEntityDefs() { return registryData.entityDefs.getAll(); }
-
     public AtlasEntityDef getEntityDefByGuid(String guid) {
         return registryData.entityDefs.getTypeDefByGuid(guid);
     }
@@ -193,11 +203,20 @@ public class AtlasTypeRegistry {
     }
 
     public Collection<String> getAllEntityDefNames() { return registryData.entityDefs.getAllNames(); }
-
     public Collection<AtlasEntityType> getAllEntityTypes() { return registryData.entityDefs.getAllTypes(); }
-
     public AtlasEntityType getEntityTypeByName(String name) { return registryData.entityDefs.getTypeByName(name); }
+    /**
+     * @return relationshipTypes
+     */
+    public Collection<AtlasRelationshipType> getAllRelationshipTypes() { return registryData.relationshipDefs.getAllTypes(); }
 
+    public AtlasRelationshipDef getRelationshipDefByGuid(String guid) {
+        return registryData.relationshipDefs.getTypeDefByGuid(guid);
+    }
+    public AtlasRelationshipDef getRelationshipDefByName(String name) {
+        return registryData.relationshipDefs.getTypeDefByName(name);
+    }
+    public AtlasRelationshipType getRelationshipTypeByName(String name) { return registryData.relationshipDefs.getTypeByName(name); }
 
     public AtlasTransientTypeRegistry lockTypeRegistryForUpdate() throws AtlasBaseException {
         return lockTypeRegistryForUpdate(DEFAULT_LOCK_MAX_WAIT_TIME_IN_SECONDS);
@@ -211,6 +230,15 @@ public class AtlasTypeRegistry {
         updateSynchronizer.releaseTypeRegistryForUpdate(transientTypeRegistry, commitUpdates);
     }
 
+    public void reportMissingRelationshipDef(String entityType1, String entityType2, String attributeName) {
+        String key = entityType1 + "->" + entityType2 + ":" + attributeName;
+
+        if (!missingRelationshipDefs.contains(key)) {
+            LOG.warn("No RelationshipDef defined between {} and {} on attribute: {}.{}", entityType1, entityType2, entityType1, attributeName);
+
+            missingRelationshipDefs.add(key);
+        }
+    }
 
     static class RegistryData {
         final TypeCache                                                       allTypes;
@@ -218,6 +246,7 @@ public class AtlasTypeRegistry {
         final TypeDefCache<AtlasStructDef, AtlasStructType>                   structDefs;
         final TypeDefCache<AtlasClassificationDef, AtlasClassificationType>   classificationDefs;
         final TypeDefCache<AtlasEntityDef, AtlasEntityType>                   entityDefs;
+        final TypeDefCache<AtlasRelationshipDef, AtlasRelationshipType>       relationshipDefs;
         final TypeDefCache<? extends AtlasBaseTypeDef, ? extends AtlasType>[] allDefCaches;
 
         RegistryData() {
@@ -226,7 +255,8 @@ public class AtlasTypeRegistry {
             structDefs         = new TypeDefCache<>(allTypes);
             classificationDefs = new TypeDefCache<>(allTypes);
             entityDefs         = new TypeDefCache<>(allTypes);
-            allDefCaches       = new TypeDefCache[] { enumDefs, structDefs, classificationDefs, entityDefs };
+            relationshipDefs   = new TypeDefCache<>(allTypes);
+            allDefCaches       = new TypeDefCache[] { enumDefs, structDefs, classificationDefs, entityDefs, relationshipDefs };
 
             init();
         }
@@ -284,6 +314,7 @@ public class AtlasTypeRegistry {
                 structDefs.updateGuid(typeName, guid);
                 classificationDefs.updateGuid(typeName, guid);
                 entityDefs.updateGuid(typeName, guid);
+                relationshipDefs.updateGuid(typeName, guid);
             }
         }
 
@@ -293,6 +324,7 @@ public class AtlasTypeRegistry {
                 structDefs.removeTypeDefByGuid(guid);
                 classificationDefs.removeTypeDefByGuid(guid);
                 entityDefs.removeTypeDefByGuid(guid);
+                relationshipDefs.removeTypeDefByGuid(guid);
             }
         }
 
@@ -302,6 +334,7 @@ public class AtlasTypeRegistry {
                 structDefs.removeTypeDefByName(typeName);
                 classificationDefs.removeTypeDefByName(typeName);
                 entityDefs.removeTypeDefByName(typeName);
+                relationshipDefs.removeTypeDefByName(typeName);
             }
         }
 
@@ -311,6 +344,7 @@ public class AtlasTypeRegistry {
             structDefs.clear();
             classificationDefs.clear();
             entityDefs.clear();
+            relationshipDefs.clear();
 
             init();
         }
@@ -329,6 +363,7 @@ public class AtlasTypeRegistry {
             addTypesWithNoRefResolve(parent.getAllStructDefs());
             addTypesWithNoRefResolve(parent.getAllClassificationDefs());
             addTypesWithNoRefResolve(parent.getAllEntityDefs());
+            addTypesWithNoRefResolve(parent.getAllRelationshipDefs());
 
             addedTypes.clear();
             updatedTypes.clear();
@@ -407,6 +442,7 @@ public class AtlasTypeRegistry {
                 addTypesWithNoRefResolve(typesDef.getStructDefs());
                 addTypesWithNoRefResolve(typesDef.getClassificationDefs());
                 addTypesWithNoRefResolve(typesDef.getEntityDefs());
+                addTypesWithNoRefResolve(typesDef.getRelationshipDefs());
             }
 
             resolveReferences();
@@ -506,6 +542,7 @@ public class AtlasTypeRegistry {
                 updateTypesWithNoRefResolve(typesDef.getStructDefs());
                 updateTypesWithNoRefResolve(typesDef.getClassificationDefs());
                 updateTypesWithNoRefResolve(typesDef.getEntityDefs());
+                updateTypesWithNoRefResolve(typesDef.getRelationshipDefs());
             }
 
             if (LOG.isDebugEnabled()) {
@@ -519,6 +556,7 @@ public class AtlasTypeRegistry {
                 removeTypesWithNoRefResolve(typesDef.getStructDefs());
                 removeTypesWithNoRefResolve(typesDef.getClassificationDefs());
                 removeTypesWithNoRefResolve(typesDef.getEntityDefs());
+                removeTypesWithNoRefResolve(typesDef.getRelationshipDefs());
             }
 
             resolveReferences();
@@ -550,6 +588,9 @@ public class AtlasTypeRegistry {
                 case ENTITY:
                     registryData.entityDefs.removeTypeDefByName(typeDef.getName());
                     break;
+                case RELATIONSHIP:
+                    registryData.relationshipDefs.removeTypeDefByName(typeDef.getName());
+                    break;
             }
             deletedTypes.add(typeDef);
         }
@@ -568,6 +609,9 @@ public class AtlasTypeRegistry {
                 case ENTITY:
                     registryData.entityDefs.removeTypeDefByGuid(typeDef.getGuid());
                     break;
+                case RELATIONSHIP:
+                    registryData.relationshipDefs.removeTypeDefByGuid(typeDef.getGuid());
+                    break;
             }
             deletedTypes.add(typeDef);
         }
@@ -645,11 +689,15 @@ public class AtlasTypeRegistry {
                     AtlasClassificationDef classificationDef = (AtlasClassificationDef) typeDef;
 
                     registryData.classificationDefs.addType(classificationDef,
-                                                            new AtlasClassificationType(classificationDef));
+                            new AtlasClassificationType(classificationDef));
                 } else if (typeDef.getClass().equals(AtlasEntityDef.class)) {
                     AtlasEntityDef entityDef = (AtlasEntityDef) typeDef;
 
                     registryData.entityDefs.addType(entityDef, new AtlasEntityType(entityDef));
+                } else if (typeDef.getClass().equals(AtlasRelationshipDef.class)) {
+                    AtlasRelationshipDef relationshipDef = (AtlasRelationshipDef) typeDef;
+
+                    registryData.relationshipDefs.addType(relationshipDef, new AtlasRelationshipType(relationshipDef));
                 }
 
                 addedTypes.add(typeDef);
@@ -663,7 +711,7 @@ public class AtlasTypeRegistry {
         private void addTypesWithNoRefResolve(Collection<? extends AtlasBaseTypeDef> typeDefs) throws AtlasBaseException {
             if (LOG.isDebugEnabled()) {
                 LOG.debug("==> AtlasTypeRegistry.addTypesWithNoRefResolve(length={})",
-                          (typeDefs == null ? 0 : typeDefs.size()));
+                        (typeDefs == null ? 0 : typeDefs.size()));
             }
 
             if (CollectionUtils.isNotEmpty(typeDefs)) {
@@ -674,7 +722,7 @@ public class AtlasTypeRegistry {
 
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AtlasTypeRegistry.addTypesWithNoRefResolve(length={})",
-                          (typeDefs == null ? 0 : typeDefs.size()));
+                        (typeDefs == null ? 0 : typeDefs.size()));
             }
         }
 
@@ -718,12 +766,17 @@ public class AtlasTypeRegistry {
 
                     registryData.classificationDefs.removeTypeDefByGuid(guid);
                     registryData.classificationDefs.addType(classificationDef,
-                                                            new AtlasClassificationType(classificationDef));
+                            new AtlasClassificationType(classificationDef));
                 } else if (typeDef.getClass().equals(AtlasEntityDef.class)) {
                     AtlasEntityDef entityDef = (AtlasEntityDef) typeDef;
 
                     registryData.entityDefs.removeTypeDefByGuid(guid);
                     registryData.entityDefs.addType(entityDef, new AtlasEntityType(entityDef));
+                } else if (typeDef.getClass().equals(AtlasRelationshipDef.class)) {
+                    AtlasRelationshipDef relationshipDef = (AtlasRelationshipDef) typeDef;
+
+                    registryData.relationshipDefs.removeTypeDefByGuid(guid);
+                    registryData.relationshipDefs.addType(relationshipDef, new AtlasRelationshipType(relationshipDef));
                 }
 
                 updatedTypes.add(typeDef);
@@ -755,12 +808,17 @@ public class AtlasTypeRegistry {
 
                     registryData.classificationDefs.removeTypeDefByName(name);
                     registryData.classificationDefs.addType(classificationDef,
-                                                            new AtlasClassificationType(classificationDef));
+                            new AtlasClassificationType(classificationDef));
                 } else if (typeDef.getClass().equals(AtlasEntityDef.class)) {
                     AtlasEntityDef entityDef = (AtlasEntityDef) typeDef;
 
                     registryData.entityDefs.removeTypeDefByName(name);
                     registryData.entityDefs.addType(entityDef, new AtlasEntityType(entityDef));
+                } else if (typeDef.getClass().equals(AtlasRelationshipDef.class)) {
+                    AtlasRelationshipDef relationshipDef = (AtlasRelationshipDef) typeDef;
+
+                    registryData.relationshipDefs.removeTypeDefByName(name);
+                    registryData.relationshipDefs.addType(relationshipDef, new AtlasRelationshipType(relationshipDef));
                 }
 
                 updatedTypes.add(typeDef);
@@ -774,7 +832,7 @@ public class AtlasTypeRegistry {
         private void updateTypesWithNoRefResolve(Collection<? extends AtlasBaseTypeDef> typeDefs) {
             if (LOG.isDebugEnabled()) {
                 LOG.debug("==> AtlasTypeRegistry.updateTypesWithNoRefResolve(length={})",
-                                                                             (typeDefs == null ? 0 : typeDefs.size()));
+                        (typeDefs == null ? 0 : typeDefs.size()));
             }
 
             if (CollectionUtils.isNotEmpty(typeDefs)) {
@@ -785,7 +843,7 @@ public class AtlasTypeRegistry {
 
             if (LOG.isDebugEnabled()) {
                 LOG.debug("<== AtlasTypeRegistry.updateTypesWithNoRefResolve(length={})",
-                                                                              (typeDefs == null ? 0 : typeDefs.size()));
+                        (typeDefs == null ? 0 : typeDefs.size()));
             }
         }
     }
@@ -807,12 +865,12 @@ public class AtlasTypeRegistry {
             boolean alreadyLockedByCurrentThread = typeRegistryUpdateLock.isHeldByCurrentThread();
 
             if (!alreadyLockedByCurrentThread) {
-                if (LOG.isDebugEnabled()) {
-                    LOG.debug("lockTypeRegistryForUpdate(): waiting for lock to be released by thread {}", lockedByThread);
+                if (lockedByThread != null) {
+                    LOG.info("lockTypeRegistryForUpdate(): waiting for lock to be released by thread {}", lockedByThread);
                 }
             } else {
                 LOG.warn("lockTypeRegistryForUpdate(): already locked. currentLockCount={}",
-                         typeRegistryUpdateLock.getHoldCount());
+                        typeRegistryUpdateLock.getHoldCount());
             }
 
             try {
@@ -846,8 +904,8 @@ public class AtlasTypeRegistry {
                 try {
                     if (typeRegistryUnderUpdate != ttr) {
                         LOG.error("releaseTypeRegistryForUpdate(): incorrect typeRegistry returned for release" +
-                                  ": found=" + ttr + "; expected=" + typeRegistryUnderUpdate,
-                                  new Exception().fillInStackTrace());
+                                        ": found=" + ttr + "; expected=" + typeRegistryUnderUpdate,
+                                new Exception().fillInStackTrace());
                     } else if (typeRegistryUpdateLock.getHoldCount() == 1) {
                         if (ttr != null && commitUpdates) {
                             typeRegistry.registryData = ttr.registryData;
@@ -865,7 +923,7 @@ public class AtlasTypeRegistry {
                 }
             } else {
                 LOG.error("releaseTypeRegistryForUpdate(): current thread does not hold the lock",
-                          new Exception().fillInStackTrace());
+                        new Exception().fillInStackTrace());
             }
 
             LOG.debug("<== releaseTypeRegistryForUpdate()");
@@ -877,15 +935,18 @@ public class AtlasTypeRegistry {
 class TypeCache {
     private final Map<String, AtlasType> typeGuidMap;
     private final Map<String, AtlasType> typeNameMap;
+    private final Set<String>            serviceTypes;
 
     public TypeCache() {
-        typeGuidMap = new ConcurrentHashMap<>();
-        typeNameMap = new ConcurrentHashMap<>();
+        typeGuidMap  = new ConcurrentHashMap<>();
+        typeNameMap  = new ConcurrentHashMap<>();
+        serviceTypes = new HashSet<>();
     }
 
     public TypeCache(TypeCache other) {
-        typeGuidMap = new ConcurrentHashMap<>(other.typeGuidMap);
-        typeNameMap = new ConcurrentHashMap<>(other.typeNameMap);
+        typeGuidMap  = new ConcurrentHashMap<>(other.typeGuidMap);
+        typeNameMap  = new ConcurrentHashMap<>(other.typeNameMap);
+        serviceTypes = new HashSet<>(other.serviceTypes);
     }
 
     public void addType(AtlasType type) {
@@ -893,6 +954,10 @@ class TypeCache {
             if (StringUtils.isNotEmpty(type.getTypeName())) {
                 typeNameMap.put(type.getTypeName(), type);
             }
+
+            if (StringUtils.isNotEmpty(type.getServiceType())) {
+                serviceTypes.add(type.getServiceType());
+            }
         }
     }
 
@@ -905,6 +970,10 @@ class TypeCache {
             if (StringUtils.isNotEmpty(typeDef.getName())) {
                 typeNameMap.put(typeDef.getName(), type);
             }
+
+            if (StringUtils.isNotEmpty(type.getServiceType())) {
+                serviceTypes.add(type.getServiceType());
+            }
         }
     }
 
@@ -920,6 +989,10 @@ class TypeCache {
         return Collections.unmodifiableCollection(typeNameMap.values());
     }
 
+    public Set<String> getAllServiceTypes() {
+        return Collections.unmodifiableSet(serviceTypes);
+    }
+
     public AtlasType getTypeByGuid(String guid) {
 
         return guid != null ? typeGuidMap.get(guid) : null;
