@@ -17,53 +17,47 @@
  */
 package org.apache.atlas.type;
 
-import com.google.common.collect.ImmutableSet;
-
-import org.apache.atlas.AtlasErrorCode;
-import org.apache.atlas.exception.AtlasBaseException;
+import org.apache.atlas.model.instance.AtlasClassification;
 import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.model.instance.AtlasEntityHeader;
 import org.apache.atlas.model.instance.AtlasObjectId;
+import org.apache.atlas.model.instance.AtlasRelatedObjectId;
+import org.apache.atlas.model.instance.AtlasStruct;
 import org.apache.atlas.model.typedef.AtlasBaseTypeDef;
 import org.apache.atlas.model.typedef.AtlasClassificationDef;
 import org.apache.atlas.model.typedef.AtlasEntityDef;
 import org.apache.atlas.model.typedef.AtlasEnumDef;
 import org.apache.atlas.model.typedef.AtlasEnumDef.AtlasEnumElementDef;
+import org.apache.atlas.model.typedef.AtlasRelationshipDef;
+import org.apache.atlas.model.typedef.AtlasRelationshipDef.PropagateTags;
+import org.apache.atlas.model.typedef.AtlasRelationshipDef.RelationshipCategory;
+import org.apache.atlas.model.typedef.AtlasRelationshipEndDef;
 import org.apache.atlas.model.typedef.AtlasStructDef;
 import org.apache.atlas.model.typedef.AtlasStructDef.AtlasAttributeDef;
 import org.apache.atlas.model.typedef.AtlasStructDef.AtlasAttributeDef.Cardinality;
 import org.apache.atlas.model.typedef.AtlasStructDef.AtlasConstraintDef;
 import org.apache.atlas.model.typedef.AtlasTypeDefHeader;
 import org.apache.atlas.model.typedef.AtlasTypesDef;
+import org.apache.atlas.type.AtlasStructType.AtlasAttribute;
+import org.apache.atlas.v1.model.typedef.AttributeDefinition;
+import org.apache.atlas.v1.model.typedef.ClassTypeDefinition;
+import org.apache.atlas.v1.model.typedef.Multiplicity;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.MapUtils;
 import org.apache.commons.lang.StringUtils;
 
-
-import java.util.Arrays;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Collections;
-import java.util.HashSet;
-import java.util.LinkedList;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-import java.util.UUID;
-
+import java.util.*;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
+import java.util.stream.Collectors;
 
-import static org.apache.atlas.model.typedef.AtlasBaseTypeDef.ATLAS_TYPE_ARRAY_PREFIX;
-import static org.apache.atlas.model.typedef.AtlasBaseTypeDef.ATLAS_TYPE_ARRAY_SUFFIX;
-import static org.apache.atlas.model.typedef.AtlasBaseTypeDef.ATLAS_TYPE_MAP_KEY_VAL_SEP;
-import static org.apache.atlas.model.typedef.AtlasBaseTypeDef.ATLAS_TYPE_MAP_PREFIX;
-import static org.apache.atlas.model.typedef.AtlasBaseTypeDef.ATLAS_TYPE_MAP_SUFFIX;
+import static org.apache.atlas.model.typedef.AtlasBaseTypeDef.*;
 
 /**
  * Utility methods for AtlasType/AtlasTypeDef.
  */
 public class AtlasTypeUtil {
+
     private static final Set<String> ATLAS_BUILTIN_TYPENAMES = new HashSet<>();
     private static final String  NAME_REGEX         = "[a-zA-Z][a-zA-Z0-9_ ]*";
     private static final String  TRAIT_NAME_REGEX   = "[a-zA-Z][a-zA-Z0-9_ .]*";
@@ -149,38 +143,56 @@ public class AtlasTypeUtil {
 
     }
 
+    public static AtlasRelationshipType findRelationshipWithLegacyRelationshipEnd(String entityTypeName, String attributeName, AtlasTypeRegistry typeRegistry) {
+        AtlasRelationshipType ret = null;
+
+        for (AtlasRelationshipDef relationshipDef : typeRegistry.getAllRelationshipDefs()) {
+            AtlasRelationshipEndDef end1Def = relationshipDef.getEndDef1();
+            AtlasRelationshipEndDef end2Def = relationshipDef.getEndDef2();
+
+            if ((end1Def.getIsLegacyAttribute() && StringUtils.equals(end1Def.getType(), entityTypeName) && StringUtils.equals(end1Def.getName(), attributeName)) ||
+                (end2Def.getIsLegacyAttribute() && StringUtils.equals(end2Def.getType(), entityTypeName) && StringUtils.equals(end2Def.getName(), attributeName))) {
+                ret = typeRegistry.getRelationshipTypeByName(relationshipDef.getName());
+
+                break;
+            }
+        }
+
+        return ret;
+    }
+
     public static AtlasAttributeDef createOptionalAttrDef(String name, AtlasType dataType) {
         return new AtlasAttributeDef(name, dataType.getTypeName(), true,
             Cardinality.SINGLE, 0, 1,
-            false, false,
+            false, false, false,
             Collections.<AtlasConstraintDef>emptyList());
     }
 
     public static AtlasAttributeDef createOptionalAttrDef(String name, String dataType) {
         return new AtlasAttributeDef(name, dataType, true,
             Cardinality.SINGLE, 0, 1,
-            false, false,
+            false, false, false,
             Collections.<AtlasConstraintDef>emptyList());
     }
 
     public static AtlasAttributeDef createRequiredAttrDef(String name, String dataType) {
         return new AtlasAttributeDef(name, dataType, false,
             Cardinality.SINGLE, 1, 1,
-            false, true,
+            false, true, false,
             Collections.<AtlasConstraintDef>emptyList());
     }
 
     public static AtlasAttributeDef createListRequiredAttrDef(String name, String dataType) {
         return new AtlasAttributeDef(name, dataType, false,
                 Cardinality.LIST, 1, Integer.MAX_VALUE,
-                false, true,
+                false, true, false,
                 Collections.<AtlasConstraintDef>emptyList());
     }
 
     public static AtlasAttributeDef createOptionalListAttrDef(String name, String dataType) {
         return new AtlasAttributeDef(name, dataType, true,
                 Cardinality.LIST, 1, Integer.MAX_VALUE,
-                false, true,
+                false, true, false,
                 Collections.<AtlasConstraintDef>emptyList());
     }
 
@@ -208,21 +220,21 @@ public class AtlasTypeUtil {
     public static AtlasAttributeDef createUniqueRequiredAttrDef(String name, AtlasType dataType) {
         return new AtlasAttributeDef(name, dataType.getTypeName(), false,
             Cardinality.SINGLE, 1, 1,
-            true, true,
+            true, true, false,
             Collections.<AtlasConstraintDef>emptyList());
     }
 
     public static AtlasAttributeDef createUniqueRequiredAttrDef(String name, String typeName) {
         return new AtlasAttributeDef(name, typeName, false,
             Cardinality.SINGLE, 1, 1,
-            true, true,
+            true, true, false,
             Collections.<AtlasConstraintDef>emptyList());
     }
 
     public static AtlasAttributeDef createRequiredAttrDef(String name, AtlasType dataType) {
         return new AtlasAttributeDef(name, dataType.getTypeName(), false,
             Cardinality.SINGLE, 1, 1,
-            false, true,
+            false, true, false,
             Collections.<AtlasConstraintDef>emptyList());
     }
 
@@ -230,18 +242,22 @@ public class AtlasTypeUtil {
         return new AtlasEnumDef(name, description, "1.0", Arrays.asList(enumValues));
     }
 
-    public static AtlasClassificationDef createTraitTypeDef(String name, ImmutableSet<String> superTypes, AtlasAttributeDef... attrDefs) {
+    public static AtlasClassificationDef createTraitTypeDef(String name, Set<String> superTypes, AtlasAttributeDef... attrDefs) {
         return createTraitTypeDef(name, null, superTypes, attrDefs);
     }
 
-    public static AtlasClassificationDef createTraitTypeDef(String name, String description, ImmutableSet<String> superTypes, AtlasAttributeDef... attrDefs) {
+    public static AtlasClassificationDef createTraitTypeDef(String name, String description, Set<String> superTypes, AtlasAttributeDef... attrDefs) {
         return createTraitTypeDef(name, description, "1.0", superTypes, attrDefs);
     }
 
-    public static AtlasClassificationDef createTraitTypeDef(String name, String description, String version, ImmutableSet<String> superTypes, AtlasAttributeDef... attrDefs) {
+    public static AtlasClassificationDef createTraitTypeDef(String name, String description, String version, Set<String> superTypes, AtlasAttributeDef... attrDefs) {
         return new AtlasClassificationDef(name, description, version, Arrays.asList(attrDefs), superTypes);
     }
 
+    public static AtlasClassificationDef createAtlasClassificationDef(String name, String description, String version, Set<String> superTypes, Set<String> entityTypes, AtlasAttributeDef... attrDefs) {
+        return new AtlasClassificationDef(name, description, version, Arrays.asList(attrDefs), superTypes, entityTypes, null);
+    }
+
     public static AtlasStructDef createStructTypeDef(String name, AtlasAttributeDef... attrDefs) {
         return createStructTypeDef(name, null, attrDefs);
     }
@@ -250,21 +266,30 @@ public class AtlasTypeUtil {
         return new AtlasStructDef(name, description, "1.0", Arrays.asList(attrDefs));
     }
 
-    public static AtlasEntityDef createClassTypeDef(String name,
-        ImmutableSet<String> superTypes, AtlasAttributeDef... attrDefs) {
+    public static AtlasEntityDef createClassTypeDef(String name, Set<String> superTypes, AtlasAttributeDef... attrDefs) {
         return createClassTypeDef(name, null, "1.0", superTypes, attrDefs);
     }
 
-    public static AtlasEntityDef createClassTypeDef(String name, String description,
-        ImmutableSet<String> superTypes, AtlasAttributeDef... attrDefs) {
+    public static AtlasEntityDef createClassTypeDef(String name, String description, Set<String> superTypes, AtlasAttributeDef... attrDefs) {
         return createClassTypeDef(name, description, "1.0", superTypes, attrDefs);
     }
 
-    public static AtlasEntityDef createClassTypeDef(String name, String description, String version,
-        ImmutableSet<String> superTypes, AtlasAttributeDef... attrDefs) {
+    public static AtlasEntityDef createClassTypeDef(String name, String description, String version, Set<String> superTypes, AtlasAttributeDef... attrDefs) {
         return new AtlasEntityDef(name, description, version, Arrays.asList(attrDefs), superTypes);
     }
 
+    public static AtlasRelationshipDef createRelationshipTypeDef(String                  name,
+                                                                 String                  description,
+                                                                 String                  version,
+                                                                 RelationshipCategory    relationshipCategory,
+                                                                 PropagateTags           propagateTags,
+                                                                 AtlasRelationshipEndDef endDef1,
+                                                                 AtlasRelationshipEndDef endDef2,
+                                                                 AtlasAttributeDef...    attrDefs) {
+        return new AtlasRelationshipDef(name, description, version, relationshipCategory, propagateTags,
+                                        endDef1, endDef2, Arrays.asList(attrDefs));
+    }
+
     public static AtlasTypesDef getTypesDef(List<AtlasEnumDef> enums,
         List<AtlasStructDef> structs,
         List<AtlasClassificationDef> traits,
@@ -294,10 +319,33 @@ public class AtlasTypeUtil {
                 headerList.add(new AtlasTypeDefHeader(entityDef));
             }
         }
+        if (CollectionUtils.isNotEmpty(typesDef.getRelationshipDefs())) {
+            for (AtlasRelationshipDef relationshipDef : typesDef.getRelationshipDefs()) {
+                headerList.add(new AtlasTypeDefHeader(relationshipDef));
+            }
+        }
 
         return headerList;
     }
 
+    public static AtlasTypesDef getTypesDef(AtlasBaseTypeDef typeDef) {
+        AtlasTypesDef ret = new AtlasTypesDef();
+
+        if (typeDef != null) {
+            if (typeDef.getClass().equals(AtlasEntityDef.class)) {
+                ret.getEntityDefs().add((AtlasEntityDef) typeDef);
+            } else if (typeDef.getClass().equals(AtlasClassificationDef.class)) {
+                ret.getClassificationDefs().add((AtlasClassificationDef) typeDef);
+            } else if (typeDef.getClass().equals(AtlasStructDef.class)) {
+                ret.getStructDefs().add((AtlasStructDef) typeDef);
+            } else if (typeDef.getClass().equals(AtlasEnumDef.class)) {
+                ret.getEnumDefs().add((AtlasEnumDef) typeDef);
+            }
+        }
+
+        return ret;
+    }
+
     public static Collection<AtlasObjectId> toObjectIds(Collection<AtlasEntity> entities) {
         List<AtlasObjectId> ret = new ArrayList<>();
 
@@ -320,10 +368,50 @@ public class AtlasTypeUtil {
         return map;
     }
 
+    public static Map toRelationshipAttributes(Map map) {
+        Map ret = null;
+
+        if (map != null && map.containsKey("typeName") && map.containsKey("relationshipAttributes") && map.get("relationshipAttributes") instanceof Map) {
+            ret = (Map) map.get("relationshipAttributes");
+        }
+
+        return ret;
+    }
+
+    public static AtlasRelatedObjectId toAtlasRelatedObjectId(AtlasEntity entity) {
+        return new AtlasRelatedObjectId(getAtlasObjectId(entity));
+    }
+
+    public static AtlasRelatedObjectId toAtlasRelatedObjectId(AtlasEntity entity, AtlasTypeRegistry typeRegistry) {
+        return new AtlasRelatedObjectId(getAtlasObjectId(entity, typeRegistry));
+    }
+
     public static AtlasObjectId getAtlasObjectId(AtlasEntity entity) {
         return new AtlasObjectId(entity.getGuid(), entity.getTypeName());
     }
 
+    public static AtlasObjectId getAtlasObjectId(AtlasEntity entity, AtlasTypeRegistry typeRegistry) {
+        String              typeName       = entity.getTypeName();
+        AtlasEntityType     entityType     = typeRegistry.getEntityTypeByName(typeName);
+        Map<String, Object> uniqAttributes = null;
+
+        if (entityType != null && MapUtils.isNotEmpty(entityType.getUniqAttributes())) {
+            for (AtlasAttribute attribute : entityType.getUniqAttributes().values()) {
+                Object attrValue = entity.getAttribute(attribute.getName());
+
+                if (attrValue != null) {
+                    if (uniqAttributes == null) {
+                        uniqAttributes = new HashMap<>();
+                    }
+
+                    uniqAttributes.put(attribute.getName(), attrValue);
+                }
+            }
+        }
+
+        return new AtlasObjectId(entity.getGuid(), typeName, uniqAttributes);
+    }
+
     public static AtlasObjectId getAtlasObjectId(AtlasEntityHeader header) {
         return new AtlasObjectId(header.getGuid(), header.getTypeName());
     }
@@ -361,13 +449,21 @@ public class AtlasTypeUtil {
     }
 
     public static boolean isAssignedGuid(String guid) {
+        /**
+         * The rule for whether a GUID is 'assigned' is that it must always be non-null, non-empty
+         * and must not start with a '-' character, because in Atlas the '-' prefix character
+         * signifies an Atlas 'unassigned' GUID. There are no other GUID formatting constraints.
+         *
+         * An object from a remote repository can be saved into Atlas with its existing (external) GUID
+         * if that GUID conforms to the same 3 conditions. If, in future, it is required to save objects from
+         * a remote repository that assigns GUIDs that can start with the '-' character, then it will be
+         * necessary to enhance this isAssignedGUID() method to accepts and check the object's homeId, such
+         * that if homeId is not null (the object is from a remote repository), then the '-' prefix constraint
+         * is relaxed. Such a change would require a pervasive change to Atlas classes and therefore should
+         * only be implemented if it is found to be necessary.
+         */
         if (guid != null) {
-            try {
-                UUID.fromString(guid);
-                return true;
-            } catch (IllegalArgumentException e) {
-                // ignore
-            }
+            return guid != null && guid.length() > 0 && guid.charAt(0) != '-';
         }
         return false;
     }
@@ -406,12 +502,140 @@ public class AtlasTypeUtil {
             sb.append("entityDefs=[");
             dumpTypeNames(typesDef.getEntityDefs(), sb);
             sb.append("]");
+
+            sb.append("relationshipDefs=[");
+            dumpTypeNames(typesDef.getRelationshipDefs(), sb);
+            sb.append("]");
         }
         sb.append("}");
 
         return sb.toString();
     }
 
+    public static ClassTypeDefinition toClassTypeDefinition(final AtlasEntityType entityType) {
+        ClassTypeDefinition ret = null;
+
+        if (entityType != null) {
+            AtlasEntityDef entityDef = entityType.getEntityDef();
+            ret = new ClassTypeDefinition();
+            ret.setTypeName(entityDef.getName());
+            ret.setTypeDescription(entityDef.getDescription());
+            ret.setTypeVersion(entityDef.getTypeVersion());
+            ret.setSuperTypes(entityDef.getSuperTypes());
+
+            if (MapUtils.isNotEmpty(entityType.getAllAttributes())) {
+                List<AttributeDefinition> attributeDefinitions = entityType.getAllAttributes()
+                                                                           .entrySet()
+                                                                           .stream()
+                                                                           .map(e -> toV1AttributeDefinition(e.getValue()))
+                                                                           .collect(Collectors.toList());
+
+                ret.setAttributeDefinitions(attributeDefinitions);
+            }
+        }
+
+        return ret;
+    }
+
+    public static AttributeDefinition toV1AttributeDefinition(AtlasStructType.AtlasAttribute attribute) {
+        AtlasAttributeDef   attributeDef = attribute.getAttributeDef();
+        AttributeDefinition ret = new AttributeDefinition();
+
+        ret.setName(attributeDef.getName());
+        ret.setDataTypeName(attributeDef.getTypeName());
+        ret.setIsUnique(attributeDef.getIsUnique());
+        ret.setIsIndexable(attributeDef.getIsIndexable());
+        ret.setIsComposite(attribute.isOwnedRef());
+        ret.setReverseAttributeName(attribute.getInverseRefAttributeName());
+        ret.setDefaultValue(attributeDef.getDefaultValue());
+        ret.setDescription(attributeDef.getDescription());
+        ret.setOptions(attributeDef.getOptions());
+        ret.setMultiplicity(getMultiplicity(attributeDef));
+
+        return ret;
+    }
+
+    public static Multiplicity getMultiplicity(AtlasAttributeDef attributeDef) {
+        int lower;
+        int upper;
+        if (attributeDef.getCardinality() == Cardinality.SINGLE) {
+            lower = attributeDef.getIsOptional() ? 0 : 1;
+            upper = 1;
+        } else {
+            if(attributeDef.getIsOptional()) {
+                lower = 0;
+            } else {
+                lower = attributeDef.getValuesMinCount() < 1 ? 1 : attributeDef.getValuesMinCount();
+            }
+
+            upper = attributeDef.getValuesMaxCount() < 2 ? Integer.MAX_VALUE : attributeDef.getValuesMaxCount();
+        }
+
+        return new Multiplicity(lower, upper, Cardinality.SET.equals(attributeDef.getCardinality()));
+    }
+
+    public static Map<String, Object> toMap(AtlasEntity entity) {
+        Map<String, Object> ret = null;
+
+        if (entity != null) {
+            ret = new LinkedHashMap<>();
+
+            // Id type
+            ret.put("$typeName$", entity.getTypeName());
+            ret.put("$id$", new LinkedHashMap<String, Object>(){{
+                put("id", entity.getGuid());
+                put("$typeName$", entity.getTypeName());
+                put("version", entity.getVersion().intValue());
+                put("state", entity.getStatus().name());
+            }});
+
+            // System attributes
+            ret.put("$systemAttributes$", new LinkedHashMap<String, String>() {{
+                put("createdBy", entity.getCreatedBy());
+                put("modifiedBy", entity.getUpdatedBy());
+                put("createdTime", entity.getCreateTime().toString());
+                put("modifiedTime", entity.getUpdateTime().toString());
+            }});
+
+            // Traits
+            if (CollectionUtils.isNotEmpty(entity.getClassifications())) {
+                Map<String, HashMap> traitDetails = entity.getClassifications()
+                                                          .stream()
+                                                          .collect(Collectors.toMap(AtlasStruct::getTypeName, AtlasTypeUtil::getNestedTraitDetails));
+                ret.put("$traits$", traitDetails);
+            }
+
+            // All attributes
+            if (MapUtils.isNotEmpty(entity.getAttributes())) {
+                for (Map.Entry<String, Object> entry : entity.getAttributes().entrySet()) {
+                    if (entry.getValue() instanceof AtlasObjectId) {
+                        ret.put(entry.getKey(), new LinkedHashMap<String, Object>(){{
+                            put("id", ((AtlasObjectId) entry.getValue()).getGuid());
+                            put("$typeName$", ((AtlasObjectId) entry.getValue()).getTypeName());
+//                        put("version", entity.getVersion().intValue());
+//                        put("state", entity.getStatus().name());
+                        }});
+                    } else {
+                        ret.put(entry.getKey(), entry.getValue());
+                    }
+                }
+            }
+
+        }
+
+        return ret;
+    }
+
+    private static HashMap getNestedTraitDetails(final AtlasClassification atlasClassification) {
+        return new HashMap<String, Object>() {{
+            put("$typeName$", atlasClassification.getTypeName());
+
+            if (MapUtils.isNotEmpty(atlasClassification.getAttributes())) {
+                putAll(atlasClassification.getAttributes());
+            }
+        }};
+    }
+
     private static void dumpTypeNames(List<? extends AtlasBaseTypeDef> typeDefs, StringBuilder sb) {
         if (CollectionUtils.isNotEmpty(typeDefs)) {
             for (int i = 0; i < typeDefs.size(); i++) {
@@ -425,4 +649,4 @@ public class AtlasTypeUtil {
             }
         }
     }
-}
\ No newline at end of file
+}
