@@ -22,12 +22,21 @@ import com.google.common.annotations.VisibleForTesting;
 import org.apache.atlas.ApplicationProperties;
 import org.apache.atlas.AtlasException;
 import org.apache.atlas.EntityAuditEvent;
+import org.apache.atlas.EntityAuditEvent.EntityAuditAction;
 import org.apache.atlas.annotation.ConditionalOnAtlasProperty;
+import org.apache.atlas.model.audit.EntityAuditEventV2;
+import org.apache.atlas.model.audit.EntityAuditEventV2.EntityAuditActionV2;
+import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.ha.HAConfiguration;
-import org.apache.atlas.listener.ActiveStateChangeHandler;
-import org.apache.atlas.service.Service;
+import org.apache.atlas.model.instance.AtlasClassification;
+import org.apache.atlas.model.instance.AtlasEntity;
+import org.apache.atlas.model.instance.AtlasEntity.AtlasEntitiesWithExtInfo;
+import org.apache.atlas.repository.converters.AtlasInstanceConverter;
+import org.apache.atlas.type.AtlasType;
+import org.apache.atlas.v1.model.instance.Referenceable;
+import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.configuration.Configuration;
-import org.apache.commons.configuration.ConfigurationConverter;
+import org.apache.commons.lang.ArrayUtils;
 import org.apache.commons.lang.StringUtils;
 import org.apache.hadoop.hbase.HBaseConfiguration;
 import org.apache.hadoop.hbase.HColumnDescriptor;
@@ -41,7 +50,10 @@ import org.apache.hadoop.hbase.client.Result;
 import org.apache.hadoop.hbase.client.ResultScanner;
 import org.apache.hadoop.hbase.client.Scan;
 import org.apache.hadoop.hbase.client.Table;
+import org.apache.hadoop.hbase.filter.BinaryPrefixComparator;
+import org.apache.hadoop.hbase.filter.CompareFilter;
 import org.apache.hadoop.hbase.filter.PageFilter;
+import org.apache.hadoop.hbase.filter.SingleColumnValueFilter;
 import org.apache.hadoop.hbase.io.compress.Compression;
 import org.apache.hadoop.hbase.io.encoding.DataBlockEncoding;
 import org.apache.hadoop.hbase.regionserver.BloomType;
@@ -50,15 +62,25 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.springframework.stereotype.Component;
 
+import javax.inject.Inject;
 import javax.inject.Singleton;
 import java.io.Closeable;
 import java.io.IOException;
 import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
-import java.util.Map;
 import java.util.Properties;
+import java.util.Set;
+
+import static org.apache.atlas.EntityAuditEvent.EntityAuditAction.TAG_ADD;
+import static org.apache.atlas.EntityAuditEvent.EntityAuditAction.TAG_DELETE;
+import static org.apache.atlas.EntityAuditEvent.EntityAuditAction.TAG_UPDATE;
+import static org.apache.atlas.EntityAuditEvent.EntityAuditAction.TERM_ADD;
+import static org.apache.atlas.EntityAuditEvent.EntityAuditAction.TERM_DELETE;
+import static org.apache.atlas.model.audit.EntityAuditEventV2.EntityAuditType;
+import static org.apache.atlas.model.audit.EntityAuditEventV2.EntityAuditType.ENTITY_AUDIT_V1;
+import static org.apache.atlas.model.audit.EntityAuditEventV2.EntityAuditType.ENTITY_AUDIT_V2;
+import static org.apache.atlas.repository.audit.EntityAuditListener.getV2AuditPrefix;
 
 /**
  * HBase based repository for entity audit events
@@ -77,76 +99,65 @@ import java.util.Properties;
 @Singleton
 @Component
 @ConditionalOnAtlasProperty(property = "atlas.EntityAuditRepository.impl", isDefault = true)
-public class HBaseBasedAuditRepository implements Service, EntityAuditRepository, ActiveStateChangeHandler {
+public class HBaseBasedAuditRepository extends AbstractStorageBasedAuditRepository {
     private static final Logger LOG = LoggerFactory.getLogger(HBaseBasedAuditRepository.class);
 
-    public static final String CONFIG_PREFIX = "atlas.audit";
-    public static final String CONFIG_TABLE_NAME = CONFIG_PREFIX + ".hbase.tablename";
+    public static final String CONFIG_TABLE_NAME  = CONFIG_PREFIX + ".hbase.tablename";
     public static final String DEFAULT_TABLE_NAME = "ATLAS_ENTITY_AUDIT_EVENTS";
-    public static final String CONFIG_PERSIST_ENTITY_DEFINITION = CONFIG_PREFIX + ".persistEntityDefinition";
+    public static final byte[] COLUMN_FAMILY      = Bytes.toBytes("dt");
+    public static final byte[] COLUMN_ACTION      = Bytes.toBytes("a");
+    public static final byte[] COLUMN_DETAIL      = Bytes.toBytes("d");
+    public static final byte[] COLUMN_USER        = Bytes.toBytes("u");
+    public static final byte[] COLUMN_DEFINITION  = Bytes.toBytes("f");
+    public static final byte[] COLUMN_TYPE        = Bytes.toBytes("t");
 
-    public static final byte[] COLUMN_FAMILY = Bytes.toBytes("dt");
-    public static final byte[] COLUMN_ACTION = Bytes.toBytes("a");
-    public static final byte[] COLUMN_DETAIL = Bytes.toBytes("d");
-    public static final byte[] COLUMN_USER = Bytes.toBytes("u");
-    public static final byte[] COLUMN_DEFINITION = Bytes.toBytes("f");
-
-    private static final String  AUDIT_REPOSITORY_MAX_SIZE_PROPERTY = "atlas.hbase.client.keyvalue.maxsize";
-    private static final String  AUDIT_EXCLUDE_ATTRIBUTE_PROPERTY   = "atlas.audit.hbase.entity";
-    private static final String  FIELD_SEPARATOR = ":";
-    private static final long    ATLAS_HBASE_KEYVALUE_DEFAULT_SIZE = 1024 * 1024;
-    private static Configuration APPLICATION_PROPERTIES = null;
-
-    private static boolean       persistEntityDefinition;
-
-    private Map<String, List<String>> auditExcludedAttributesCache = new HashMap<>();
-
-    static {
-        try {
-            persistEntityDefinition = ApplicationProperties.get().getBoolean(CONFIG_PERSIST_ENTITY_DEFINITION, false);
-        } catch (AtlasException e) {
-            throw new RuntimeException(e);
-        }
-    }
     private TableName tableName;
     private Connection connection;
+    private final AtlasInstanceConverter instanceConverter;
 
-    /**
-     * Add events to the event repository
-     * @param events events to be added
-     * @throws AtlasException
-     */
-    @Override
-    public void putEvents(EntityAuditEvent... events) throws AtlasException {
-        putEvents(Arrays.asList(events));
+    @Inject
+    public HBaseBasedAuditRepository(AtlasInstanceConverter instanceConverter) {
+        this.instanceConverter = instanceConverter;
     }
 
-    @Override
     /**
      * Add events to the event repository
      * @param events events to be added
      * @throws AtlasException
      */
-    public void putEvents(List<EntityAuditEvent> events) throws AtlasException {
+    @Override
+    public void putEventsV1(List<EntityAuditEvent> events) throws AtlasException {
         if (LOG.isDebugEnabled()) {
             LOG.debug("Putting {} events", events.size());
         }
 
         Table table = null;
+
         try {
-            table = connection.getTable(tableName);
+            table          = connection.getTable(tableName);
             List<Put> puts = new ArrayList<>(events.size());
-            for (EntityAuditEvent event : events) {
-                LOG.debug("Adding entity audit event {}", event);
-                Put put = new Put(getKey(event.getEntityId(), event.getTimestamp()));
+
+            for (int index = 0; index < events.size(); index++) {
+                EntityAuditEvent event = events.get(index);
+
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Adding entity audit event {}", event);
+                }
+
+                Put put = new Put(getKey(event.getEntityId(), event.getTimestamp(), index));
+
                 addColumn(put, COLUMN_ACTION, event.getAction());
                 addColumn(put, COLUMN_USER, event.getUser());
                 addColumn(put, COLUMN_DETAIL, event.getDetails());
+                addColumn(put, COLUMN_TYPE, ENTITY_AUDIT_V1);
+
                 if (persistEntityDefinition) {
                     addColumn(put, COLUMN_DEFINITION, event.getEntityDefinitionString());
                 }
+
                 puts.add(put);
             }
+
             table.put(puts);
         } catch (IOException e) {
             throw new AtlasException(e);
@@ -155,19 +166,224 @@ public class HBaseBasedAuditRepository implements Service, EntityAuditRepository
         }
     }
 
+    @Override
+    public void putEventsV2(List<EntityAuditEventV2> events) throws AtlasBaseException {
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Putting {} events", events.size());
+        }
+
+        Table table = null;
+
+        try {
+            table          = connection.getTable(tableName);
+            List<Put> puts = new ArrayList<>(events.size());
+
+            for (int index = 0; index < events.size(); index++) {
+                EntityAuditEventV2 event = events.get(index);
+
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Adding entity audit event {}", event);
+                }
+
+                Put put = new Put(getKey(event.getEntityId(), event.getTimestamp(), index));
+
+                addColumn(put, COLUMN_ACTION, event.getAction());
+                addColumn(put, COLUMN_USER, event.getUser());
+                addColumn(put, COLUMN_DETAIL, event.getDetails());
+                addColumn(put, COLUMN_TYPE, ENTITY_AUDIT_V2);
+
+                if (persistEntityDefinition) {
+                    addColumn(put, COLUMN_DEFINITION, event.getEntityDefinitionString());
+                }
+
+                puts.add(put);
+            }
+
+            table.put(puts);
+        } catch (IOException e) {
+            throw new AtlasBaseException(e);
+        } finally {
+            try {
+                close(table);
+            } catch (AtlasException e) {
+                throw new AtlasBaseException(e);
+            }
+        }
+    }
+
+    @Override
+    public List<EntityAuditEventV2> listEventsV2(String entityId, String startKey, short n) throws AtlasBaseException {
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Listing events for entity id {}, starting timestamp {}, #records {}", entityId, startKey, n);
+        }
+
+        Table         table   = null;
+        ResultScanner scanner = null;
+
+        try {
+            table = connection.getTable(tableName);
+
+            /**
+             * Scan Details:
+             * In hbase, the events are stored in increasing order of timestamp. So, doing reverse scan to get the latest event first
+             * Page filter is set to limit the number of results returned.
+             * Stop row is set to the entity id to avoid going past the current entity while scanning
+             * small is set to true to optimise RPC calls as the scanner is created per request
+             */
+            Scan scan = new Scan().setReversed(true).setFilter(new PageFilter(n))
+                                  .setStopRow(Bytes.toBytes(entityId))
+                                  .setCaching(n)
+                                  .setSmall(true);
+
+            if (StringUtils.isEmpty(startKey)) {
+                //Set start row to entity id + max long value
+                byte[] entityBytes = getKey(entityId, Long.MAX_VALUE, Integer.MAX_VALUE);
+                scan = scan.setStartRow(entityBytes);
+            } else {
+                scan = scan.setStartRow(Bytes.toBytes(startKey));
+            }
+
+            scanner = table.getScanner(scan);
+            List<EntityAuditEventV2> events = new ArrayList<>();
+
+            Result result;
+
+            //PageFilter doesn't ensure n results are returned. The filter is per region server.
+            //So, adding extra check on n here
+            while ((result = scanner.next()) != null && events.size() < n) {
+                EntityAuditEventV2 event = fromKeyV2(result.getRow());
+
+                //In case the user sets random start key, guarding against random events
+                if (!event.getEntityId().equals(entityId)) {
+                    continue;
+                }
+
+                event.setUser(getResultString(result, COLUMN_USER));
+                event.setAction(EntityAuditActionV2.fromString(getResultString(result, COLUMN_ACTION)));
+                event.setDetails(getEntityDetails(result));
+                event.setType(getAuditType(result));
+
+                if (persistEntityDefinition) {
+                    event.setEntityDefinition(getEntityDefinition(result));
+                }
+
+                events.add(event);
+            }
+
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Got events for entity id {}, starting timestamp {}, #records {}", entityId, startKey, events.size());
+            }
+
+            return events;
+        } catch (IOException e) {
+            throw new AtlasBaseException(e);
+        } finally {
+            try {
+                close(scanner);
+                close(table);
+            } catch (AtlasException e) {
+                throw new AtlasBaseException(e);
+            }
+        }
+    }
+
+    private String getEntityDefinition(Result result) throws AtlasBaseException {
+        String ret = getResultString(result, COLUMN_DEFINITION);
+
+        if (getAuditType(result) != ENTITY_AUDIT_V2) {
+            Referenceable referenceable = AtlasType.fromV1Json(ret, Referenceable.class);
+            AtlasEntity   entity        = toAtlasEntity(referenceable);
+
+            ret = AtlasType.toJson(entity);
+        }
+
+        return ret;
+    }
+
+    private String getEntityDetails(Result result) throws AtlasBaseException {
+        String ret;
+
+        if (getAuditType(result) == ENTITY_AUDIT_V2) {
+            ret = getResultString(result, COLUMN_DETAIL);
+        } else {
+            // convert v1 audit detail to v2
+            ret = getV2Details(result);
+        }
+
+        return ret;
+    }
+
+    private EntityAuditType getAuditType(Result result) {
+        String          typeString = getResultString(result, COLUMN_TYPE);
+        EntityAuditType ret        = (typeString != null) ? EntityAuditType.valueOf(typeString) : ENTITY_AUDIT_V1;
+
+        return ret;
+    }
+
+    private String  getV2Details(Result result) throws AtlasBaseException {
+        String ret                 = null;
+        String v1DetailsWithPrefix = getResultString(result, COLUMN_DETAIL);
+
+        if (StringUtils.isNotEmpty(v1DetailsWithPrefix)) {
+            EntityAuditAction v1AuditAction = EntityAuditAction.fromString(getResultString(result, COLUMN_ACTION));
+
+            if (v1AuditAction == TERM_ADD || v1AuditAction == TERM_DELETE) {
+                // for terms audit v1 and v2 structure is same
+                ret = v1DetailsWithPrefix;
+            } else {
+                String            v1AuditPrefix = EntityAuditListener.getV1AuditPrefix(v1AuditAction);
+                String[]          split         = v1DetailsWithPrefix.split(v1AuditPrefix);
+
+                if (ArrayUtils.isNotEmpty(split) && split.length == 2) {
+                    String        v1AuditDetails = split[1];
+                    Referenceable referenceable  = AtlasType.fromV1Json(v1AuditDetails, Referenceable.class);
+                    String        v2Json         = (referenceable != null) ? toV2Json(referenceable, v1AuditAction) : v1AuditDetails;
+
+                    if (v2Json != null) {
+                        ret = getV2AuditPrefix(v1AuditAction) + v2Json;
+                    }
+                } else {
+                    ret = v1DetailsWithPrefix;
+                }
+            }
+        }
+
+        return ret;
+    }
+
+    private String toV2Json(Referenceable referenceable, EntityAuditAction action) throws AtlasBaseException {
+        String ret;
+
+        if (action == TAG_ADD || action == TAG_UPDATE || action == TAG_DELETE) {
+            AtlasClassification classification = instanceConverter.toAtlasClassification(referenceable);
+
+            ret = AtlasType.toJson(classification);
+        } else {
+            AtlasEntity entity = toAtlasEntity(referenceable);
+
+            ret = AtlasType.toJson(entity);
+        }
+
+        return ret;
+    }
+
+    private AtlasEntity toAtlasEntity(Referenceable referenceable) throws AtlasBaseException {
+        AtlasEntity              ret                 = null;
+        AtlasEntitiesWithExtInfo entitiesWithExtInfo = instanceConverter.toAtlasEntity(referenceable);
+
+        if (entitiesWithExtInfo != null && CollectionUtils.isNotEmpty(entitiesWithExtInfo.getEntities())) {
+            ret = entitiesWithExtInfo.getEntities().get(0);
+        }
+
+        return ret;
+    }
+
     private <T> void addColumn(Put put, byte[] columnName, T columnValue) {
         if (columnValue != null && !columnValue.toString().isEmpty()) {
             put.addColumn(COLUMN_FAMILY, columnName, Bytes.toBytes(columnValue.toString()));
         }
     }
 
-    private byte[] getKey(String id, Long ts) {
-        assert id != null : "entity id can't be null";
-        assert ts != null : "timestamp can't be null";
-        String keyStr = id + FIELD_SEPARATOR + ts;
-        return Bytes.toBytes(keyStr);
-    }
-
     /**
      * List events for the given entity id in decreasing order of timestamp, from the given startKey. Returns n results
      * @param entityId entity id
@@ -176,7 +392,7 @@ public class HBaseBasedAuditRepository implements Service, EntityAuditRepository
      * @return list of events
      * @throws AtlasException
      */
-    public List<EntityAuditEvent> listEvents(String entityId, String startKey, short n)
+    public List<EntityAuditEvent> listEventsV1(String entityId, String startKey, short n)
             throws AtlasException {
         if (LOG.isDebugEnabled()) {
             LOG.debug("Listing events for entity id {}, starting timestamp {}, #records {}", entityId, startKey, n);
@@ -200,7 +416,7 @@ public class HBaseBasedAuditRepository implements Service, EntityAuditRepository
                                   .setSmall(true);
             if (StringUtils.isEmpty(startKey)) {
                 //Set start row to entity id + max long value
-                byte[] entityBytes = getKey(entityId, Long.MAX_VALUE);
+                byte[] entityBytes = getKey(entityId, Long.MAX_VALUE, Integer.MAX_VALUE);
                 scan = scan.setStartRow(entityBytes);
             } else {
                 scan = scan.setStartRow(Bytes.toBytes(startKey));
@@ -219,7 +435,7 @@ public class HBaseBasedAuditRepository implements Service, EntityAuditRepository
                     continue;
                 }
                 event.setUser(getResultString(result, COLUMN_USER));
-                event.setAction(EntityAuditEvent.EntityAuditAction.valueOf(getResultString(result, COLUMN_ACTION)));
+                event.setAction(EntityAuditEvent.EntityAuditAction.fromString(getResultString(result, COLUMN_ACTION)));
                 event.setDetails(getResultString(result, COLUMN_DETAIL));
                 if (persistEntityDefinition) {
                     String colDef = getResultString(result, COLUMN_DEFINITION);
@@ -243,52 +459,6 @@ public class HBaseBasedAuditRepository implements Service, EntityAuditRepository
         }
     }
 
-    @Override
-    public long repositoryMaxSize() throws AtlasException {
-        long ret;
-        initApplicationProperties();
-
-        if (APPLICATION_PROPERTIES == null) {
-            ret = ATLAS_HBASE_KEYVALUE_DEFAULT_SIZE;
-        } else {
-            ret = APPLICATION_PROPERTIES.getLong(AUDIT_REPOSITORY_MAX_SIZE_PROPERTY, ATLAS_HBASE_KEYVALUE_DEFAULT_SIZE);
-        }
-
-        return ret;
-    }
-
-    @Override
-    public List<String> getAuditExcludeAttributes(String entityType) throws AtlasException {
-        List<String> ret = null;
-
-        initApplicationProperties();
-
-        if (auditExcludedAttributesCache.containsKey(entityType)) {
-            ret = auditExcludedAttributesCache.get(entityType);
-        } else if (APPLICATION_PROPERTIES != null) {
-            String[] excludeAttributes = APPLICATION_PROPERTIES.getStringArray(AUDIT_EXCLUDE_ATTRIBUTE_PROPERTY + "." +
-                    entityType + "." +  "attributes.exclude");
-
-            if (excludeAttributes != null) {
-                ret = Arrays.asList(excludeAttributes);
-            }
-
-            auditExcludedAttributesCache.put(entityType, ret);
-        }
-
-        return ret;
-    }
-
-    private void initApplicationProperties() {
-        if (APPLICATION_PROPERTIES == null) {
-            try {
-                APPLICATION_PROPERTIES = ApplicationProperties.get();
-            } catch (AtlasException ex) {
-                // ignore
-            }
-        }
-    }
-
     private String getResultString(Result result, byte[] columnName) {
         byte[] rawValue = result.getValue(COLUMN_FAMILY, columnName);
         if ( rawValue != null) {
@@ -309,6 +479,20 @@ public class HBaseBasedAuditRepository implements Service, EntityAuditRepository
         return event;
     }
 
+    private EntityAuditEventV2 fromKeyV2(byte[] keyBytes) {
+        String             key   = Bytes.toString(keyBytes);
+        EntityAuditEventV2 event = new EntityAuditEventV2();
+
+        if (StringUtils.isNotEmpty(key)) {
+            String[] parts = key.split(FIELD_SEPARATOR);
+            event.setEntityId(parts[0]);
+            event.setTimestamp(Long.valueOf(parts[1]));
+            event.setEventKey(key);
+        }
+
+        return event;
+    }
+
     private void close(Closeable closeable) throws AtlasException {
         if (closeable != null) {
             try {
@@ -326,9 +510,8 @@ public class HBaseBasedAuditRepository implements Service, EntityAuditRepository
      * @param atlasConf
      */
     public static org.apache.hadoop.conf.Configuration getHBaseConfiguration(Configuration atlasConf) throws AtlasException {
-        Configuration                        subsetAtlasConf = ApplicationProperties.getSubsetConfiguration(atlasConf, CONFIG_PREFIX);
-        Properties                           properties      = ConfigurationConverter.getProperties(subsetAtlasConf);
-        org.apache.hadoop.conf.Configuration hbaseConf       = HBaseConfiguration.create();
+        Properties                           properties = ApplicationProperties.getSubsetAsProperties(atlasConf, CONFIG_PREFIX);
+        org.apache.hadoop.conf.Configuration hbaseConf  = HBaseConfiguration.create();
 
         for (String key : properties.stringPropertyNames()) {
             String value = properties.getProperty(key);
@@ -337,6 +520,7 @@ public class HBaseBasedAuditRepository implements Service, EntityAuditRepository
 
             hbaseConf.set(key, value);
         }
+
         return hbaseConf;
     }
 
@@ -365,6 +549,52 @@ public class HBaseBasedAuditRepository implements Service, EntityAuditRepository
         }
     }
 
+    @Override
+    public Set<String> getEntitiesWithTagChanges(long fromTimestamp, long toTimestamp) throws AtlasBaseException {
+        final String classificationUpdatesAction = "CLASSIFICATION_";
+
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Listing events for fromTimestamp {}, toTimestamp {}, action {}", fromTimestamp, toTimestamp);
+        }
+
+        Table table = null;
+        ResultScanner scanner = null;
+
+        try {
+            Set<String> guids = new HashSet<>();
+
+            table = connection.getTable(tableName);
+
+            byte[] filterValue = Bytes.toBytes(classificationUpdatesAction);
+            BinaryPrefixComparator binaryPrefixComparator = new BinaryPrefixComparator(filterValue);
+            SingleColumnValueFilter filter = new SingleColumnValueFilter(COLUMN_FAMILY, COLUMN_ACTION, CompareFilter.CompareOp.EQUAL, binaryPrefixComparator);
+            Scan scan = new Scan().setFilter(filter).setTimeRange(fromTimestamp, toTimestamp);
+
+            Result result;
+            scanner = table.getScanner(scan);
+            while ((result = scanner.next()) != null) {
+                EntityAuditEvent event = fromKey(result.getRow());
+
+                if (event == null) {
+                    continue;
+                }
+
+                guids.add(event.getEntityId());
+            }
+
+            return guids;
+        } catch (IOException e) {
+            throw new AtlasBaseException(e);
+        } finally {
+            try {
+                close(scanner);
+                close(table);
+            } catch (AtlasException e) {
+                throw new AtlasBaseException(e);
+            }
+        }
+    }
+
     @Override
     public void start() throws AtlasException {
         Configuration configuration = ApplicationProperties.get();
@@ -406,13 +636,4 @@ public class HBaseBasedAuditRepository implements Service, EntityAuditRepository
         createTableIfNotExists();
     }
 
-    @Override
-    public void instanceIsPassive() {
-        LOG.info("Reacting to passive: No action for now.");
-    }
-
-    @Override
-    public int getHandlerOrder() {
-        return HandlerOrder.HBASE_AUDIT_REPOSITORY.getOrder();
-    }
 }
