@@ -20,10 +20,11 @@ package org.apache.atlas.repository.impexp;
 import com.google.common.annotations.VisibleForTesting;
 import org.apache.atlas.AtlasErrorCode;
 import org.apache.atlas.RequestContext;
-import org.apache.atlas.RequestContextV1;
 import org.apache.atlas.exception.AtlasBaseException;
+import org.apache.atlas.model.TypeCategory;
 import org.apache.atlas.model.impexp.AtlasExportRequest;
 import org.apache.atlas.model.impexp.AtlasExportResult;
+import org.apache.atlas.model.instance.AtlasClassification;
 import org.apache.atlas.model.instance.AtlasEntity;
 import org.apache.atlas.model.instance.AtlasEntity.AtlasEntityWithExtInfo;
 import org.apache.atlas.model.instance.AtlasObjectId;
@@ -31,14 +32,20 @@ import org.apache.atlas.model.typedef.AtlasBaseTypeDef;
 import org.apache.atlas.model.typedef.AtlasClassificationDef;
 import org.apache.atlas.model.typedef.AtlasEntityDef;
 import org.apache.atlas.model.typedef.AtlasEnumDef;
+import org.apache.atlas.model.typedef.AtlasRelationshipDef;
 import org.apache.atlas.model.typedef.AtlasStructDef;
+import org.apache.atlas.model.typedef.AtlasStructDef.AtlasAttributeDef;
 import org.apache.atlas.model.typedef.AtlasTypesDef;
-import org.apache.atlas.repository.graph.GraphHelper;
 import org.apache.atlas.repository.graphdb.AtlasGraph;
-import org.apache.atlas.repository.graphdb.AtlasVertex;
-import org.apache.atlas.repository.store.graph.v1.EntityGraphRetriever;
+import org.apache.atlas.repository.store.graph.v2.EntityGraphRetriever;
 import org.apache.atlas.repository.util.UniqueList;
+import org.apache.atlas.type.AtlasArrayType;
+import org.apache.atlas.type.AtlasClassificationType;
 import org.apache.atlas.type.AtlasEntityType;
+import org.apache.atlas.type.AtlasEnumType;
+import org.apache.atlas.type.AtlasMapType;
+import org.apache.atlas.type.AtlasRelationshipType;
+import org.apache.atlas.type.AtlasStructType;
 import org.apache.atlas.type.AtlasStructType.AtlasAttribute;
 import org.apache.atlas.type.AtlasType;
 import org.apache.atlas.type.AtlasTypeRegistry;
@@ -120,7 +127,7 @@ public class ExportService {
     }
 
     private long getCurrentChangeMarker() {
-        return Math.min(RequestContextV1.earliestActiveRequestTime(), RequestContext.earliestActiveRequestTime());
+        return RequestContext.earliestActiveRequestTime();
     }
 
     private void updateSinkWithOperationMetrics(String userName, ExportContext context,
@@ -170,6 +177,12 @@ public class ExportService {
 
             typesDef.getEnumDefs().add(enumDef);
         }
+
+        for (String relationshipType : context.relationshipTypes) {
+            AtlasRelationshipDef relationshipDef = typeRegistry.getRelationshipDefByName(relationshipType);
+
+            typesDef.getRelationshipDefs().add(relationshipDef);
+        }
     }
 
     private AtlasExportResult.OperationStatus[] processItems(AtlasExportRequest request, ExportContext context) {
@@ -445,13 +458,13 @@ public class ExportService {
             context.bindings.clear();
             context.bindings.put(QUERY_BINDING_START_GUID, entity.getGuid());
 
-            List<HashMap<String, Object>> result = executeGremlinQuery(query, context);
+            List<Map<String, Object>> result = executeGremlinQuery(query, context);
 
             if (CollectionUtils.isEmpty(result)) {
                 continue;
             }
 
-            for (HashMap<String, Object> hashMap : result) {
+            for (Map<String, Object> hashMap : result) {
                 String             guid             = (String) hashMap.get(PROPERTY_GUID);
                 TraversalDirection currentDirection = context.guidDirection.get(guid);
                 boolean            isLineage        = (boolean) hashMap.get(PROPERTY_IS_PROCESS);
@@ -495,13 +508,13 @@ public class ExportService {
         context.bindings.clear();
         context.bindings.put(QUERY_BINDING_START_GUID, entity.getGuid());
 
-        List<HashMap<String, Object>> result = executeGremlinQuery(query, context);
+        List<Map<String, Object>> result = executeGremlinQuery(query, context);
 
         if (CollectionUtils.isEmpty(result)) {
             return;
         }
 
-        for (HashMap<String, Object> hashMap : result) {
+        for (Map<String, Object> hashMap : result) {
             String  guid      = (String) hashMap.get(PROPERTY_GUID);
             boolean isLineage = (boolean) hashMap.get(PROPERTY_IS_PROCESS);
 
@@ -546,9 +559,145 @@ public class ExportService {
         context.reportProgress();
     }
 
-    private List<HashMap<String, Object>> executeGremlinQuery(String query, ExportContext context) {
+    private void addTypes(AtlasEntity entity, ExportContext context) {
+        addEntityType(entity.getTypeName(), context);
+
+        if(CollectionUtils.isNotEmpty(entity.getClassifications())) {
+            for (AtlasClassification c : entity.getClassifications()) {
+                addClassificationType(c.getTypeName(), context);
+            }
+        }
+    }
+
+    private void addType(String typeName, ExportContext context) {
+        AtlasType type = null;
+
         try {
-            return (List<HashMap<String, Object>>) atlasGraph.executeGremlinScript(context.scriptEngine, context.bindings, query, false);
+            type = typeRegistry.getType(typeName);
+
+            addType(type, context);
+        } catch (AtlasBaseException excp) {
+            LOG.error("unknown type {}", typeName);
+        }
+    }
+
+    private void addEntityType(String typeName, ExportContext context) {
+        if (!context.entityTypes.contains(typeName)) {
+            AtlasEntityType entityType = typeRegistry.getEntityTypeByName(typeName);
+
+            addEntityType(entityType, context);
+        }
+    }
+
+    private void addClassificationType(String typeName, ExportContext context) {
+        if (!context.classificationTypes.contains(typeName)) {
+            AtlasClassificationType classificationType = typeRegistry.getClassificationTypeByName(typeName);
+
+            addClassificationType(classificationType, context);
+        }
+    }
+
+    private void addType(AtlasType type, ExportContext context) {
+        if (type.getTypeCategory() == TypeCategory.PRIMITIVE) {
+            return;
+        }
+
+        if (type instanceof AtlasArrayType) {
+            AtlasArrayType arrayType = (AtlasArrayType)type;
+
+            addType(arrayType.getElementType(), context);
+        } else if (type instanceof AtlasMapType) {
+            AtlasMapType mapType = (AtlasMapType)type;
+
+            addType(mapType.getKeyType(), context);
+            addType(mapType.getValueType(), context);
+        } else if (type instanceof AtlasEntityType) {
+            addEntityType((AtlasEntityType)type, context);
+        } else if (type instanceof AtlasClassificationType) {
+            addClassificationType((AtlasClassificationType)type, context);
+        } else if (type instanceof AtlasStructType) {
+            addStructType((AtlasStructType)type, context);
+        } else if (type instanceof AtlasEnumType) {
+            addEnumType((AtlasEnumType)type, context);
+        } else if (type instanceof AtlasRelationshipType) {
+            addRelationshipType(type.getTypeName(), context);
+        }
+    }
+
+    private void addEntityType(AtlasEntityType entityType, ExportContext context) {
+        if (!context.entityTypes.contains(entityType.getTypeName())) {
+            context.entityTypes.add(entityType.getTypeName());
+
+            addAttributeTypes(entityType, context);
+            addRelationshipTypes(entityType, context);
+
+            if (CollectionUtils.isNotEmpty(entityType.getAllSuperTypes())) {
+                for (String superType : entityType.getAllSuperTypes()) {
+                    addEntityType(superType, context);
+                }
+            }
+        }
+    }
+
+    private void addClassificationType(AtlasClassificationType classificationType, ExportContext context) {
+        if (!context.classificationTypes.contains(classificationType.getTypeName())) {
+            context.classificationTypes.add(classificationType.getTypeName());
+
+            addAttributeTypes(classificationType, context);
+
+            if (CollectionUtils.isNotEmpty(classificationType.getAllSuperTypes())) {
+                for (String superType : classificationType.getAllSuperTypes()) {
+                    addClassificationType(superType, context);
+                }
+            }
+        }
+    }
+
+    private void addStructType(AtlasStructType structType, ExportContext context) {
+        if (!context.structTypes.contains(structType.getTypeName())) {
+            context.structTypes.add(structType.getTypeName());
+
+            addAttributeTypes(structType, context);
+        }
+    }
+
+    private void addEnumType(AtlasEnumType enumType, ExportContext context) {
+        if (!context.enumTypes.contains(enumType.getTypeName())) {
+            context.enumTypes.add(enumType.getTypeName());
+        }
+    }
+
+    private void addRelationshipType(String relationshipTypeName, ExportContext context) {
+        if (!context.relationshipTypes.contains(relationshipTypeName)) {
+            AtlasRelationshipType relationshipType = typeRegistry.getRelationshipTypeByName(relationshipTypeName);
+
+            if (relationshipType != null) {
+                context.relationshipTypes.add(relationshipTypeName);
+
+                addAttributeTypes(relationshipType, context);
+                addEntityType(relationshipType.getEnd1Type(), context);
+                addEntityType(relationshipType.getEnd2Type(), context);
+            }
+        }
+    }
+
+    private void addAttributeTypes(AtlasStructType structType, ExportContext context) {
+        for (AtlasAttributeDef attributeDef : structType.getStructDef().getAttributeDefs()) {
+            addType(attributeDef.getTypeName(), context);
+        }
+    }
+
+    private void addRelationshipTypes(AtlasEntityType entityType, ExportContext context) {
+        for (Map.Entry<String, Map<String, AtlasAttribute>> entry : entityType.getRelationshipAttributes().entrySet()) {
+            for (String relationshipType : entry.getValue().keySet()) {
+                addRelationshipType(relationshipType, context);
+            }
+        }
+    }
+
+    private List<Map<String, Object>> executeGremlinQuery(String query, ExportContext context) {
+        try {
+            return (List<Map<String, Object>>) atlasGraph.executeGremlinScript(context.scriptEngine, context.bindings, query, false);
         } catch (ScriptException e) {
             LOG.error("Script execution failed for query: ", query, e);
             return null;
@@ -608,6 +757,7 @@ public class ExportService {
         final Set<String>                     classificationTypes = new HashSet<>();
         final Set<String>                     structTypes         = new HashSet<>();
         final Set<String>                     enumTypes           = new HashSet<>();
+        final Set<String>                     relationshipTypes   = new HashSet<>();
         final AtlasExportResult               result;
         private final ZipSink                 sink;
 
