@@ -17,8 +17,15 @@
  */
 package org.apache.atlas.repository.store.bootstrap;
 
+
+import com.fasterxml.jackson.annotation.JsonAutoDetect;
+import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
+import com.fasterxml.jackson.databind.annotation.JsonSerialize;
+
 import org.apache.atlas.AtlasErrorCode;
 import org.apache.atlas.AtlasException;
+import org.apache.atlas.RequestContext;
+import org.apache.atlas.authorize.AtlasAuthorizerFactory;
 import org.apache.atlas.exception.AtlasBaseException;
 import org.apache.atlas.ha.HAConfiguration;
 import org.apache.atlas.listener.ActiveStateChangeHandler;
@@ -27,20 +34,23 @@ import org.apache.atlas.model.typedef.AtlasClassificationDef;
 import org.apache.atlas.model.typedef.AtlasEntityDef;
 import org.apache.atlas.model.typedef.AtlasEnumDef;
 import org.apache.atlas.model.typedef.AtlasEnumDef.AtlasEnumElementDef;
+import org.apache.atlas.model.typedef.AtlasRelationshipDef;
+import org.apache.atlas.model.typedef.AtlasRelationshipDef.RelationshipCategory;
+import org.apache.atlas.model.typedef.AtlasRelationshipEndDef;
 import org.apache.atlas.model.typedef.AtlasStructDef;
 import org.apache.atlas.model.typedef.AtlasStructDef.AtlasAttributeDef;
 import org.apache.atlas.model.typedef.AtlasTypesDef;
 import org.apache.atlas.store.AtlasTypeDefStore;
+import org.apache.atlas.type.AtlasEntityType;
+import org.apache.atlas.type.AtlasStructType.AtlasAttribute;
 import org.apache.atlas.type.AtlasType;
 import org.apache.atlas.type.AtlasTypeRegistry;
+import org.apache.atlas.type.AtlasTypeUtil;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.MapUtils;
 import org.apache.commons.configuration.Configuration;
 import org.apache.commons.lang.ObjectUtils;
 import org.apache.commons.lang3.StringUtils;
-import org.codehaus.jackson.annotate.JsonAutoDetect;
-import org.codehaus.jackson.annotate.JsonIgnoreProperties;
-import org.codehaus.jackson.map.annotate.JsonSerialize;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.springframework.stereotype.Service;
@@ -54,13 +64,13 @@ import java.io.File;
 import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
-import static org.codehaus.jackson.annotate.JsonAutoDetect.Visibility.NONE;
-import static org.codehaus.jackson.annotate.JsonAutoDetect.Visibility.PUBLIC_ONLY;
-
+import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
+import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;
 
 /**
  * Class that handles initial loading of models and patches into typedef store
@@ -68,6 +78,10 @@ import static org.codehaus.jackson.annotate.JsonAutoDetect.Visibility.PUBLIC_ONL
 @Service
 public class AtlasTypeDefStoreInitializer implements ActiveStateChangeHandler {
     private static final Logger LOG = LoggerFactory.getLogger(AtlasTypeDefStoreInitializer.class);
+    public static final String PATCHES_FOLDER_NAME    = "patches";
+    public static final String RELATIONSHIP_LABEL     = "relationshipLabel";
+    public static final String RELATIONSHIP_CATEGORY  = "relationshipCategory";
+    public static final String RELATIONSHIP_SWAP_ENDS = "swapEnds";
 
     private final AtlasTypeDefStore atlasTypeDefStore;
     private final AtlasTypeRegistry atlasTypeRegistry;
@@ -87,8 +101,13 @@ public class AtlasTypeDefStoreInitializer implements ActiveStateChangeHandler {
 
         if (!HAConfiguration.isHAEnabled(conf)) {
             atlasTypeDefStore.init();
-
             loadBootstrapTypeDefs();
+
+            try {
+                AtlasAuthorizerFactory.getAtlasAuthorizer();
+            } catch (Throwable t) {
+                LOG.error("AtlasTypeDefStoreInitializer.init(): Unable to obtain AtlasAuthorizer", t);
+            }
         } else {
             LOG.info("AtlasTypeDefStoreInitializer.init(): deferring type loading until instance activation");
         }
@@ -96,57 +115,95 @@ public class AtlasTypeDefStoreInitializer implements ActiveStateChangeHandler {
         LOG.info("<== AtlasTypeDefStoreInitializer.init()");
     }
 
+    /**
+     * This method is looking for folders in alphabetical order in the models directory. It loads each of these folders and their associated patches in order.
+     * It then loads any models in the top level folder and its patches.
+     *
+     * This allows models to be grouped into folders to help managability.
+     *
+     */
     private void loadBootstrapTypeDefs() {
         LOG.info("==> AtlasTypeDefStoreInitializer.loadBootstrapTypeDefs()");
 
-        String atlasHomeDir = System.getProperty("atlas.home");
-        String typesDirName = (StringUtils.isEmpty(atlasHomeDir) ? "." : atlasHomeDir) + File.separator + "models";
-        File   typesDir     = new File(typesDirName);
+        String atlasHomeDir  = System.getProperty("atlas.home");
+        String modelsDirName = (StringUtils.isEmpty(atlasHomeDir) ? "." : atlasHomeDir) + File.separator + "models";
+
+        if (modelsDirName == null || modelsDirName.length() == 0) {
+            LOG.info("Types directory {} does not exist or not readable or has no typedef files", modelsDirName);
+        } else {
+            // look for folders we need to load models from
+            File   topModeltypesDir  = new File(modelsDirName);
+            File[] modelsDirContents = topModeltypesDir.exists() ? topModeltypesDir.listFiles() : null;
+
+            if (modelsDirContents != null && modelsDirContents.length > 0) {
+	            Arrays.sort(modelsDirContents);
+
+	            for (File folder : modelsDirContents) {
+	                    if (folder.isFile()) {
+	                        // ignore files
+	                        continue;
+	                    } else if (!folder.getName().equals(PATCHES_FOLDER_NAME)){
+	                        // load the models alphabetically in the subfolders apart from patches
+	                        loadModelsInFolder(folder);
+	                    }
+	            }
+            }
+
+            // load any files in the top models folder and any associated patches.
+            loadModelsInFolder(topModeltypesDir);
+        }
+        LOG.info("<== AtlasTypeDefStoreInitializer.loadBootstrapTypeDefs()");
+    }
+
+    /**
+     * Load all the model files in the supplied folder followed by the contents of the patches folder.
+     * @param typesDir
+     */
+    private void loadModelsInFolder(File typesDir) {
+        LOG.info("==> AtlasTypeDefStoreInitializer({})", typesDir);
+
+        String typesDirName = typesDir.getName();
         File[] typeDefFiles = typesDir.exists() ? typesDir.listFiles() : null;
 
         if (typeDefFiles == null || typeDefFiles.length == 0) {
-            LOG.info("Types directory {} does not exist or not readable or has no typedef files", typesDirName);
+            LOG.info("Types directory {} does not exist or not readable or has no typedef files", typesDirName );
+        } else {
 
-            return;
-        }
+            // sort the files by filename
+            Arrays.sort(typeDefFiles);
 
-        // sort the files by filename
-        Arrays.sort(typeDefFiles);
+            for (File typeDefFile : typeDefFiles) {
+                if (typeDefFile.isFile()) {
+                    try {
+                        String        jsonStr  = new String(Files.readAllBytes(typeDefFile.toPath()), StandardCharsets.UTF_8);
+                        AtlasTypesDef typesDef = AtlasType.fromJson(jsonStr, AtlasTypesDef.class);
 
-        for (File typeDefFile : typeDefFiles) {
-            if (!typeDefFile.isFile()) {
-                continue;
+                        if (typesDef == null || typesDef.isEmpty()) {
+                            LOG.info("No type in file {}", typeDefFile.getAbsolutePath());
+
+                            continue;
+                        }
+
+                        AtlasTypesDef typesToCreate = getTypesToCreate(typesDef, atlasTypeRegistry);
+                        AtlasTypesDef typesToUpdate = getTypesToUpdate(typesDef, atlasTypeRegistry, true);
+
+                        if (!typesToCreate.isEmpty() || !typesToUpdate.isEmpty()) {
+                            atlasTypeDefStore.createUpdateTypesDef(typesToCreate, typesToUpdate);
+
+                            LOG.info("Created/Updated types defined in file {}", typeDefFile.getAbsolutePath());
+                        } else {
+                            LOG.info("No new type in file {}", typeDefFile.getAbsolutePath());
+                        }
+
+                    } catch (Throwable t) {
+                        LOG.error("error while registering types in file {}", typeDefFile.getAbsolutePath(), t);
+                    }
+                }
             }
 
-            try {
-                String jsonStr = new String(Files.readAllBytes(typeDefFile.toPath()), StandardCharsets.UTF_8);
-                AtlasTypesDef typesDef = AtlasType.fromJson(jsonStr, AtlasTypesDef.class);
-
-                if (typesDef == null || typesDef.isEmpty()) {
-                    LOG.info("No type in file {}", typeDefFile.getAbsolutePath());
-
-                    continue;
-                }
-
-                AtlasTypesDef typesToCreate = getTypesToCreate(typesDef, atlasTypeRegistry);
-                AtlasTypesDef typesToUpdate = getTypesToUpdate(typesDef, atlasTypeRegistry, true);
-
-                if (!typesToCreate.isEmpty() || !typesToUpdate.isEmpty()) {
-                    atlasTypeDefStore.createUpdateTypesDef(typesToCreate, typesToUpdate);
-
-                    LOG.info("Created/Updated types defined in file {}", typeDefFile.getAbsolutePath());
-                } else {
-                    LOG.info("No new type in file {}", typeDefFile.getAbsolutePath());
-                }
-
-            } catch (Throwable t) {
-                LOG.error("error while registering types in file {}", typeDefFile.getAbsolutePath(), t);
-            }
+            applyTypePatches(typesDir.getPath());
         }
-
-        applyTypePatches(typesDirName);
-
-        LOG.info("<== AtlasTypeDefStoreInitializer.loadBootstrapTypeDefs()");
+        LOG.info("<== AtlasTypeDefStoreInitializer({})", typesDir);
     }
 
     public static AtlasTypesDef getTypesToCreate(AtlasTypesDef typesDef, AtlasTypeRegistry typeRegistry) {
@@ -184,6 +241,14 @@ public class AtlasTypeDefStoreInitializer implements ActiveStateChangeHandler {
             }
         }
 
+        if (CollectionUtils.isNotEmpty(typesDef.getRelationshipDefs())) {
+            for (AtlasRelationshipDef relationshipDef : typesDef.getRelationshipDefs()) {
+                if (!typeRegistry.isRegisteredType(relationshipDef.getName())) {
+                    typesToCreate.getRelationshipDefs().add(relationshipDef);
+                }
+            }
+        }
+
         return typesToCreate;
     }
 
@@ -254,6 +319,20 @@ public class AtlasTypeDefStoreInitializer implements ActiveStateChangeHandler {
             }
         }
 
+        if (CollectionUtils.isNotEmpty(typesDef.getRelationshipDefs())) {
+            for (AtlasRelationshipDef relationshipDef : typesDef.getRelationshipDefs()) {
+                AtlasRelationshipDef  oldRelationshipDef = typeRegistry.getRelationshipDefByName(relationshipDef.getName());
+
+                if (oldRelationshipDef == null) {
+                    continue;
+                }
+
+                if (updateTypeAttributes(oldRelationshipDef, relationshipDef, checkTypeVersion)) {
+                    typesToUpdate.getRelationshipDefs().add(relationshipDef);
+                }
+            }
+        }
+
         return typesToUpdate;
     }
 
@@ -265,6 +344,12 @@ public class AtlasTypeDefStoreInitializer implements ActiveStateChangeHandler {
             atlasTypeDefStore.init();
 
             loadBootstrapTypeDefs();
+
+            try {
+                AtlasAuthorizerFactory.getAtlasAuthorizer();
+            } catch (Throwable t) {
+                LOG.error("AtlasTypeDefStoreInitializer.instanceIsActive(): Unable to obtain AtlasAuthorizer", t);
+            }
         } catch (AtlasBaseException e) {
             LOG.error("Failed to init after becoming active", e);
         }
@@ -315,68 +400,69 @@ public class AtlasTypeDefStoreInitializer implements ActiveStateChangeHandler {
     }
 
     private void applyTypePatches(String typesDirName) {
-        String typePatchesDirName = typesDirName + File.separator + "patches";
+        String typePatchesDirName = typesDirName + File.separator + PATCHES_FOLDER_NAME;
         File   typePatchesDir     = new File(typePatchesDirName);
         File[] typePatchFiles     = typePatchesDir.exists() ? typePatchesDir.listFiles() : null;
 
         if (typePatchFiles == null || typePatchFiles.length == 0) {
             LOG.info("Type patches directory {} does not exist or not readable or has no patches", typePatchesDirName);
+        } else {
+            LOG.info("Type patches directory {} is being processed", typePatchesDirName);
 
-            return;
-        }
+            // sort the files by filename
+            Arrays.sort(typePatchFiles);
 
-        // sort the files by filename
-        Arrays.sort(typePatchFiles);
+            PatchHandler[] patchHandlers = new PatchHandler[] {
+                    new AddAttributePatchHandler(atlasTypeDefStore, atlasTypeRegistry),
+                    new UpdateAttributePatchHandler(atlasTypeDefStore, atlasTypeRegistry),
+                    new RemoveLegacyRefAttributesPatchHandler(atlasTypeDefStore, atlasTypeRegistry),
+                    new UpdateTypeDefOptionsPatchHandler(atlasTypeDefStore, atlasTypeRegistry),
+                    new SetServiceTypePatchHandler(atlasTypeDefStore, atlasTypeRegistry)
+            };
 
-        PatchHandler[] patchHandlers = new PatchHandler[] {
-                new AddAttributePatchHandler(atlasTypeDefStore, atlasTypeRegistry),
-                new UpdateTypeDefOptionsPatchHandler(atlasTypeDefStore, atlasTypeRegistry),
-                new UpdateAttributePatchHandler(atlasTypeDefStore, atlasTypeRegistry)
-        };
+            Map<String, PatchHandler> patchHandlerRegistry = new HashMap<>();
 
-        Map<String, PatchHandler> patchHandlerRegistry = new HashMap<>();
-
-        for (PatchHandler patchHandler : patchHandlers) {
-            for (String supportedAction : patchHandler.getSupportedActions()) {
-                patchHandlerRegistry.put(supportedAction, patchHandler);
-            }
-        }
-
-        for (File typePatchFile : typePatchFiles) {
-            if (!typePatchFile.isFile()) {
-                continue;
-            }
-
-            LOG.info("Applying patches in file {}", typePatchFile.getAbsolutePath());
-
-            try {
-                String         jsonStr = new String(Files.readAllBytes(typePatchFile.toPath()), StandardCharsets.UTF_8);
-                TypeDefPatches patches = AtlasType.fromJson(jsonStr, TypeDefPatches.class);
-
-                if (patches == null || CollectionUtils.isEmpty(patches.getPatches())) {
-                    LOG.info("No patches in file {}", typePatchFile.getAbsolutePath());
-
-                    continue;
+            for (PatchHandler patchHandler : patchHandlers) {
+                for (String supportedAction : patchHandler.getSupportedActions()) {
+                    patchHandlerRegistry.put(supportedAction, patchHandler);
                 }
+            }
 
-                for (TypeDefPatch patch : patches.getPatches()) {
-                    PatchHandler patchHandler = patchHandlerRegistry.get(patch.getAction());
+            for (File typePatchFile : typePatchFiles) {
+                if (typePatchFile.isFile()) {
 
-                    if (patchHandler == null) {
-                        LOG.error("Unknown patch action {} in file {}. Ignored",
-                                  patch.getAction(), typePatchFile.getAbsolutePath());
-
-                        continue;
-                    }
+                    LOG.info("Applying patches in file {}", typePatchFile.getAbsolutePath());
 
                     try {
-                        patchHandler.applyPatch(patch);
-                    } catch (AtlasBaseException excp) {
-                        LOG.error("Failed to apply {} patch in file {}. Ignored", patch.getAction(), typePatchFile.getAbsolutePath(), excp);
+                        String jsonStr = new String(Files.readAllBytes(typePatchFile.toPath()), StandardCharsets.UTF_8);
+                        TypeDefPatches patches = AtlasType.fromJson(jsonStr, TypeDefPatches.class);
+
+                        if (patches == null || CollectionUtils.isEmpty(patches.getPatches())) {
+                            LOG.info("No patches in file {}", typePatchFile.getAbsolutePath());
+
+                            continue;
+                        }
+
+                        for (TypeDefPatch patch : patches.getPatches()) {
+                            PatchHandler patchHandler = patchHandlerRegistry.get(patch.getAction());
+
+                            if (patchHandler == null) {
+                                LOG.error("Unknown patch action {} in file {}. Ignored",
+                                        patch.getAction(), typePatchFile.getAbsolutePath());
+
+                                continue;
+                            }
+
+                            try {
+                                patchHandler.applyPatch(patch);
+                            } catch (AtlasBaseException excp) {
+                                LOG.error("Failed to apply {} patch in file {}. Ignored", patch.getAction(), typePatchFile.getAbsolutePath(), excp);
+                            }
+                        }
+                    } catch (Throwable t) {
+                        LOG.error("Failed to apply patches in file {}. Ignored", typePatchFile.getAbsolutePath(), t);
                     }
                 }
-            } catch (Throwable t) {
-                LOG.error("Failed to apply patches in file {}. Ignored", typePatchFile.getAbsolutePath(), t);
             }
         }
     }
@@ -397,6 +483,7 @@ public class AtlasTypeDefStoreInitializer implements ActiveStateChangeHandler {
         private Map<String, Object>     params;
         private List<AtlasAttributeDef> attributeDefs;
         private Map<String, String>     typeDefOptions;
+        private String                  serviceType;
 
         public String getAction() {
             return action;
@@ -453,6 +540,14 @@ public class AtlasTypeDefStoreInitializer implements ActiveStateChangeHandler {
         public void setTypeDefOptions(Map<String, String> typeDefOptions) {
             this.typeDefOptions = typeDefOptions;
         }
+
+        public String getServiceType() {
+            return serviceType;
+        }
+
+        public void setServiceType(String serviceType) {
+            this.serviceType = serviceType;
+        }
     }
 
     /**
@@ -616,6 +711,116 @@ public class AtlasTypeDefStoreInitializer implements ActiveStateChangeHandler {
         }
     }
 
+    class RemoveLegacyRefAttributesPatchHandler extends PatchHandler {
+        public RemoveLegacyRefAttributesPatchHandler(AtlasTypeDefStore typeDefStore, AtlasTypeRegistry typeRegistry) {
+            super(typeDefStore, typeRegistry, new String[] { "REMOVE_LEGACY_REF_ATTRIBUTES" });
+        }
+
+        @Override
+        public void applyPatch(TypeDefPatch patch) throws AtlasBaseException {
+            String           typeName = patch.getTypeName();
+            AtlasBaseTypeDef typeDef  = typeRegistry.getTypeDefByName(typeName);
+
+            if (typeDef == null) {
+                throw new AtlasBaseException(AtlasErrorCode.PATCH_FOR_UNKNOWN_TYPE, patch.getAction(), typeName);
+            }
+
+            if (isPatchApplicable(patch, typeDef)) {
+                if (typeDef.getClass().equals(AtlasRelationshipDef.class)) {
+                    AtlasRelationshipDef    relationshipDef = (AtlasRelationshipDef) typeDef;
+                    AtlasRelationshipEndDef end1Def         = relationshipDef.getEndDef1();
+                    AtlasRelationshipEndDef end2Def         = relationshipDef.getEndDef2();
+                    AtlasEntityType         end1Type        = typeRegistry.getEntityTypeByName(end1Def.getType());
+                    AtlasEntityType         end2Type        = typeRegistry.getEntityTypeByName(end2Def.getType());
+
+                    String               newRelationshipLabel    = null;
+                    RelationshipCategory newRelationshipCategory = null;
+                    boolean              swapEnds                = false;
+
+                    if (patch.getParams() != null) {
+                        Object relLabel    = patch.getParams().get(RELATIONSHIP_LABEL);
+                        Object relCategory = patch.getParams().get(RELATIONSHIP_CATEGORY);
+                        Object relSwapEnds = patch.getParams().get(RELATIONSHIP_SWAP_ENDS);
+
+                        if (relLabel != null) {
+                            newRelationshipLabel = relLabel.toString();
+                        }
+
+                        if (relCategory != null) {
+                            newRelationshipCategory = RelationshipCategory.valueOf(relCategory.toString());
+                        }
+
+                        if (relSwapEnds != null) {
+                            swapEnds = Boolean.valueOf(relSwapEnds.toString());
+                        }
+                    }
+
+                    if (StringUtils.isEmpty(newRelationshipLabel)) {
+                        if (end1Def.getIsLegacyAttribute()) {
+                            if (!end2Def.getIsLegacyAttribute()) {
+                                AtlasAttribute legacyAttribute = end1Type.getAttribute(end1Def.getName());
+
+                                newRelationshipLabel = "__" + legacyAttribute.getQualifiedName();
+                            } else { // if both ends are legacy attributes, RELATIONSHIP_LABEL should be specified in the patch
+                                throw new AtlasBaseException(AtlasErrorCode.PATCH_MISSING_RELATIONSHIP_LABEL, patch.getAction(), typeName);
+                            }
+                        } else if (end2Def.getIsLegacyAttribute()) {
+                            AtlasAttribute legacyAttribute = end2Type.getAttribute(end2Def.getName());
+
+                            newRelationshipLabel = "__" + legacyAttribute.getQualifiedName();
+                        } else {
+                            newRelationshipLabel = relationshipDef.getRelationshipLabel();
+                        }
+                    }
+
+                    AtlasRelationshipDef updatedDef = new AtlasRelationshipDef(relationshipDef);
+
+                    if (swapEnds) {
+                        AtlasRelationshipEndDef tmp = updatedDef.getEndDef1();
+
+                        updatedDef.setEndDef1(updatedDef.getEndDef2());
+                        updatedDef.setEndDef2(tmp);
+                    }
+
+                    end1Def  = updatedDef.getEndDef1();
+                    end2Def  = updatedDef.getEndDef2();
+                    end1Type = typeRegistry.getEntityTypeByName(end1Def.getType());
+                    end2Type = typeRegistry.getEntityTypeByName(end2Def.getType());
+
+                    updatedDef.setRelationshipLabel(newRelationshipLabel);
+
+                    if (newRelationshipCategory != null) {
+                        updatedDef.setRelationshipCategory(newRelationshipCategory);
+                    }
+
+                    updatedDef.setTypeVersion(patch.getUpdateToVersion());
+
+                    AtlasEntityDef updatedEntityDef1 = new AtlasEntityDef(end1Type.getEntityDef());
+                    AtlasEntityDef updatedEntityDef2 = new AtlasEntityDef(end2Type.getEntityDef());
+
+                    updatedEntityDef1.removeAttribute(end1Def.getName());
+                    updatedEntityDef2.removeAttribute(end2Def.getName());
+
+                    AtlasTypesDef typesDef = new AtlasTypesDef();
+
+                    typesDef.setEntityDefs(Arrays.asList(updatedEntityDef1, updatedEntityDef2));
+                    typesDef.setRelationshipDefs(Collections.singletonList(updatedDef));
+
+                    try {
+                        RequestContext.get().setInTypePatching(true); // to allow removal of attributes
+
+                        typeDefStore.updateTypesDef(typesDef);
+                    } finally {
+                        RequestContext.get().setInTypePatching(false);
+                    }
+                }
+            } else {
+                LOG.info("patch skipped: typeName={}; applyToVersion={}; updateToVersion={}",
+                         patch.getTypeName(), patch.getApplyToVersion(), patch.getUpdateToVersion());
+            }
+        }
+    }
+
     class UpdateTypeDefOptionsPatchHandler extends PatchHandler {
         public UpdateTypeDefOptionsPatchHandler(AtlasTypeDefStore typeDefStore, AtlasTypeRegistry typeRegistry) {
             super(typeDefStore, typeRegistry, new String[] { "UPDATE_TYPEDEF_OPTIONS" });
@@ -635,58 +840,44 @@ public class AtlasTypeDefStoreInitializer implements ActiveStateChangeHandler {
             }
 
             if (isPatchApplicable(patch, typeDef)) {
-                if (typeDef.getClass().equals(AtlasEntityDef.class)) {
-                    AtlasEntityDef updatedDef = new AtlasEntityDef((AtlasEntityDef)typeDef);
-
-                    if (updatedDef.getOptions() == null) {
-                        updatedDef.setOptions(patch.getTypeDefOptions());
-                    } else {
-                        updatedDef.getOptions().putAll(patch.getTypeDefOptions());
-                    }
-                    updatedDef.setTypeVersion(patch.getUpdateToVersion());
-
-                    typeDefStore.updateEntityDefByName(typeName, updatedDef);
-                } else if (typeDef.getClass().equals(AtlasClassificationDef.class)) {
-                    AtlasClassificationDef updatedDef = new AtlasClassificationDef((AtlasClassificationDef)typeDef);
-
-                    if (updatedDef.getOptions() == null) {
-                        updatedDef.setOptions(patch.getTypeDefOptions());
-                    } else {
-                        updatedDef.getOptions().putAll(patch.getTypeDefOptions());
-                    }
-                    updatedDef.setTypeVersion(patch.getUpdateToVersion());
-
-                    typeDefStore.updateClassificationDefByName(typeName, updatedDef);
-                } else if (typeDef.getClass().equals(AtlasStructDef.class)) {
-                    AtlasStructDef updatedDef = new AtlasStructDef((AtlasStructDef)typeDef);
-
-                    if (updatedDef.getOptions() == null) {
-                        updatedDef.setOptions(patch.getTypeDefOptions());
-                    } else {
-                        updatedDef.getOptions().putAll(patch.getTypeDefOptions());
-                    }
-                    updatedDef.setTypeVersion(patch.getUpdateToVersion());
-
-                    typeDefStore.updateStructDefByName(typeName, updatedDef);
-                } else if (typeDef.getClass().equals(AtlasEnumDef.class)) {
-                    AtlasEnumDef updatedDef = new AtlasEnumDef((AtlasEnumDef)typeDef);
-
-                    if (updatedDef.getOptions() == null) {
-                        updatedDef.setOptions(patch.getTypeDefOptions());
-                    } else {
-                        updatedDef.getOptions().putAll(patch.getTypeDefOptions());
-                    }
-                    updatedDef.setTypeVersion(patch.getUpdateToVersion());
-
-                    typeDefStore.updateEnumDefByName(typeName, updatedDef);
+                if (typeDef.getOptions() == null) {
+                    typeDef.setOptions(patch.getTypeDefOptions());
                 } else {
-                    throw new AtlasBaseException(AtlasErrorCode.PATCH_NOT_APPLICABLE_FOR_TYPE,
-                                                 patch.getAction(), typeDef.getClass().getSimpleName());
+                    typeDef.getOptions().putAll(patch.getTypeDefOptions());
                 }
+                typeDef.setTypeVersion(patch.getUpdateToVersion());
+
+                typeDefStore.updateTypesDef(AtlasTypeUtil.getTypesDef(typeDef));
             } else {
                 LOG.info("patch skipped: typeName={}; applyToVersion={}; updateToVersion={}",
                          patch.getTypeName(), patch.getApplyToVersion(), patch.getUpdateToVersion());
             }
         }
     }
+
+    class SetServiceTypePatchHandler extends PatchHandler {
+        public SetServiceTypePatchHandler(AtlasTypeDefStore typeDefStore, AtlasTypeRegistry typeRegistry) {
+            super(typeDefStore, typeRegistry, new String[] { "SET_SERVICE_TYPE" });
+        }
+
+        @Override
+        public void applyPatch(TypeDefPatch patch) throws AtlasBaseException {
+            String           typeName = patch.getTypeName();
+            AtlasBaseTypeDef typeDef  = typeRegistry.getTypeDefByName(typeName);
+
+            if (typeDef == null) {
+                throw new AtlasBaseException(AtlasErrorCode.PATCH_FOR_UNKNOWN_TYPE, patch.getAction(), typeName);
+            }
+
+            if (isPatchApplicable(patch, typeDef)) {
+                typeDef.setServiceType(patch.getServiceType());
+                typeDef.setTypeVersion(patch.getUpdateToVersion());
+
+                typeDefStore.updateTypesDef(AtlasTypeUtil.getTypesDef(typeDef));
+            } else {
+                LOG.info("patch skipped: typeName={}; applyToVersion={}; updateToVersion={}",
+                        patch.getTypeName(), patch.getApplyToVersion(), patch.getUpdateToVersion());
+            }
+        }
+    }
 }
