@@ -19,7 +19,6 @@
 package org.apache.atlas.web.filters;
 
 import org.apache.atlas.ApplicationProperties;
-import org.apache.atlas.RequestContext;
 import org.apache.atlas.security.SecurityProperties;
 import org.apache.atlas.utils.AuthenticationUtil;
 import org.apache.atlas.web.security.AtlasAuthenticationProvider;
@@ -735,11 +734,6 @@ public class AtlasAuthenticationFilter extends AuthenticationFilter {
 
                     NDC.push(requestUser + ":" + httpRequest.getMethod() + httpRequest.getRequestURI());
 
-                    RequestContext requestContext = RequestContext.get();
-                    if (requestContext != null) {
-                        requestContext.setUser(requestUser);
-                    }
-
                     LOG.info("Request from authenticated user: {}, URL={}", requestUser, Servlets.getRequestURI(httpRequest));
 
                     filterChain.doFilter(servletRequest, servletResponse);
