@@ -19,21 +19,23 @@
 package org.apache.atlas.web.resources;
 
 import com.sun.jersey.multipart.FormDataParam;
-
 import org.apache.atlas.ApplicationProperties;
 import org.apache.atlas.AtlasClient;
 import org.apache.atlas.AtlasErrorCode;
-import org.apache.atlas.authorize.AtlasActionTypes;
-import org.apache.atlas.authorize.AtlasResourceTypes;
-import org.apache.atlas.authorize.simple.AtlasAuthorizationUtils;
+import org.apache.atlas.authorize.AtlasAdminAccessRequest;
+import org.apache.atlas.authorize.AtlasEntityAccessRequest;
+import org.apache.atlas.authorize.AtlasPrivilege;
+import org.apache.atlas.authorize.AtlasAuthorizationUtils;
 import org.apache.atlas.discovery.SearchContext;
 import org.apache.atlas.exception.AtlasBaseException;
+import org.apache.atlas.model.discovery.AtlasSearchResult;
 import org.apache.atlas.model.impexp.AtlasServer;
 import org.apache.atlas.model.impexp.AtlasExportRequest;
 import org.apache.atlas.model.impexp.AtlasExportResult;
 import org.apache.atlas.model.impexp.AtlasImportRequest;
 import org.apache.atlas.model.impexp.AtlasImportResult;
 import org.apache.atlas.model.impexp.ExportImportAuditEntry;
+import org.apache.atlas.model.impexp.MigrationStatus;
 import org.apache.atlas.model.instance.AtlasCheckStateRequest;
 import org.apache.atlas.model.instance.AtlasCheckStateResult;
 import org.apache.atlas.model.metrics.AtlasMetrics;
@@ -41,13 +43,16 @@ import org.apache.atlas.repository.impexp.AtlasServerService;
 import org.apache.atlas.repository.impexp.ExportImportAuditService;
 import org.apache.atlas.repository.impexp.ExportService;
 import org.apache.atlas.repository.impexp.ImportService;
+import org.apache.atlas.repository.impexp.MigrationProgressService;
 import org.apache.atlas.repository.impexp.ZipSink;
 import org.apache.atlas.repository.impexp.ZipSource;
 import org.apache.atlas.repository.store.graph.AtlasEntityStore;
 import org.apache.atlas.services.MetricsService;
 import org.apache.atlas.type.AtlasType;
+import org.apache.atlas.type.AtlasTypeRegistry;
 import org.apache.atlas.util.SearchTracker;
 import org.apache.atlas.utils.AtlasPerfTracer;
+import org.apache.atlas.utils.AtlasJson;
 import org.apache.atlas.web.filters.AtlasCSRFPreventionFilter;
 import org.apache.atlas.web.service.ServiceState;
 import org.apache.atlas.web.util.Servlets;
@@ -55,8 +60,6 @@ import org.apache.commons.configuration.Configuration;
 import org.apache.commons.configuration.ConfigurationException;
 import org.apache.commons.configuration.PropertiesConfiguration;
 import org.apache.commons.lang.StringUtils;
-import org.codehaus.jettison.json.JSONException;
-import org.codehaus.jettison.json.JSONObject;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.springframework.security.core.Authentication;
@@ -83,10 +86,14 @@ import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.Response;
 import java.io.IOException;
 import java.io.InputStream;
+import java.util.Arrays;
 import java.util.Collection;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
+import java.util.TimeZone;
 import java.util.concurrent.locks.ReentrantLock;
 
 
@@ -100,34 +107,36 @@ public class AdminResource {
     private static final Logger LOG = LoggerFactory.getLogger(AdminResource.class);
     private static final Logger PERF_LOG = AtlasPerfTracer.getPerfLogger("AdminResource");
 
+    private static final String isCSRF_ENABLED                 = "atlas.rest-csrf.enabled";
+    private static final String BROWSER_USER_AGENT_PARAM       = "atlas.rest-csrf.browser-useragents-regex";
+    private static final String CUSTOM_METHODS_TO_IGNORE_PARAM = "atlas.rest-csrf.methods-to-ignore";
+    private static final String CUSTOM_HEADER_PARAM            = "atlas.rest-csrf.custom-header";
+    private static final String isEntityUpdateAllowed          = "atlas.entity.update.allowed";
+    private static final String isEntityCreateAllowed          = "atlas.entity.create.allowed";
+    private static final String editableEntityTypes            = "atlas.ui.editable.entity.types";
+    private static final String DEFAULT_EDITABLE_ENTITY_TYPES  = "hdfs_path";
+    private static final List TIMEZONE_LIST  = Arrays.asList(TimeZone.getAvailableIDs());
+
     @Context
     private HttpServletRequest httpServletRequest;
 
     @Context
     private HttpServletResponse httpServletResponse;
 
-    private final ReentrantLock importExportOperationLock;
-
-    private static final String isCSRF_ENABLED = "atlas.rest-csrf.enabled";
-    private static final String BROWSER_USER_AGENT_PARAM = "atlas.rest-csrf.browser-useragents-regex";
-    private static final String CUSTOM_METHODS_TO_IGNORE_PARAM = "atlas.rest-csrf.methods-to-ignore";
-    private static final String CUSTOM_HEADER_PARAM = "atlas.rest-csrf.custom-header";
-    private static final String isTaxonomyEnabled = "atlas.feature.taxonomy.enable";
-    private static final String isEntityUpdateAllowed = "atlas.entity.update.allowed";
-    private static final String isEntityCreateAllowed = "atlas.entity.create.allowed";
-    private static final String editableEntityTypes = "atlas.ui.editable.entity.types";
-    private static final String DEFAULT_EDITABLE_ENTITY_TYPES = "hdfs_path,hbase_table,hbase_column,hbase_column_family,kafka_topic";
     private Response version;
 
-    private final ServiceState             serviceState;
-    private final MetricsService           metricsService;
-    private static Configuration           atlasProperties;
-    private final ExportService            exportService;
-    private final ImportService            importService;
-    private final SearchTracker            activeSearches;
-    private final AtlasServerService       atlasServerService;
-    private final ExportImportAuditService exportImportAuditService;
-    private final AtlasEntityStore         entityStore;
+    private static Configuration            atlasProperties;
+    private final  ServiceState             serviceState;
+    private final  MetricsService           metricsService;
+    private final  ExportService            exportService;
+    private final  ImportService            importService;
+    private final  SearchTracker            activeSearches;
+    private final  AtlasTypeRegistry        typeRegistry;
+    private final  MigrationProgressService migrationProgressService;
+    private final  ReentrantLock            importExportOperationLock;
+    private final  ExportImportAuditService exportImportAuditService;
+    private final  AtlasServerService       atlasServerService;
+    private final  AtlasEntityStore         entityStore;
 
     static {
         try {
@@ -138,17 +147,21 @@ public class AdminResource {
     }
 
     @Inject
-    public AdminResource(ServiceState serviceState, MetricsService metricsService, ExportService exportService,
-                         ImportService importService, SearchTracker activeSearches, AtlasServerService serverService,
+    public AdminResource(ServiceState serviceState, MetricsService metricsService, AtlasTypeRegistry typeRegistry,
+                         ExportService exportService, ImportService importService, SearchTracker activeSearches,
+                         MigrationProgressService migrationProgressService,
+                         AtlasServerService serverService,
                          ExportImportAuditService exportImportAuditService, AtlasEntityStore entityStore) {
         this.serviceState              = serviceState;
         this.metricsService            = metricsService;
         this.exportService             = exportService;
         this.importService             = importService;
         this.activeSearches            = activeSearches;
+        this.typeRegistry              = typeRegistry;
+        this.migrationProgressService  = migrationProgressService;
         this.atlasServerService        = serverService;
-        this.exportImportAuditService  = exportImportAuditService;
         this.entityStore               = entityStore;
+        this.exportImportAuditService  = exportImportAuditService;
         this.importExportOperationLock = new ReentrantLock();
     }
 
@@ -205,7 +218,7 @@ public class AdminResource {
             try {
                 PropertiesConfiguration configProperties = new PropertiesConfiguration("atlas-buildinfo.properties");
 
-                JSONObject response = new JSONObject();
+                Map<String, Object> response = new HashMap<String, Object>();
                 response.put("Version", configProperties.getString("build.version", "UNKNOWN"));
                 response.put("Revision",configProperties.getString("vc.revision", "UNKNOWN"));
                 response.put("Name", configProperties.getString("project.name", "apache-atlas"));
@@ -214,8 +227,8 @@ public class AdminResource {
 
                 // todo: add hadoop version?
                 // response.put("Hadoop", VersionInfo.getVersion() + "-r" + VersionInfo.getRevision());
-                version = Response.ok(response).build();
-            } catch (JSONException | ConfigurationException e) {
+                version = Response.ok(AtlasJson.toV1Json(response)).build();
+            } catch (ConfigurationException e) {
                 throw new WebApplicationException(Servlets.getErrorResponse(e, Response.Status.INTERNAL_SERVER_ERROR));
             }
         }
@@ -235,16 +248,19 @@ public class AdminResource {
             LOG.debug("==> AdminResource.getStatus()");
         }
 
-        Response response;
+        Map<String, Object> responseData = new HashMap() {{
+                put(AtlasClient.STATUS, serviceState.getState().toString());
+            }};
 
-        try {
-            JSONObject responseData = new JSONObject();
-            responseData.put(AtlasClient.STATUS, serviceState.getState().toString());
-            response = Response.ok(responseData).build();
-        } catch (JSONException e) {
-            throw new WebApplicationException(Servlets.getErrorResponse(e, Response.Status.INTERNAL_SERVER_ERROR));
+        if(serviceState.isInstanceInMigration()) {
+            MigrationStatus status = migrationProgressService.getStatus();
+            if (status != null) {
+                responseData.put("MigrationStatus", status);
+            }
         }
 
+        Response response = Response.ok(AtlasJson.toV1Json(responseData)).build();
+
         if (LOG.isDebugEnabled()) {
             LOG.debug("<== AdminResource.getStatus()");
         }
@@ -261,48 +277,38 @@ public class AdminResource {
         }
 
         Response response;
-        Boolean enableTaxonomy = false;
-        try {
-            if(atlasProperties != null) {
-                enableTaxonomy = atlasProperties.getBoolean(isTaxonomyEnabled, false);
+
+        boolean isEntityUpdateAccessAllowed = false;
+        boolean isEntityCreateAccessAllowed = false;
+        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
+        String userName = null;
+        Set<String> groups = new HashSet<>();
+        if (auth != null) {
+            userName = auth.getName();
+            Collection<? extends GrantedAuthority> authorities = auth.getAuthorities();
+            for (GrantedAuthority c : authorities) {
+                groups.add(c.getAuthority());
             }
 
-            boolean isEntityUpdateAccessAllowed = false;
-            boolean isEntityCreateAccessAllowed = false;
-            Authentication auth = SecurityContextHolder.getContext().getAuthentication();
-            String userName = null;
-            Set<String> groups = new HashSet<>();
-            if (auth != null) {
-                userName = auth.getName();
-                Collection<? extends GrantedAuthority> authorities = auth.getAuthorities();
-                for (GrantedAuthority c : authorities) {
-                    groups.add(c.getAuthority());
-                }
-
-                isEntityUpdateAccessAllowed = AtlasAuthorizationUtils.isAccessAllowed(AtlasResourceTypes.ENTITY,
-                        AtlasActionTypes.UPDATE, userName, groups, httpServletRequest);
-                isEntityCreateAccessAllowed = AtlasAuthorizationUtils.isAccessAllowed(AtlasResourceTypes.ENTITY,
-                        AtlasActionTypes.CREATE, userName, groups, httpServletRequest);
-            }
-
-            JSONObject responseData = new JSONObject();
-
-            responseData.put(isCSRF_ENABLED, AtlasCSRFPreventionFilter.isCSRF_ENABLED);
-            responseData.put(BROWSER_USER_AGENT_PARAM, AtlasCSRFPreventionFilter.BROWSER_USER_AGENTS_DEFAULT);
-            responseData.put(CUSTOM_METHODS_TO_IGNORE_PARAM, AtlasCSRFPreventionFilter.METHODS_TO_IGNORE_DEFAULT);
-            responseData.put(CUSTOM_HEADER_PARAM, AtlasCSRFPreventionFilter.HEADER_DEFAULT);
-            responseData.put(isTaxonomyEnabled, enableTaxonomy);
-            responseData.put(isEntityUpdateAllowed, isEntityUpdateAccessAllowed);
-            responseData.put(isEntityCreateAllowed, isEntityCreateAccessAllowed);
-            responseData.put(editableEntityTypes, getEditableEntityTypes(atlasProperties));
-            responseData.put("userName", userName);
-            responseData.put("groups", groups);
-
-            response = Response.ok(responseData).build();
-        } catch (JSONException e) {
-            throw new WebApplicationException(Servlets.getErrorResponse(e, Response.Status.INTERNAL_SERVER_ERROR));
+            isEntityUpdateAccessAllowed = AtlasAuthorizationUtils.isAccessAllowed(new AtlasEntityAccessRequest(typeRegistry, AtlasPrivilege.ENTITY_UPDATE));
+            isEntityCreateAccessAllowed = AtlasAuthorizationUtils.isAccessAllowed(new AtlasEntityAccessRequest(typeRegistry, AtlasPrivilege.ENTITY_CREATE));
         }
 
+        Map<String, Object> responseData = new HashMap<>();
+
+        responseData.put(isCSRF_ENABLED, AtlasCSRFPreventionFilter.isCSRF_ENABLED);
+        responseData.put(BROWSER_USER_AGENT_PARAM, AtlasCSRFPreventionFilter.BROWSER_USER_AGENTS_DEFAULT);
+        responseData.put(CUSTOM_METHODS_TO_IGNORE_PARAM, AtlasCSRFPreventionFilter.METHODS_TO_IGNORE_DEFAULT);
+        responseData.put(CUSTOM_HEADER_PARAM, AtlasCSRFPreventionFilter.HEADER_DEFAULT);
+        responseData.put(isEntityUpdateAllowed, isEntityUpdateAccessAllowed);
+        responseData.put(isEntityCreateAllowed, isEntityCreateAccessAllowed);
+        responseData.put(editableEntityTypes, getEditableEntityTypes(atlasProperties));
+        responseData.put("userName", userName);
+        responseData.put("groups", groups);
+        responseData.put("timezones", TIMEZONE_LIST);
+
+        response = Response.ok(AtlasJson.toV1Json(responseData)).build();
+
         if (LOG.isDebugEnabled()) {
             LOG.debug("<== AdminResource.getUserProfile()");
         }
@@ -313,12 +319,12 @@ public class AdminResource {
     @GET
     @Path("metrics")
     @Produces(Servlets.JSON_MEDIA_TYPE)
-    public AtlasMetrics getMetrics(@QueryParam("ignoreCache") boolean ignoreCache) {
+    public AtlasMetrics getMetrics() {
         if (LOG.isDebugEnabled()) {
             LOG.debug("==> AdminResource.getMetrics()");
         }
 
-        AtlasMetrics metrics = metricsService.getMetrics(ignoreCache);
+        AtlasMetrics metrics = metricsService.getMetrics();
 
         if (LOG.isDebugEnabled()) {
             LOG.debug("<== AdminResource.getMetrics()");
@@ -339,6 +345,8 @@ public class AdminResource {
             LOG.debug("==> AdminResource.export()");
         }
 
+        AtlasAuthorizationUtils.verifyAccess(new AtlasAdminAccessRequest(AtlasPrivilege.ADMIN_EXPORT), "export");
+
         acquireExportImportLock("export");
 
         ZipSink exportSink = null;
@@ -385,6 +393,8 @@ public class AdminResource {
             LOG.debug("==> AdminResource.importData(jsonData={}, inputStream={})", jsonData, (inputStream != null));
         }
 
+        AtlasAuthorizationUtils.verifyAccess(new AtlasAdminAccessRequest(AtlasPrivilege.ADMIN_IMPORT), "importData");
+
         acquireExportImportLock("import");
         AtlasImportResult result = null;
 
@@ -398,12 +408,12 @@ public class AdminResource {
         } catch (AtlasBaseException excp) {
             if (excp.getAtlasErrorCode().equals(AtlasErrorCode.IMPORT_ATTEMPTING_EMPTY_ZIP)) {
                 LOG.info(excp.getMessage());
-                return new AtlasImportResult();
             } else {
                 LOG.error("importData(binary) failed", excp);
-                throw excp;
             }
 
+            throw excp;
+
         } catch (Exception excp) {
             LOG.error("importData(binary) failed", excp);
 
@@ -427,6 +437,8 @@ public class AdminResource {
             LOG.debug("==> AdminResource.importFile()");
         }
 
+        AtlasAuthorizationUtils.verifyAccess(new AtlasAdminAccessRequest(AtlasPrivilege.ADMIN_IMPORT), "importFile");
+
         acquireExportImportLock("importFile");
 
         AtlasImportResult result;
